
      include 'pi.inc'
      include 'version.inc' 
      include 'bdate.inc' 

      character fin*40, fout*40, type(6)*4, fdat(4)*2, stat*6
      character ans*1, line*80, froot*40
      real rz(100), iz(100), ez(100), period(100), coh(100)
      complex z

      character*24 date_string

      data type/ 'RXX', 'RXY', 'RYX', 'RYY', 'TZX', 'TZY' /
      data fdat/ '11', '12', '21', '22' /

c-------------------------------------------------------------------------------
c     get date

      call fdate(date_string)
c-------------------------------------------------------------------------------


      factor = 4. * PI * 1.e-4

      froot = 'sno106as103'
      call tin( 'Give file root name', froot )
      call trunc( froot, ls )
      stat = froot(1:6)
      fout = froot(:ls)//'c.dat'
      open(unit=7, file=fout )
      write(*,*)'Opened output file >',fout(1:lnblnk(fout))

      deltat = 0.2
      call rin( 'Give delta-t (in Hz)', deltat )
      deltat = 1./deltat

c...write start of info block
      write(7,'(a)') '# c2j: ver='//ver
      write(7,'(a)') '# date :'//date_string
      write(7,'(a)') '#'
      write(7,'(2a)')'# Input file root name: ', froot(1:lnblnk(froot))
      write(7,'(a)') '#'

c...see if a .blk file exists. If so, open it and read then write
      fin=froot(1:lnblnk(froot))//'.blk'
      open(unit=4,file=fin,status='old',err=10)
      do while(.true.)
        read(4,'(a)',end=20) line
        write(7,'(a)') line(1:lnblnk(line))
        if( index(line, '>AZIMUTH').ne.0 ) then
          read(line(13:),*) azimuth
        endif
      enddo

10    write(*,'(a)') 'No <stn>.blk file found >', fin(1:lnblnk(fin))
      write(7,'(a,a)') '>STATION   =      ', stat
c...get azimuth
      azimuth = -999.
      call rin( 'Give GEOGRAPHIC azimuth of data', azimuth )
      write(7,'(a,f)') '>AZIMUTH   =', azimuth

20    write(7,'(a6,1x,f10.1)' ) stat, azimuth
      ans = 'm'
      call tin( 'MT/GDS (m) or GDS-only (g)', ans )
      call shiftdown( ans )
      if( ans.eq.'m' ) then
        its = 1
      else if( ans.eq.'g' ) then
        its = 5
      else
        write(*,*)'Erroneous answer: m or g only'
        goto 20
      endif


      n = 100

      do itype = its, 6
        if( itype.eq.1 ) then
          fin = froot(:ls)//'.1r1.rf'
          call tin( 'Give zxx data filename', fin)
          call trunc( fin, lf )
        else  if( itype.eq.2 ) then
          fin = fin(1:lf-6)//'1r2.rf'
          call tin( 'Give zxy data filename', fin)
        else  if( itype.eq.3 ) then
          fin = fin(1:lf-6)//'2r1.rf'
          call tin( 'Give zyx data filename', fin)
        else  if( itype.eq.4 ) then
          fin = fin(1:lf-6)//'2r2.rf'
          call tin( 'Give zyy data filename', fin)
        else  if( itype.eq.5 ) then
          fin = fin(1:lf-6)//'3r1.rf'
          call tin( 'Give tzx data filename', fin)
        else  if( itype.eq.6 ) then
          fin = fin(1:lf-6)//'3r2.rf'
          call tin( 'Give tzy data filename', fin)
       end if

       open( unit=4, file=fin, status='OLD', err=9998 )

        do i = 1, n
          read(4,*,end=4) freq, rz(i), iz(i), ez(i), degf, coh(i)
          period(i) = deltat / freq
        end do
        close(unit=4) 

    4   n = i-1

c...write out impedances
        if( itype.le.4 ) then
          write(7,'(a)' ) 'Z'//type(itype)(2:3)
          write(7,*) n
          do i = n, 1, -1
            if( period(i).gt.1. ) then
              per = period(i)
            else
              per = -1. / period(i)
            endif
            write(7,*) per, factor * rz(i), factor * iz(i),
     &                    factor * ez(i), coh(i)
          enddo

c...write out rho & pha
          write(7,'(a)' ) 'R'//type(itype)(2:3)
          write(7,*) n
          do i = n, 1, -1
            z = factor * cmplx(rz(i),iz(i))
            zerr = factor * ez(i)
            rho   = convz2r( z, period(i) )
            phase = convz2p( z, period(i) )
            zr = zerr*cosd(phase)
            zi = zerr*sind(phase)
            rhomax = convz2r( z+cmplx(zr,zi), period(i) )
            rhomin = convz2r( z-cmplx(zr,zi), period(i) )
            phaerr = atand( zerr/cabs(z) )
            phamax = phase + phaerr
            phamin = phase - phaerr
            weight = coh(i)
            if( period(i).gt.1. ) then
              per = period(i)
            else
              per = -1. / period(i)
            endif
            write(7,2002) per, rho, phase, rhomax, rhomin, 
     &                    phamax, phamin, weight, weight
          enddo

c...write out transfer functions
        else
          write(7,'(a)' ) type(itype)
          write(7,*) n
          do i = n, 1, -1
            if( period(i).gt.1. ) then
              per = period(i)
            else
              per = -1. / period(i)
            endif
            write(7,*) per, rz(i), iz(i), ez(i), coh(i)
          enddo
        endif


      enddo

      stop

 9998 write(*,*)'Error opening file = ', fin 
      stop
      
 2002 format( g11.5,1x,g11.5,1x,f6.1,1x,2(g11.5,1x),
     &                    2(f6.1,1x),f5.2,1x,f5.2 )

      end
