/*
 * Copyright 1989 O'Reilly and Associates, Inc.
 * See ../Copyright for complete rights and liability information.
 */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

#include <stdio.h>

#define PIXELS_PER_INCH (100)
#define MM_PER_INCH (25.4)
#define XFLUSH 0
#define MAX_COLOURS 16

/* These are used as arguments to nearly every Xlib routine, so it saves 
 * routine arguments to declare them global.  If there were 
 * additional source files, they would be declared extern there. */
Display *display;
int screen_num;
Window win;
GC gc;
unsigned int win_width, win_height;  /*  window size  */
int last_x=0;
int last_y=0;
static int can_use_colours;
static unsigned long colour_value [MAX_COLOURS];
static char *colour_name[] = {"black", "white", "red", "green",
			"blue", "cyan", "magenta", "yellow",
			"orange", "yellow green", "turquoise", "blue violet",
			"violet", "purple", "lavender", "aquamarine"};

void xbuinit_ ();
void xbuplot_ ();
void xbuline_ ();
void xbulines_ ();
void xbusymbol_ ();
void xbuerase_ ();
void xbuflush_ ();
void xbuclose_ ();
unsigned int mm_to_pixels ();
float pwr ();
int colour_available ();
unsigned long get_colour ();
void xbucolour_ ();
void xburect_ ();


static char *progname; /* name this program was invoked by */

void xbuinit_(x_mm_size, y_mm_size)
float *x_mm_size, *y_mm_size;
{
	unsigned int width, height;	/* window size */
	int x, y; 	/* window position */
	unsigned int border_width = 4;	/* four pixels */
	unsigned int display_width, display_height;
	unsigned int icon_width, icon_height;
	char *window_name = "Xbuplot-new";
	char *icon_name = "xbunew";

	XSizeHints size_hints;
	XIconSize *size_list;
	int count;
	XEvent report;
	GC gc;
	XFontStruct *font_info;
	char *display_name = NULL;
	XSetWindowAttributes set_window_attributes;


	/* connect to X server */
	if ( (display=XOpenDisplay(display_name)) == NULL )
	{
		(void) fprintf( stderr, "%s: cannot connect to X server %s\n", 
				progname, XDisplayName(display_name));
		exit( -1 );
	}

	/* get screen size from display structure macro */
	screen_num = DefaultScreen(display);
	display_width = DisplayWidth(display, screen_num);
	display_height = DisplayHeight(display, screen_num);
	win_height = display_height;

	/* Note that in a real application, x and y would default to 0
	 * but would be settable from the command line or resource database.  
	 */
	x = y = 0;

	/* size window with enough room for text */
/*	width = display_width/3, height = display_height/4; */
	width = mm_to_pixels (*x_mm_size);
	height = mm_to_pixels (*y_mm_size);
	

	/* create opaque window */
	win = XCreateSimpleWindow(display, RootWindow(display,screen_num), 
			x, y, width, height, border_width, BlackPixel(display,
			screen_num), WhitePixel(display,screen_num));


	/* Set size hints for window manager.  The window manager may
	 * override these settings.  Note that in a real
	 * application if size or position were set by the user
	 * the flags would be UPosition and USize, and these would
	 * override the window manager's preferences for this window. */
	/* x, y, width, and height hints are now taken from
	 * the actual settings of the window when mapped. Note
	 * that PPosition and PSize must be specified anyway. */

	size_hints.flags = PPosition | USSize | PMaxSize | PMinSize;
	size_hints.min_width = size_hints.max_width = win_width =width;
	size_hints.min_height = size_hints.max_height = win_height=height;


	{
	XWMHints wm_hints;
	XClassHint class_hints;

	/* format of the window name and icon name 
	 * arguments has changed in R4 */
	XTextProperty windowName, iconName;

	/* These calls store window_name and icon_name into
	 * XTextProperty structures and set their other 
	 * fields properly. */
	if (XStringListToTextProperty(&window_name, 1, &windowName) == 0) {
		(void) fprintf( stderr, "%s: structure allocation for windowName failed.\n", 
				progname);
		exit(-1);
	}
		
	if (XStringListToTextProperty(&icon_name, 1, &iconName) == 0) {
		(void) fprintf( stderr, "%s: structure allocation for iconName failed.\n", 
				progname);
		exit(-1);
	}

	wm_hints.initial_state = NormalState;
	wm_hints.input = True;
	wm_hints.flags = StateHint | InputHint;

	class_hints.res_name = "xbuplot";
	class_hints.res_class = "Xbuplot";

	XSetWMProperties(display, win, &windowName, &iconName, 
			NULL, 0, &size_hints, &wm_hints, 
			&class_hints);
	}

	/*  Set backing store to on since we can't service Expose events  */
/*	set_window_attributes.backing_store = Always;
	XChangeWindowAttributes (display,win, CWBackingStore, &set_window_attributes);*/
	/* Select event types wanted */
	XSelectInput(display, win, ExposureMask | KeyPressMask | 
			ButtonPressMask | StructureNotifyMask);

	load_font(&font_info);

	/* create GC for text and drawing */
	getGC(win, &gc, font_info);

	/*  check to see if colours are available  */
	/*  if not set foreground colour to white  */
	if (!(can_use_colours = colour_available ()))
	    XSetForeground (display, gc, WhitePixel (display, screen_num));

	/* Display window */
	XMapWindow(display, win);

	/* get events, use first to display text and graphics */
	while (1)  {
		XNextEvent(display, &report);
		(void) fprintf(stdout,"xevent typecode: %d should be %d\n", report.type, Expose);
		
		switch  (report.type) {
		case Expose:
			/* unless this is the last contiguous expose,
			 * don't draw the window */
			if (report.xexpose.count != 0) 
				break;
			(void) fprintf(stdout,"leaving xbuinit\n");
			return;

		case ConfigureNotify:
			/* window has been resized, change width and
			 * height to send to draw_text and draw_graphics
			 * in next Expose */
			width = report.xconfigure.width;
			height = report.xconfigure.height;
			break;
		case ButtonPress:
			/* trickle down into KeyPress (no break) */
		case KeyPress:
			XUnloadFont(display, font_info->fid);
			XFreeGC(display, gc);
			XCloseDisplay(display);
			exit(1);
		default:
			/* all events selected by StructureNotifyMask
			 * except ConfigureNotify are thrown away here,
			 * since nothing is done with them */
			break;
		} /* end switch */
	} /* end while */
}

getGC(win, gc, font_info)
Window win;
GC *gc;
XFontStruct *font_info;
{
	unsigned long valuemask = 0; /* ignore XGCvalues and use defaults */
	XGCValues values;
	unsigned int line_width = 6;
	int line_style = LineOnOffDash;
	int cap_style = CapRound;
	int join_style = JoinRound;
	int dash_offset = 0;
	static char dash_list[] = {12, 24};
	int list_length = 2;

	/* Create default Graphics Context */
	*gc = XCreateGC(display, win, valuemask, &values);

	/* specify font */
	XSetFont(display, *gc, font_info->fid);

	/* specify black foreground since default window background is 
	 * white and default foreground is undefined. */
	XSetForeground(display, *gc, BlackPixel(display,screen_num));

	/* set line attributes */
	XSetLineAttributes(display, *gc, line_width, line_style, 
			cap_style, join_style);

	/* set dashes */
	XSetDashes(display, *gc, dash_offset, dash_list, list_length);
}

load_font(font_info)
XFontStruct **font_info;
{
	char *fontname = "9x15";

	/* Load font and get font information structure. */
	if ((*font_info = XLoadQueryFont(display,fontname)) == NULL)
	{
		(void) fprintf( stderr, "%s: Cannot open 9x15 font\n", 
				progname);
		exit( -1 );
	}
}

draw_text(win, gc, font_info, win_width, win_height)
Window win;
GC gc;
XFontStruct *font_info;
unsigned int win_width, win_height;
{
	char *string1 = "Hi! I'm a window, who are you?";
	char *string2 = "To terminate program; Press any key";
	char *string3 = "or button while in this window.";
	char *string4 = "Screen Dimensions:";
	int len1, len2, len3, len4;
	int width1, width2, width3;
	char cd_height[50], cd_width[50], cd_depth[50];
	int font_height;
	int initial_y_offset, x_offset;


	/* need length for both XTextWidth and XDrawString */
	len1 = strlen(string1);
	len2 = strlen(string2);
	len3 = strlen(string3);

	/* get string widths for centering */
	width1 = XTextWidth(font_info, string1, len1);
	width2 = XTextWidth(font_info, string2, len2);
	width3 = XTextWidth(font_info, string3, len3);

	font_height = font_info->ascent + font_info->descent;

	/* output text, centered on each line */
	XDrawString(display, win, gc, (win_width - width1)/2, 
			font_height,
			string1, len1);
	XDrawString(display, win, gc, (win_width - width2)/2, 
			(int)(win_height - (2 * font_height)),
			string2, len2);
	XDrawString(display, win, gc, (win_width - width3)/2, 
			(int)(win_height - font_height),
			string3, len3);

	/* copy numbers into string variables */
	(void) sprintf(cd_height, " Height - %d pixels", 
			DisplayHeight(display,screen_num));
	(void) sprintf(cd_width, " Width  - %d pixels", 
			DisplayWidth(display,screen_num));
	(void) sprintf(cd_depth, " Depth  - %d plane(s)", 
			DefaultDepth(display, screen_num));

	/* reuse these for same purpose */
	len4 = strlen(string4);
	len1 = strlen(cd_height);
	len2 = strlen(cd_width);
	len3 = strlen(cd_depth);

	/* To center strings vertically, we place the first string
	 * so that the top of it is two font_heights above the center
	 * of the window.  Since the baseline of the string is what we
	 * need to locate for XDrawString, and the baseline is one
	 * font_info->ascent below the top of the character,
	 * the final offset of the origin up from the center of the 
	 * window is one font_height + one descent. */

	initial_y_offset = win_height/2 - font_height - font_info->descent;
	x_offset = (int) win_width/4;
	XDrawString(display, win, gc, x_offset, (int) initial_y_offset, 
			string4,len4);

	XDrawString(display, win, gc, x_offset, (int) initial_y_offset + 
			font_height,cd_height,len1);
	XDrawString(display, win, gc, x_offset, (int) initial_y_offset + 
			2 * font_height,cd_width,len2);
	XDrawString(display, win, gc, x_offset, (int) initial_y_offset + 
			3 * font_height,cd_depth,len3);
}

draw_graphics(win, gc, window_width, window_height)
Window win;
GC gc;
unsigned int window_width, window_height;
{
	int x, y;
	int width, height;

	height = window_height/2;
	width = 3 * window_width/4;
	x = window_width/2 - width/2;  /* center */
	y = window_height/2 - height/2;
	XDrawRectangle(display, win, gc, x, y, width, height);
}

/********  xbuplot_  ********/
void xbuplot_ (x_mm, y_mm, pen_down)
  float *x_mm, *y_mm;  /*  position in mm of where to move pen  */
  int *pen_down;  /*  pen = 1 --> pen down; pen = 0 --> pen up  */
{
  int x, y;  /*  position in pixels of where to move pen  */

  x = mm_to_pixels (*x_mm);
  y = win_height - mm_to_pixels (*y_mm);
  if (*pen_down)
  {
    XDrawLine (display, win, gc, last_x, last_y, x, y);
#if XFLUSH
    XFlush (display);
#endif
  }
  last_x = x;
  last_y = y;
}
  

/********  xbuline_  ********/
void xbuline_ (x1_mm, y1_mm, x2_mm, y2_mm)
  float *x1_mm, *y1_mm;  /*  first point in line  */
  float *x2_mm, *y2_mm;  /*  second point in line  */
{
  int x1, y1, x2, y2;  /*  x y coords of line in pixels  */

  x1 = mm_to_pixels (*x1_mm);
  y1 = win_height - mm_to_pixels (*y1_mm);
  x2 = mm_to_pixels (*x2_mm);
  y2 = win_height - mm_to_pixels (*y2_mm);
  XDrawLine (display, win, gc, x1, y1, x2, y2);
#if XFLUSH
  XFlush (display);
#endif
  last_x = x2;
  last_y = y2;
}
  

/********  xbulines_  ********/
void xbulines_ (x_mm, y_mm, num_points)
  float *x_mm, *y_mm;  /*  points in line  */
  int *num_points;  /*  number of points in line  */
{
  int x, y;  /*  x y coords of line in pixels  */
  int point = 0;

  last_x = mm_to_pixels (x_mm [0]);
  last_y = win_height - mm_to_pixels (y_mm [0]);
  for (point = 1; point <= *num_points; point++)
  {
    x = mm_to_pixels (x_mm [point - 1]);
    y = win_height - mm_to_pixels (y_mm [point - 1]);
    XDrawLine (display, win, gc, last_x, last_y, x, y);
    last_x = x;
    last_y = y;
  }
#if XFLUSH
  XFlush (display);
#endif
}

  
/********  xbusymbol_  ********/
void xbusymbol_ (x_mm, y_mm, height_mm, string, angle, num_chars)
  float *x_mm, *y_mm;  /*  point to draw string at  */
  float *height_mm;  /*  height of text (not yet implemented)  */
  char *string;  /*  string to be drawn  */
  float *angle;  /*  angle in degrees of text. only 0 and 90 supported  */
  int *num_chars;  /*  number of characters in string  */
{
  int x, y;  /*  x and y coords of the bottom left corner of text  */
  int i;
  int height;
  char character [2];

  x = mm_to_pixels (*x_mm);
  y = win_height - mm_to_pixels (*y_mm);
  height = mm_to_pixels (*height_mm);
  character [1] = 0;

  if ((int) (*angle + 0.5) == 0) 
  {
    XDrawString (display, win, gc, x, y, string, *num_chars);
  } 
  else if ((int) (*angle + 0.5) == 90)
  {
    for (i = 1; i <= *num_chars; i++)
    {
      character [0] = string [i - 1];
      XDrawString (display, win, gc, x, 
			(unsigned int) (y - ((*num_chars - i) * 1.2 * height)), 
			character, 1);
    }
  }

#if XFLUSH
  XFlush (display);
#endif
}


/********  xbunumber_  ********/
void xbunumber_ (x_mm, y_mm, height_mm, number, angle, num_dec_points)
  float *x_mm, *y_mm;  /*  point to draw string at  */
  float *height_mm;  /*  height of text (not yet implemented)  */
  float *number;  /*  number to be plotted  */
  float *angle;   /*  angle in degrees of text, only 0 and 90 supported  */
  int *num_dec_points;  /*  number of digits to follow decimal point  */
{
  int x, y;  /*  x and y coords of the bottom left corner of text  */
  char string[80];
  int num_chars;
  float rounded_number;
  int ndp;
  int i;
  int height;
  char character [2];

  x = mm_to_pixels (*x_mm);
  y = win_height - mm_to_pixels (*y_mm);
  height = mm_to_pixels (*height_mm);
  character [1] = 0;
  ndp = *num_dec_points;
  rounded_number = (float) ((long) (*number * pwr (10.0, ndp) + 0.5) 
                        / pwr (10.0,  ndp));
  sprintf (string, "%g", rounded_number);
  num_chars = strlen (string);

  if ((int) (*angle + 0.5) == 0) 
  {
    XDrawString (display, win, gc, x, y, string, num_chars);
  } 
  else if ((int) (*angle + 0.5) == 90)
  {
    for (i = 1; i <= num_chars; i++)
    {
      character [0] = string [i - 1];
      XDrawString (display, win, gc, x, 
                        (unsigned int) (y - ((num_chars - i) * 1.2 * height)), 
                        character, 1);
    }
  }

#if XFLUSH
  XFlush (display);
#endif
}


/********  xbuerase_  ********/
void xbuerase_ ()
{

  XClearArea (display, win, 0, 0, 0, 0, False);
#if XFLUSH
  XFlush (display);
#endif
}


/********  xbuflush_  ********/
void xbuflush_ ()
{

  XFlush (display);
}


/********  xbuclose_  ********/
void xbuclose_ ()
{
/*        if(font_info) XUnloadFont (display, font_info->fid); */
        XFreeGC (display, gc);
        XCloseDisplay (display);
}


/********  mm_to_pixels  ********/
unsigned int mm_to_pixels (mm)
  float mm;
{
  return ((unsigned int) (mm * PIXELS_PER_INCH / MM_PER_INCH + 0.5));
}


/********  pwr  ********/
float pwr (number, exponent)
  float number;
  int exponent;
{
  int i;
  float value;

  value = 1.0;
  
  for (i = 1; i <= exponent; i++)
  {
    value *= number;
  }
  return (value);
}


/********  colour_available  ********/
/*  find out in the display supports colour. 
 *  i.e. PseudoColor, TrueColor, DirectColor, and StaticColor
 *  return 0 is colour_available == false
 *  return 1 is colour_available == true
 */
int colour_available ()
{
  int default_depth;
  int class = 5;
  XVisualInfo visual_info;

  default_depth = DefaultDepth (display, screen_num);
  if (default_depth == 1) 
  {
    /*  Must be static grey. use black and white  */
    /*  return value of 0 for "colour_available == false"  */
    return (0);
  }

  /*  check to see if a visual class that supports colour is available  */
  while (!XMatchVisualInfo (display, screen_num, default_depth, class--, 
				&visual_info))
    ;
  class++;  /*  set class back to which class was found  */
  if (class < 2)  /*  colour visual classes are 2 to 5  */
  {
    /*  No color visual available at default_depth. use black and white  */
    /*  return value of 0 for "colour_available == false"  */
    return (0);
  }
  /*  color visual is available. colours are available  */
  /*  return value of 1 for "colour_available == true"  */
  return (1);
}


/********  get_colour  ********/
/*  returns pixel value for given colour  */
/*  allocates colour if necessary  */
unsigned long get_colour (colour)
  int colour;
{
  Colormap default_colour_map;
  XColor exact_def;

  /*  check to make sure colour is in range, if not return white  */
  if (colour > MAX_COLOURS - 1)
    return (WhitePixel (display, screen_num));

  /*  check to see if colour has alread been allocated. if so return value  */
  if (colour_value [colour])  /*  colour_value is non-zero if allocated  */
    return (colour_value [colour]);

  /*  get default colour map  */
  default_colour_map   = DefaultColormap (display, screen_num);

  /*  parse colour name to see if colour is available. if not use white  */
  if (!XParseColor (display, default_colour_map, colour_name [colour], 
			&exact_def)) 
    return (WhitePixel (display, screen_num));

  /*  allocate colour. if not available use white  */
  if (!XAllocColor (display, default_colour_map, &exact_def))
    return (WhitePixel (display, screen_num));

  /*  we got the colour so store it and return it's pixel value  */
  colour_value [colour] = exact_def.pixel;
  return (exact_def.pixel);
}


/********  xbucolour  ********/
/*  set the foreground colour in the gc to the value of colour  */
void xbucolour_ (colour)
  int *colour;
{

  if (can_use_colours) 
    XSetForeground (display, gc, get_colour (*colour));
}


/********  xburect  ********/
/*  draw a rectangle and, possibly, fill it  */
void xburect_ (x1_mm, y1_mm, x2_mm, y2_mm, fill)
  float *x1_mm, *y1_mm;  /*  bottom left corner in mm  */
  float *x2_mm, *y2_mm;  /*  top right corner in mm  */
  int *fill;  /*  fill flag: 0 == no fill, 1 == fill  */
{
  unsigned int x1, y1;  /*  bottom left corner in pixels  */
  unsigned int x2, y2;  /*  top right right corner in pixels  */

  x1 = mm_to_pixels (*x1_mm);
  y1 = win_height - mm_to_pixels (*y1_mm);
  x2 = mm_to_pixels (*x2_mm);
  y2 = win_height - mm_to_pixels (*y2_mm);

  if (*fill)
  {
    XDrawRectangle (display, win, gc, x1, y1, x2 - x1, y2 - y1);
    XFillRectangle (display, win, gc, x1 + 1, y1 + 1, x2 - x1 - 1, y2 - y1 - 1);
  }
  else
  {
    XDrawRectangle (display, win, gc, x1, y1, x2 - x1, y2 - y1);
  }
}
