      subroutine bessel7
c
c  Program to compute the complex transfer function response of a
c     6-pole Bessel filter.
c     Ron Kurtz September 4, 1988.
c     Modified from program provided by Philip Munro, August 3, 1988.
c
      include 'maxflims.inc'
      integer ipre(7), ntrace, n, i, j, k
      real    delta, tlen, period, cuts(7), pi, w, ww1, ww2, ww3
      real    q1, q2, q3, cut1, cut2, w1, w2, w3, fac, denom, freq, csum

      COMPLEX f(MAXF), Z(4), RESP1, RESP2
      COMPLEX S7
      COMMON /TRACES7/ IPRE,NTRACE
      COMMON /POWER7/ S7(7,7,MAXF)
      COMMON /TIME/ DELTA,TLEN,PERIOD(MAXF)
      COMMON /CUTSL7/ cuts
      COMMON N

      PI=4.*ATAN(1.)

c...check that filter needed: return if all cuts set to 0.0
      csum = 0.
      do i = 1, 7
        csum = csum + cuts(i)
      enddo
      if( csum.eq.0. ) return

c  Define the complex frequency.
      do 100  I=1,N
        FREQ=1./PERIOD(I)
	W=2*PI*FREQ
  100   f(i) = CMPLX(0.0,W)
c
c  NOTE: a) for a Bessel filtre the -3dB point is at 0.57 x corner 
c           frequency
c        b) the corner frequency has been chosen as 1/4 the sample rate by 
c           design.
c        c) the amplitude is down by 10 dB at the corner frequency.
c 
C     TRANSFER FUNCTION FOR 6-POLE BESSEL FILTER AT 15 HZ
C     W(0)=94.2478
c     To change from 15 Hz to 2 Hz divide WW1, WW2 and WW3 ONLY by 7.5
C     THE 6-POLE FILTER IS REPRESENTED AS THREE TWO POLE FILTERS OF
C     EQUIVALENT EFFECT.
C     FILTER PARAMETERS ARE TAKEN FROM TABLE 6.2 PRESENTED IN THE BOOK
C     'FUNCTION CIRCUITS; DESIGN AND APPLICATION'; BURR-BROWN ELECTRONIC
C     SERIES; WONG,Y.J. AND W.E.OTT; McGRAW-HILL; TK7867.W66.
C     The parameters are:



      WW1 = 102.9846
      WW2 = 91.32704
      WW3 = 86.72125
      Q1 = 1.023310
      Q2 = 0.611195
      Q3 = 0.510318
C
      DO 200 I=1,7
        cut1 = cuts(1)
        DO 200 J=1,7
          cut2 = cuts(2)

          DO 200 K=1,N

            FAC = 15.0 * CUT1
            W1 = WW1/FAC
            W2 = WW2/FAC
            W3 = WW3/FAC
            Z(1)=(W1*W1) * (W2*W2) * (W3*W3)
            Z(2)=1.0/(f(k)**2 + f(k)*W1/Q1 + W1**2)
            Z(3)=1.0/(f(k)**2 + f(k)*W2/Q2 + W2**2)
            Z(4)=1.0/(f(k)**2 + f(k)*W3/Q3 + W3**2)
            RESP1 = Z(1)*Z(2)*Z(3)*Z(4)

            FAC = 15.0 * CUT2
            W1 = WW1/FAC
            W2 = WW2/FAC
            W3 = WW3/FAC
            Z(1)=(W1*W1) * (W2*W2) * (W3*W3)
            Z(2)=1.0/(f(k)**2 + f(k)*W1/Q1 + W1**2)
            Z(3)=1.0/(f(k)**2 + f(k)*W2/Q2 + W2**2)
            Z(4)=1.0/(f(k)**2 + f(k)*W3/Q3 + W3**2)
            RESP2 = Z(1)*Z(2)*Z(3)*Z(4)

            DENOM = (CABS(RESP1)**2)*(CABS(RESP2)**2)
            S7(I,J,K)=S7(I,J,K)*RESP1*CONJG(RESP2)/DENOM
  200 CONTINUE

      return
      END
