c======================================================================
c
c D A T C H K            checks that a given date is valid
c
c     i/p   char*19 time     time in format  yyyy-mm-dd hh:mm:ss
c
c     o/p   datchk = 0 if no error found
c                  = 1 for error discovered
 
      integer function datchk( time )
      character time*19
      integer yy, mm, dd, hh, mn, ss
      external date
 
      call date( 0, time, yy, mm, dd, hh, mn, ss )
 
      if( (yy.lt.0.or.yy.gt.3000) .or. (mm.lt.1.or.mm.gt.12) .or.
     &    (dd.lt.1.or.dd.gt.31) .or. (hh.lt.0.or.hh.gt.23) .or.
     &    (mn.lt.0.or.mn.gt.59) .or. (ss.lt.0.or.ss.gt.59)) then
        write(*,*)' Erroneous time given to DATCHK, time=',time
        datchk = 1
      else
        datchk = 0
      end if
 
      return
      end
 
c======================================================================
      integer function tdiff( time1, time2 )
c
c     T D I F F        derives the time difference in seconds
c                      between two times given in format yymmddhhmmss
c
c    Note that this is an INTEGER*4 function to cope with large
c    time diferences (up to 300 years)
c
      character*19 time1, time2
      integer year1, month1, day1, hour1, min1, sec1
      integer year2, month2, day2, hour2, min2, sec2
      integer year, ddiff, hdiff, mdiff, sdiff, dayyr
      external date
 
      call date( 0, time1, year1, month1, day1, hour1, min1, sec1 )
      call date( 0, time2, year2, month2, day2, hour2, min2, sec2 )
 
      if(year1.eq.year2) then
        ddiff = dayyr(year2,month2,day2 )  - dayyr(year1,month1,day1)
      else
        if( (year2-year1).ne.1 ) then
          write(*,*)'tdiff: ERROR: years differ by more than 1!'
          write(*,*)' year1=',year1
          write(*,*)' year2=',year2
          tdiff=-9999
        endif

        ddiff = dayyr(year2,month2,day2 ) + (year2-year1)*365 -
     &          dayyr(year1,month1,day1)
        if( ((mod(year1,4).eq.0  .and. mod(year1,100).ne.0)
     &     .or. (mod(year1,400).eq.0))
     &    .and. dayyr(year1,month1,day1).le.60) ddiff = ddiff + 1
      endif
      hdiff = hour2 - hour1
      mdiff = min2  - min1
      sdiff = sec2 - sec1
 
      do 1 year = year1+1, year2-1
        if( (mod(year,4).eq.0  .and. mod(year,100).ne.0)
     &                .or. (mod(year,400).eq.0) ) then
          ddiff = ddiff + 366
        else
          ddiff = ddiff + 365
        end if
    1 continue
 
      tdiff = ddiff*24*60*60  +  hdiff*60*60  + mdiff*60 + sdiff
 
      return
      end


c======================================================================
      subroutine duration( time1, time2, dur )
      implicit none

c     DURATION         derives the duration between two times 
c     dddd-hh-mn-ss    given in format yyyy-mm-dd hh:mn:ss

      character*19 time1, time2
      character*13 dur
      integer tsec, tdiff, dd, hh, mn, ss
      external tdiff

c...first convert to seconds

      tsec = tdiff( time1, time2 ) 

c...now determine number of days, hours, mins & secs
      dd = tsec/(24*60*60)
      hh = tsec/(   60*60) - dd*24
      mn = tsec/(      60) - (dd*24 + hh)*60
      ss = tsec  - ((dd*24 + hh)*60 + mn)*60

c...now write to character string
      dur = '0000-00:00:00'
      write( dur( 1: 4),'(i4.4)') dd
      write( dur( 6: 7),'(i2.2)') hh
      write( dur( 9:10),'(i2.2)') mn
      write( dur(12:13),'(i2.2)') ss
 
      return
      end

c======================================================================
      subroutine s2dhms( sec, dd, hh, mm, ss )
      integer sec, dd, hh, mm, ss

      dd =  sec / (24*60*60)
      hh = (sec - dd*24*60*60 ) / (60*60)
      mm = (sec - dd*24*60*60 - hh*60*60 ) / 60
      ss =  sec - dd*24*60*60 - hh*60*60 - mm*60

      return
      end

c======================================================================
c N O W T I M        function to add or subtract a time difference from
c                    a relative time, and return the new "now" time
c
c   I/P       char*19 reftim:    gives reference time as   yyyy-mm-dd hh:mm:ss
c             int     dday       difference days
c             int     dhour      difference hours   (can be >23)
c             int     dmin       difference minutes (can be >59)
c             int     dsec       difference seconds (can be >59)
c
c   O/P       char*19 nowtim:    gives new "now" time as   yyyy-mm-dd hh:mm:ss
c
      character*19 function nowtim( reftim, dday, dhour, dmin, dsec )
      character*19 reftim
      integer dday, dhour, dmin, dsec, dayyr

      include 'ctrlblk.inc'
 
      character*19 time
      integer refyy, refmm, refdd, refhh, refmin, refss
      integer month, day, hour, min, sec
      external date

      if( iprint.ge.10 ) then
        write(*,*)'nowtim: entered with :'
        write(*,*)'reftim =', reftim
        write(*,*)'dday   =', dday
        write(*,*)'dhour  =', dhour
        write(*,*)'dmin   =', dmin
        write(*,*)'dsec   =', dsec
      endif

      call date( 0, reftim, refyy, refmm, refdd, refhh, refmin, refss )
 
      if( iprint.ge.10 ) then
        write(*,*)'nowtim: return from date with :'
        write(*,*)'refyy  =', refyy
        write(*,*)'refmm  =', refmm
        write(*,*)'refdd  =', refdd
        write(*,*)'refhh  =', refhh
        write(*,*)'refmin =', refmin
        write(*,*)'refss  =', refss
      endif

      sec  = 60*60*refhh + 60*refmin + refss + 
     &       60*60*dhour + 60*dmin   + dsec
      if( iprint.ge.50 ) write(*,*)'nowtim: total sec =', sec
      min  = sec/60
      day  = dday + min/1440
      hour = mod( min/60, 24 )
      min  = mod( min, 60)
      sec  = mod( sec, 60)
      if( sec.lt.0 .or. min.lt.0 .or. hour.lt.0 ) then
        day = day-1
        hour = 23+hour
        min = 60+min
        sec = 60+sec
        hour = mod( hour + min/60, 24 )
        min  = mod( min, 60)
        sec  = mod( sec, 60)
      endif

      if( iprint.ge.10 ) then
        write(*,*)'nowtim: calculated times :'
        write(*,*)'day  =', day
        write(*,*)'hour =', hour
        write(*,*)'min  =', min
        write(*,*)'sec  =', sec
      endif

      if( day .ne. 0 ) then
        day = day + dayyr( refyy, refmm, refdd )
c-------Note: following will not work for 1st Jan after a leap year
        refyy = refyy + day/365
        day   = mod( day, 365 )
        call mmdd( refyy, day, month, day )
      else
        month = refmm
        day   = refdd
      end if
      refyy = refyy + (month-1)/12
      month = mod(month-1,12) + 1
 
      call date( 1, time, refyy, month, day, hour, min, sec )
 
      nowtim = time
 
      return
      end
 
c======================================================================
      integer function dayyr( year, month, day )
      integer year, month, day, leap, i
      integer daytab(2,12)
      data daytab/31,31, 28,29, 31,31, 30,30, 31,31, 30,30,
     &              31,31, 31,31, 30,30, 31,31, 30,30, 31,31/
 
      if( (mod(year,4).eq.0  .and. mod(year,100).ne.0)
     &              .or. (mod(year,400).eq.0) ) then
        leap = 2
      else
        leap = 1
      end if
 
      dayyr = day
      do 1 i = 1, month-1
        dayyr = dayyr + daytab(leap,i)
   1  continue
 
      return
      end
 
c======================================================================
      subroutine mmdd( year, yday, month, day )
      integer year, yday, yydd, month, day, leap, i
      integer daytab(2,12)
      data daytab/31,31, 28,29, 31,31, 30,30, 31,31, 30,30,
     &            31,31, 31,31, 30,30, 31,31, 30,30, 31,31/
 
      yydd = yday

3     if( (mod(year,4).eq.0  .and. mod(year,100).ne.0)
     &              .or. (mod(year,400).eq.0) ) then
        leap = 2
      else
        leap = 1
      end if

      if( leap.eq.2 .and. yydd.gt.366 ) then
        year = year + 1
        yydd = yydd - 366
      elseif( leap.eq.1 .and. yydd.gt.365 ) then
        year = year + 1
        yydd = yydd - 365
      endif
 
      do 1 i = 1, 12
        yydd = yydd - daytab(leap,i)
        if( yydd.le.0 ) goto 2
1     continue

2     month = i
      day  = yydd + daytab(leap,i)
      if( day.eq.0 ) then
        month = i-1
        day = daytab(leap,i-1)
      end if
 
      return
      end
 
c======================================================================
      subroutine date( conv, cdate, year, month, day, hour, min, sec )
      character cdate*19
      integer conv, year, month, day, hour, min, sec

      include 'ctrlblk.inc'
c
c     converts from date string format to integer format
c
c     conv = 1:  convert from 6 integers to character*19 string
c          = 0:  convert from character*19 string to 6 integers

      if( iprint.ge.3 ) then
        write(*,*)'date: conv = ', conv
        if( conv.eq.1 ) then
          write(*,*) '     yymmhhddmmss=', year, month, day, hour, 
     &                                     min, sec
        else
          write(*,*) '     cdate=',cdate
        endif
      endif
 
      if( conv.eq.1) then
        cdate = '0000-00-00 00:00:00'
        write( cdate( 1: 4), '(i4.4)' ) year
        write( cdate( 6: 7), '(i2.2)' ) month
        write( cdate( 9:10), '(i2.2)' ) day
        write( cdate(12:13), '(i2.2)' ) hour
        write( cdate(15:16), '(i2.2)' ) min
        write( cdate(18:19), '(i2.2)' ) sec
      else
        read( cdate( 1: 4), '(i4.4)' ) year
        read( cdate( 6: 7), '(i2.2)' ) month
        read( cdate( 9:10), '(i2.2)' ) day
        read( cdate(12:13), '(i2.2)' ) hour
        read( cdate(15:16), '(i2.2)' ) min
        read( cdate(18:19), '(i2.2)' ) sec
      endif
      if( iprint.ge.3 ) write(*,*)'date: conversion successful'
      return
 
 9001 write(*,*)'date: error converting ints to char'
      write(*,*)' year,month,day,hour,min=',year,month,day,hour,min,sec
      cdate='0000-00-00 00:00:00'
      return
 
 9002 write(*,'(2a)')'date: error converting char to ints, cdate=',
     &               cdate
      year  = 0
      month = 0
      day   = 0
      hour  = 0
      min   = 0
      sec   = 0
      return
 
      end
 
      SUBROUTINE chkdat (yy, mm, dd, hh, min, ss, err )
c  Checks for valid dates and times.
c     R.D. Kurtz,  May 13, 1988.
c
c  August 18, 1992 (Angela Tchen)
c  Separated from parent program file lims_cascade.f

      integer yy, mm, dd, hh, min, ss, err
      err = 0
      if( yy.lt.0 .or. yy.gt.3000 ) then
       write (*,'(a,i4.4,a)')' Invalid year:',yy,'  Try again.'
       err = 1
      end if

      if(( mm .lt. 1 ) .or. ( mm .gt. 12 )) then
       write (*,'(a,i3,a)')' Invalid month:',mm,'  Try again.'
       err = 1
      end if

      if(( dd .lt. 1 ) .or. ( dd .gt. 31 )) then
       write (*,'(a,i3,a)')' Invalid day:',dd,'  Try again.'
       err = 1
      end if

      if(( hh .lt. 0 ) .or. ( hh .gt. 24 )) then
       write (*,'(a,i3,a)')' Invalid hour:',hh,'  Try again.'
       err = 1
      end if

      if(( min .lt. 0 ) .or. ( min .gt. 60 )) then
       write (*,'(a,i3,a)')' Invalid minute:',min,'  Try again.'
       err = 1
      end if

      if(( ss .lt. 0 ) .or. ( ss .gt. 60 )) then
       write (*,'(a,i3,a)')' Invalid second:',ss,'  Try again.'
       err = 1
      end if

      return 
      end

      subroutine convtimev5( stime )
      character stime*19, stimenew*19
c
c     converts old v4 format to v5 format
c
      stimenew( 1: 2) = '19'
      stimenew( 3: 4) = stime(1:2)
      stimenew( 5: 5) = '-'
      stimenew( 6: 7) = stime(3:4)
      stimenew( 8: 8) = '-'
      stimenew( 9:10) = stime(5:6)
      stimenew(11:11) = ' '
      stimenew(12:13) = stime(7:8)
      stimenew(14:14) = ':'
      stimenew(15:16) = stime(9:10)
      stimenew(17:17) = ':'
      stimenew(18:19) = stime(11:12)

      stime = stimenew

      return
      end
