      subroutine scmax2s7( MT, CTYPE, NTYPE, N, THRESH, TOL, NREJ, S,
     &                    stot, iflag, WGTS, weight)
 
      complex   s(7,7,*), stot(7,7), stmp(7,7)
      real      weight(*), thresh, tol, tolp, tolm, wgt, fbcc
      real      wgtl, wgtmax, wgtl2, wgtl3
      integer   iflag(*), ntype, n, nrej, i, j, k
      integer   jmaxl, jmaxl2, icycle, imiss, jmax
      logical   MT, WGTS
      character ctype*3, cotype*1

      include 'ctrlblk.inc'

C
C     SUBROUTINE TO DERIVE RESPONSE FUNCTION FROM MAXIMISING COHERENCE
C
C    MT         .TRUE. for MT reponse, .FALSE. for GDS
C    CTYPE      type of function to maximize/minimize COH & VAR valid
C    NTYPE      type of function to maximize (COH) or minimize (VAR)
C    N          no. of spectra available
C               CTYPE = COH NTYPE = 1,   maximize Mcoh(ex,hx,hy)*Mcoh(ey,hx,hy)
C                                   2             Mcoh(ex,hx,hy)
C                                   3             Mcoh(ey,hx,hy)
C                                   4             Pcoh(ex,hy,hx)*Pcoh(ey,hx,hy)
C                                   5             Pcoh(ex,hy,hx)
C                                   6             Pcoh(ey,hx,hy)
C                                   7             Mcoh(rx,hx,hy)*Mcoh(ry,hx,hy)
C                       VAR         1             Sd31*Sd32*Sd41*Sd42
C                                   2             Sd31*Sd32
C                                   3             Sd41*Sd42
C                                   4             Sd32*Sd41
C    THRESH     threshold value when to stop scheme. For THRESH=0 then NREJ poi
C               rejected
C    TOL        permitted tolerance for maximum (COH) or minimum(VAR)
C    NREJ       number to reject, reset to atr least 4
C    S          input cross-spectra array
C    SAV        output average array
C    IFLAG      integer array containing a 1 if cross-spectra used, 0 if not
C
C     INITIALISE
C
C
      TOLP = 1. + TOL
      TOLM = 1. - TOL
 
      if( iprint.ge.2 ) then
        write(*,*) 'Entered SCMAX2S7 with N = ', n
        write(*,*) ' ctype=', ctype,'   ntype=', ntype,
     &             '  thresh=', thresh
      end if
 
      if( (n-nrej).lt.4 ) nrej = n-4
 
      if( ctype.eq.'COH' ) then
        if( ntype.ge.1 .and. ntype.le.3 ) then
          cotype = 'M'
        else if( ntype.ge.4 .and. ntype.le.6 ) then
          cotype = 'P'
        else if( ntype.eq.7 ) then
          cotype = 'M'
        else
          write(*,*)'SCMAX2S7: erroneous NTYPE for CTYPE=COH, ', ntype
          stop
        end if
      else if( ctype.eq.'VAR' ) then
        continue
      else
        write(*,*)'SCMAX2S7: erroneous CTYPE=', ctype
        stop
      end if


      do i = 1, n
       iflag( i ) = 1
      end do

      do i = 1, 7
        do j = 1, 7
          stot(I,J)= cmplx(0.0,0.0)
        end do
      end do

      do k = 1, n
        if( WGTS ) then
          call wgtmt( ctype, ntype, n, s(1,1,k), wgt )
          weight(k) = wgt
          if( iprint.ge.2 ) then
            write(*,*) 'k"th estimate & wgt =>', k, wgt
          end if
        else
          weight(k) = 1.
        end if
        do j = 1, 7
          do i = 1, 7
            stot(I,J) = stot(I,J) + s(i,j,k)*weight(k)
          end do
        end do
      end do

      if( n.le. 4 ) then
        write(*,*)'SCMAX2S7: error, n < 5, n = ', n
        return
      end if

      if( iprint.ge.2 ) then
        write(*,*)'stot ==>'
        do i = 1, 7
          do j = i, 7
            write(*,*) i, j, stot(i,j)
          end do
        end do
      end if

      if( MT ) then
        call wgtmt( ctype, ntype, n, stot, wgt )
      else
        call wgtgds( ctype, ntype, n, stot, wgt )
      end if
      write(*,*) 'Starting returned wgt =>', wgt
      if( ctype.eq.'COH' ) then
        wgt = fbcc( 2*n, wgt, cotype )
      else
        continue
      end if
      wgtl  = wgt
      wgtl2 = wgt/2.
      wgtl3 = wgt/3.
      jmaxl  = 0
      jmaxl2 = 0
C
      if(iprint.ge.2) WRITE( *,2001) CTYPE, NTYPE, TOL, N,
     &                  wgt
C
C---- CYCLE -----------------------------------------------------------
C
      DO ICYCLE = 1, NREJ
        wgtmax = 0.

        DO 4 IMISS=1,N
          IF( IFLAG(IMISS) .EQ. 0) GOTO 4
C
C--------remove IMISS contribution from total spectral estimate
C
          do j = 1, 7
            do i = 1, 7
              stmp(I,J) = stot(I,J) - s(I,J,IMISS)*weight(imiss)
            end do
          end do

          if( MT ) then
            call wgtmt( ctype, ntype, n-icycle, stmp, wgt )
          else
            call wgtgds( ctype, ntype, n-icycle, stmp, wgt )
          end if

          if( iprint.ge.3 ) then
            write(*,*)'Icycle, imiss, wgt = ', icycle, imiss, wgt
          end if

          if( wgt.gt.wgtmax ) then
            wgtmax=wgt
            jmax = imiss
          end if

    4   continue


c---compute corrected wgtmax
        if( ctype.eq.'COH' ) then
          wgtmax = fbcc( 2*(n-icycle), wgtmax, cotype )
        else
          continue
        end if

        wgt = wgtmax

        if( iprint.ge.2 ) write(*,2002) icycle, jmax, wgt


c---see if wgt < wgtl < wgtl2 ; if true, then return after adding jmaxl
c                               estimate back into stack
c
        if( wgt.lt.wgtl .and. wgtl.lt.wgtl2 ) then
          write(*,2004) icycle-2, wgtl2
          iflag(jmaxl) = 1
          do j = 1, 7
            do i = 1, 7
              stot(i,j) = stot(i,j) + s(i,j,jmaxl)*weight(jmaxl)
            end do
          end do
          write(*,'(a,4f8.3)')'last 4 weights =',
     &              wgtl3, wgtl2, wgtl, wgt
          return
        end if

C--------remove JMAX contribution from total spectral estimate
C
        iflag(jmax) = 0
        do j = 1, 7
          do i = 1, 7
            stot(I,J) = stot(I,J) - s(i,j,jmax)*weight(jmax)
          end do
        end do

C---- see if multiple coherence is now above, or sd below,  threshold
C
        IF( ctype.eq.'COH' .and. wgt.gt.thresh )THEN
          if( iprint.ge.2 ) WRITE( *, 2003) ICYCLE, wgt
          RETURN
        END IF

        wgtl3 = wgtl2
        wgtl2 = wgtl
        wgtl  = wgt
        jmaxl2=jmaxl
        jmaxl = jmax

      end do


c---ended loop: check if wgt < wgtl

      write( *,2005)
      if( wgt.lt.wgtl ) then
        write(*,2007) nrej-1, wgtl2
        iflag(jmaxl) = 1
        do j = 1, 7
          do i = 1, 7
            stot(i,j) = stot(i,j) + s(i,j,jmaxl)*weight(jmaxl)
          end do
        end do
      else
        write(*,2008) nrej, wgtl
      end if
 
      return



 2001 FORMAT(/, 1x,'Initial weight in SCMAX2S7 with CTYPE=',A3,
     &       ' NTYPE=', I1,' TOL=',F5.3,'  ==>', /, 2X,
     1       'N=', I4, ' weight=', F9.4)
 2002 FORMAT(1X,'cycle=>',I5,'  jmax=', I5, '  wgtmax=',F9.4)
 2003 FORMAT(1X,'Leaving SCMAX2S7, crossed threshold with',
     &       ' rejected=', I5, ' wgt=',F9.4)
 2004 FORMAT(1X,'SCMAX2S7: maximum found with',
     &       ' rejected=', I5, ' wgtmax=',F9.4)
 2005 FORMAT(1X, 'SCMAX2S7: maximum number of cycles:', $ )
 2006 FORMAT(1X,'Leaving SCMAX2S7, erroneous SDs derived, using ',
     &          'previous iteration')
 2007 format('replacing last spectra: rejected=',i5,'  wgtmax=',f9.4)
 2008 format('rejected=',i5, '  wgtmax=',f9.4 )
      END
