c===============================================================================
      REAL FUNCTION FBCC( NDEG, COH, TYPE)
c-------------------------------------------------------------------------------

c  August 27, 1992 (Angela Tchen)
c  Checked for SUN compatibility--no changes were required.

      REAL ROOTR(3), ROOTI(3), COH, BIAS, DEG, A, B, C, D
      INTEGER NDEVE, NDEVM, NDEG
      CHARACTER*1 TYPE
      COMMON /NWRITE/ NDEVE, NDEVM
C
C   FUNCTION TO CALCULATE THE BIAS-CORRECTED COHERENCE
C   FUNCTION (EITHER ORDINARY, MULTIPLE, OR PARTIAL) FROM
C   THE ESTIMATED COHERENCE FUNCTION AND ITS ASSOCIATED
C   NUMBER OF DEGREES OF FREEDOM
C
C   FOR  TYPE = 'O' THEN THE ORDINARY BIAS IS TAKEN
C               'M'          MULTIPLE
C               'P'          PARTIAL
C
C
      BIAS = 0.0
      DEG = FLOAT(NDEG)

      IF(TYPE .EQ. 'O') THEN
        IF( DEG .GE. 4. ) THEN
          BIAS = 2./DEG
        ELSE
          FBCC = -1.0
          RETURN
        END IF
      ELSE IF(TYPE .EQ. 'M') THEN
        IF( DEG .GE. 8. ) THEN
          BIAS = 4./(DEG - 2.)
        ELSE
          FBCC = -1.0
          RETURN
        END IF
      ELSE IF(TYPE .EQ. 'P') THEN
        IF( DEG .GE. 8. ) THEN
          BIAS = (2.*DEG + 4.)/((DEG - 2.)**2)
        ELSE
          FBCC = -1.0
          RETURN
        END IF
      ELSE
        WRITE( NDEVE,* ) ' FBCC: incorrect type = ', TYPE
        FBCC = -1.0
        RETURN
      END IF
C
C--- solve CUBIC in "C"
C
      A = 4.*BIAS
      B = BIAS*(DEG - 8.)
      C = DEG + 4.*BIAS - 2.*DEG*BIAS
      D = (BIAS - COH)*DEG
C
      CALL CUBIC( A, B, C, D, ROOTR, ROOTI)
C
      FBCC = ROOTR(1)
      IF(FBCC .LT. 0.) FBCC = 0.0
C
      RETURN
C
 2001 FORMAT(/, 5X, '***Incorrect parameter input to BCCF, NDEG=',
     &       I3, '  TYPE=', A, '  BCCF = -1.0 returned')
      END

c===============================================================================
      REAL FUNCTION FMCOH( IS, S, K, I, J )
c-------------------------------------------------------------------------------

c  August 26, 1992 (Angela Tchen)
c  checked for SUN compatibility--no changes required.

      IMPLICIT NONE
      INTEGER I, J, K, IS, NORM
      COMPLEX S(IS,IS)
      COMPLEX*16 SII,SIJ,SIK,SJI,SJJ,SJK,SKI,SKJ,SKK, CNUM,CDEN
      REAL*8 NUM, DEN, scale
C
C     MULTIPLE COHERENCE FUNCTION GKIJ
C     products scaled so that will not run into machine precision problems 
C
C
      if( real(s(i,i)).le.0. .or. real(s(j,j)).le.0. .or.
     &    real(s(k,k)).le.0.      ) then
        fmcoh = 0.0
        return
      end if

      NORM = (LOG10(REAL(S(I,I)))+LOG10(REAL(S(J,J)))
     &              +LOG10(REAL(S(K,K))))/3
      scale = 10.D0**(-NORM)
      SII = S(I,I)*scale
      SIJ = S(I,J)*scale
      SIK = S(I,K)*scale
      SJI = CONJG( SIJ )
      SJJ = S(J,J)*scale
      SJK = S(J,K)*scale
      SKI = CONJG( SIK )
      SKJ = CONJG( SJK )
      SKK = S(K,K)*scale
      CNUM = SII*SJK*SKJ + SJJ*SIK*SKI - SIJ*SJK*SKI - SIK*SKJ*SJI
      CDEN = SKK * (SII*SJJ - SIJ*SJI)
      NUM = REAL(CNUM)
      DEN = REAL(CDEN)
      IF(DEN .GT. 0.D0) THEN
        FMCOH = AMAX1( AMIN1( SNGL(NUM/DEN), 1.0 ) , 0.0 )
      ELSE
        FMCOH = -1.
      END IF
C
      RETURN
      END

c===============================================================================
      real function focoh( is, s, i, j )
c-------------------------------------------------------------------------------

c  August 26, 1992 (Angela Tchen)
c  checked for SUN compatibility--no changes required.


      integer i, j, is, norm
      real denom
      complex s(is,is), sii,sij,sji,sjj
c
      sii = s(i,i)
      sjj = s(j,j)
      if( real(sii).le.0. .or. real(sjj).le.0. ) then
        focoh = 0.0
        return
      end if
      norm = (log10(real(sii))+log10(real(sjj)))/2
      sii = s(i,i)*(10.**(-norm))
      sij = s(i,j)*(10.**(-norm))
      sji = conjg( sij )
      sjj = s(j,j)*(10.**(-norm))
      focoh = real( sij*sji )
      denom = real( sii*sjj )
      if(denom .gt. 0.) then
        focoh = amax1( amin1( focoh/denom, 1.0 ) , 0.0 )
      else
        focoh = -1.
      end if
c
      return
      end

c===============================================================================
      real function frccf1( n, f, ocoh, syy, sxx )
c-------------------------------------------------------------------------------

c  August 19, 1992 (Angela Tchen)
c  Checked for SUN customization--no changes were required.
 
      real f, ocoh, syy, sxx
      integer n
 
      include 'ctrlblk.inc'

      if( iprint.ge.3 ) then
        write(*,*)'frccf1: entered with n=',n
      endif

      if( iprint.ge.4 ) then
        write(*,*)'        f, ocoh  =', f, ocoh
        write(*,*)'        syy, sxx =', syy, sxx
      endif

      if( ocoh.le.0. .or. ocoh.ge.1. ) then
        frccf1 = -1.
      else if( n.le.2 ) then
        frccf1 = -1.
      else
        frccf1 = (2./float(n-2)) * f * (1.-ocoh) * (syy/sxx)
      end if
 
      if( iprint.ge.3 ) then
        write(*,*)'        leaving with frccf1 =', frccf1
      endif

      return
      end
 
c===============================================================================
      real function frccf2( n, f, mcoh, ocoh, syy, sxx )
c-------------------------------------------------------------------------------

c  August 19, 1992 (Angela Tchen)
c  Checked for SUN customization--no changes were required.
 
      real f, mcoh, ocoh, syy, sxx
      integer n 
      include 'ctrlblk.inc'

      if( iprint.ge.3 ) then
        write(*,*)'frccf2: entered'
      endif

      if( mcoh.le.0. .or. mcoh.ge.1. ) then
        frccf2 = -1.
      else if( ocoh.le.0. .or. ocoh.ge.1. ) then
        frccf2 = -1.
      else if( n.le.4 ) then
         frccf2 = -1.
      else
      frccf2 = (4./float(n-4)) * f * ((1.-mcoh)/(1.-ocoh)) * (syy/sxx)
      end if
 
      if( iprint.ge.3 ) then
        write(*,*)'        leaving with frccf2 =', frccf2
      endif

      return
      end
 
c===============================================================================
      SUBROUTINE RCCF2(NDEG,MULT,OCOH,SYY,SII,  R)
c-------------------------------------------------------------------------------
c
      REAL MULT, OCOH, SYY, SII, R, F, fdist1, ALPHA, SYYN, SIIN,
     &     NORM
      INTEGER NDEG
      EXTERNAL FDIST1
      COMMON /CONFL/ ALPHA
      include 'ctrlblk.inc'
C 
C     CALC. 'ALPHA' CIRCLE OF CONFIDENCE FOR TWO I/P - SINGLE O/P
C 
      IF( iprint.ge.3 ) then
        WRITE(*,*) 'RCCF2: ENTERED'
        write(*,*) 'ndeg=',ndeg,' mult=',mult,' ocoh=',ocoh
        write(*,*) 'syy, sii = ', syy, sii
      endif
C 
      IF( NDEG.GT.4 .AND. OCOH.LT.1. .AND. OCOH.GT.0. 
     &              .AND. MULT.LT.1. .AND. MULT.GT.0. ) THEN
        NORM = (LOG10(REAL(SII))+LOG10(REAL(SYY)))/2
        SIIN = SII*(10.**(-NORM))
        SYYN = SYY*(10.**(-NORM))
        F = FDIST1( 4, NDEG-4, ALPHA )
        R=(4.*F*(1.-MULT)*SYYN)/((FLOAT(NDEG-4))*(1.-OCOH)*SIIN)
        if ( iprint.ge.3 ) then
          write(*,*) 'f,   r^2 = ', f, r
        endif
        R=SQRT(R)
      ELSE
        R=-1.
        if ( iprint.ge.3 ) then
          write(*,*) '     r   = ', r
        endif
      END IF

      RETURN
      END

c===============================================================================
      REAL FUNCTION FDIST1(NDEG1,NDEG2,ALPH)
c-------------------------------------------------------------------------------

      integer n_deg1, n_deg2, ndeg1, ndeg2, ier, limit
      real alpha, alph, eps, start1, start2, fdist2, f, value
      EXTERNAL FCT_F
      COMMON /F$DIST/ N_DEG1,N_DEG2,ALPHA
      DATA EPS/1.E-3/,LIMIT/100/
C
C**FIRST GUESS GIVEN BY APPROXIMATION 26.7.5 IN ABROMOVICH & STEGUN

      N_DEG1=NDEG1
      N_DEG2=NDEG2
      ALPHA=ALPH
C
      START1=1.0
      IF(ALPHA.GT.0.33)START1=0.40
      START2=100.0
      IF(NDEG1.GT.1 .AND. NDEG2.GT.1) START2 = FDIST2(NDEG1,NDEG2,ALPH)
C
      CALL RTMI(F,VALUE,FCT_F,START1,5.*START2,EPS,LIMIT,IER)
C
      IF(IER.EQ.0)GOTO 1
      WRITE(6,2999)IER
 2999 FORMAT('FDIST1: ERROR RETURN FROM RTMI, IER=',I3,' FDIST1=0.0')
      write(*,*)'ndeg1=',ndeg1,'   ndeg2=',ndeg2
      write(*,*)'start1=',start1,'  start2=',start2
      FDIST1=0.0
      RETURN

    1 FDIST1=F
      RETURN
      END


c===============================================================================
      REAL FUNCTION FCT_F(F)
c-------------------------------------------------------------------------------

      integer n_deg1, n_deg2, ier
      real    f, alpha, val, q
      COMMON /F$DIST/ N_DEG1,N_DEG2,ALPHA
C
      CALL MDFD(F,N_DEG1,N_DEG2,Q,IER)
C
      IF(IER.EQ.0)GOTO 1
    2 WRITE(6,2999)IER,N_DEG1,N_DEG2,ALPHA
 2999 FORMAT('FCT_F: ERROR RETURN FROM MDFD, IER=',I3, '  NDEG1=',I5,
     &         '  NDEG2=',I3,'  ALPHA=',F8.6)
      VAL=0.0
      RETURN
C
C
    1 FCT_F=(1.-Q) - ALPHA
C
      RETURN
      END


c===============================================================================
      REAL FUNCTION FDIST2(N1,N2,ALPHA)
c-------------------------------------------------------------------------------

      integer n1, n2
      real alpha
      DOUBLE PRECISION LAMBDA,A,B,H,W,y,dqinv
C
C**FUNCTION TO DERIVE THE INVERSE 'F' FUNCTION BY
C  APPROXIMATIONS 26.6.16 WITH 26.5.22 IN
C  ABROMOVICH & STEGUN
C
C**TEST THAT N1 & N2 BOTH > 1
      IF(N1.GT.1 .AND. N2.GT.1)GOTO 1
C
      WRITE(6,2999)N1,N2
 2999 FORMAT(2X,'**FDIST2:  !ERROR! N1=',I2,'  N2=',I2,
     &          '  FDIST2 SET TO 0.0')
      FDIST2=0.0
      RETURN
    1 A=DFLOAT(N2)/2.D0
      B=DFLOAT(N1)/2.D0
C
      Y=DQINV(DBLE(ALPHA))
      LAMBDA=(Y*Y - 3.D0)/6.D0
      H=2.D0/(1.D0/(2.D0*A-1.D0) + 1.D0/(2.D0*B-1.D0))
      W=Y*((H+Y)**0.5D0)/H - (1.D0/(2.D0*B-1.D0) - 1.D0/(2.D0*A-1.D0))*
     &                        (LAMBDA + 5.D0/6.D0 - 2.D0/(3.D0*H))
      FDIST2=SNGL(DEXP(2.D0*W))
      RETURN
      END


c===============================================================================
      SUBROUTINE MDFD(F,N1,N2,P,IER)
c-------------------------------------------------------------------------------
      INTEGER N1,N2,IER
      REAL F,P, erfc
      INTEGER I1,I2P,I2,I,L2,MNM,MXM
      REAL ACONS,A,BIGE,B,CBR1,CBR2,C,DPL,DP,F1F,F1,F2P,F2,R1D3,R2D9,
     &     R2DPI,RINFP,STS,S,TEMP1,TEMP,THETA,VP,X1,X2,XI
      character*2 name(3)
      data name(1)/'MD'/, name(2)/'FD'/, name(3)/'  '/
      DATA RINFP/1.E32/
      DATA R2D9/.2222222/,R1D3/.3333333/
      DATA R2DPI/.6366198/
      DATA BIGE/174.6/,ACONS/1.E25/
      MXM=MAX0(N1,N2)
      MNM=MIN0(N1,N2)
c$$$      IF(MNM.LT.1.OR.MXM.GT.(20000-MNM))GOTO 100    ---OLD
      IF(MNM.LT.1.OR.MXM.GT.(100000-MNM))GOTO 100
      IF(F.LT.0.0)GOTO 105
      IER=0
      IF(F.EQ.0.0)GOTO 115
      F1=N1
      F2=N2
      DP=0.0
      VP=F1+F2-2.0
      F1F=F1*F
      F2P=F2+F1F
      X1=F2/F2P
      X2=1.0-X1
      IF(X2.EQ.0.0)GOTO 115
      IF((N1/2)*2-N1.EQ.0.AND.N1.LE.500)GOTO 5
      IF((N2/2)*2-N2.EQ.0.AND.N2.LE.500)GOTO 30
      IF(N1+N2.LE.500)GOTO 55
      F1=R2D9/F1
      F2=R2D9/F2
      CBR1=R1D3*ALOG(F)
      IF(ABS(CBR1).GT.BIGE)GOTO 120
      CBR1=EXP(CBR1)
      CBR2=CBR1*CBR1
      S=(CBR1*(1.0-F2)-1.0+F1)/SQRT(F1+CBR2*F2)
      P=.7071068
      P=.5*ERFC(-P*S)
      GOTO 95
    5 TEMP1=0.
      TEMP=.5*F2*ALOG(X1)
      IF(N1.EQ.2)GOTO 25
      I1=N1-2
      XI=F1
      DO 10I2=2,I1,2
      L2=I2
      XI=XI-2.
      VP=VP-2.
      DP=X2*VP/XI*(1.+DP)
      IF(DP.GT.ACONS)GOTO 15
   10 CONTINUE
      GOTO 25
   15 IF(L2.GE.I1)GOTO 25
      DPL=ALOG(DP)
      I2P=L2+2
      XI=F1-I2P
      DO 20I2=I2P,I1,2
      VP=VP-2.
      DPL=DPL+ALOG(X2*VP/XI)
      XI=XI-2.
   20 CONTINUE
      TEMP=TEMP+DPL
      IF(ABS(TEMP).LE.BIGE)TEMP1=EXP(TEMP)
      P=1.-TEMP1
      GOTO 95
   25 IF(ABS(TEMP).LE.BIGE)TEMP1=EXP(TEMP)
      P=1.0-TEMP1*(1.0+DP)
      GOTO 95
   30 TEMP1=0.
      TEMP=.5*F1*ALOG(X2)
      IF(N2.EQ.2)GOTO 50
      I1=N2-2
      XI=F2
      DO 35I2=2,I1,2
      L2=I2
      XI=XI-2.
      VP=VP-2.
      DP=X1*VP/XI*(1.+DP)
      IF(DP.GT.ACONS)GOTO 40
   35 CONTINUE
      GOTO 50
   40 IF(L2.GE.I1)GOTO 50
      DPL=ALOG(DP)
      I2P=L2+2
      XI=F2-I2P
      DO 45I2=I2P,I1,2
      VP=VP-2.
      DPL=DPL+ALOG(X1*VP/XI)
      XI=XI-2.
   45 CONTINUE
      TEMP=TEMP+DPL
      IF(ABS(TEMP).LE.BIGE)TEMP1=EXP(TEMP)
      P=TEMP1
      GOTO 95
   50 IF(ABS(TEMP).LE.BIGE)TEMP1=EXP(TEMP)
      P=TEMP1*(1.+DP)
      GOTO 95
   55 DP=SQRT(F1F/F2)
      THETA=ATAN(DP)
      STS=F1F/F2P
      A=0.0
      B=0.0
      IF(N2.EQ.1)GOTO 70
      IF(N2.EQ.3)GOTO 65
      I1=N2-3
      XI=F2
      DO 60I2=2,I1,2
      XI=XI-2.
      A=X1*(XI-1.0)/XI*(1.0+A)
   60 CONTINUE
   65 A=X1*DP*(1.0+A)
   70 A=A+THETA
      IF(N1.EQ.1)GOTO 90
      IF(N1.EQ.3)GOTO 80
      I1=N1-3
      XI=F1
      DO 75I2=2,I1,2
      XI=XI-2.
      VP=VP-2.
      B=STS*VP/XI*(1.0+B)
   75 CONTINUE
   80 B=DP*X1*(1.0+B)
      IF(N2.EQ.1)GOTO 90
      I2=N2/2
      C=1.0
      DO 85I=1,I2
      B=B*X1*C/(C-0.5)
      C=C+1.0
   85 CONTINUE
   90 P=R2DPI*(A-B)
   95 IF(P.LT.0.0)P=0.0
      IF(P.GT.1.0)P=1.0
      GOTO 9005
  100 IER=129
      GOTO 110
  105 IER=130
  110 P=RINFP
      GOTO 9000
  115 P=0.0
      GOTO 9005
  120 P=.5
      GOTO 9005
 9000 CONTINUE
      CALL UERTST(IER, name )
 9005 RETURN
      END

c===============================================================================
C                                                                       RTMI  10
C     ..................................................................RTMI  20
C                                                                       RTMI  30
C        SUBROUTINE RTMI                                                RTMI  40
C                                                                       RTMI  50
C        PURPOSE                                                        RTMI  60
C           TO SOLVE GENERAL NONLINEAR EQUATIONS OF THE FORM FCT(X)=0   RTMI  70
C           BY MEANS OF MUELLER-S ITERATION METHOD.                     RTMI  80
C                                                                       RTMI  90
C        USAGE                                                          RTMI 100
C           CALL RTMI (X,F,FCT,XLI,XRI,EPS,IEND,IER)                    RTMI 110
C           PARAMETER FCT REQUIRES AN EXTERNAL STATEMENT.               RTMI 120
C                                                                       RTMI 130
C        DESCRIPTION OF PARAMETERS                                      RTMI 140
C           X      - RESULTANT ROOT OF EQUATION FCT(X)=0.               RTMI 150
C           F      - RESULTANT FUNCTION VALUE AT ROOT X.                RTMI 160
C           FCT    - NAME OF THE EXTERNAL FUNCTION SUBPROGRAM USED.     RTMI 170
C           XLI    - INPUT VALUE WHICH SPECIFIES THE INITIAL LEFT BOUND RTMI 180
C                    OF THE ROOT X.                                     RTMI 190
C           XRI    - INPUT VALUE WHICH SPECIFIES THE INITIAL RIGHT BOUNDRTMI 200
C                    OF THE ROOT X.                                     RTMI 210
C           EPS    - INPUT VALUE WHICH SPECIFIES THE UPPER BOUND OF THE RTMI 220
C                    ERROR OF RESULT X.                                 RTMI 230
C           IEND   - MAXIMUM NUMBER OF ITERATION STEPS SPECIFIED.       RTMI 240
C           IER    - RESULTANT ERROR PARAMETER CODED AS FOLLOWS         RTMI 250
C                     IER=0 - NO ERROR,                                 RTMI 260
C                     IER=1 - NO CONVERGENCE AFTER IEND ITERATION STEPS RTMI 270
C                             FOLLOWED BY IEND SUCCESSIVE STEPS OF      RTMI 280
C                             BISECTION,                                RTMI 290
C                     IER=2 - BASIC ASSUMPTION FCT(XLI)*FCT(XRI) LESS   RTMI 300
C                             THAN OR EQUAL TO ZERO IS NOT SATISFIED.   RTMI 310
C                                                                       RTMI 320
C        REMARKS                                                        RTMI 330
C           THE PROCEDURE ASSUMES THAT FUNCTION VALUES AT INITIAL       RTMI 340
C           BOUNDS XLI AND XRI HAVE NOT THE SAME SIGN. IF THIS BASIC    RTMI 350
C           ASSUMPTION IS NOT SATISFIED BY INPUT VALUES XLI AND XRI, THERTMI 360
C           PROCEDURE IS BYPASSED AND GIVES THE ERROR MESSAGE IER=2.    RTMI 370
C                                                                       RTMI 380
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED                  RTMI 390
C           THE EXTERNAL FUNCTION SUBPROGRAM FCT(X) MUST BE FURNISHED   RTMI 400
C           BY THE USER.                                                RTMI 410
C                                                                       RTMI 420
C        METHOD                                                         RTMI 430
C           SOLUTION OF EQUATION FCT(X)=0 IS DONE BY MEANS OF MUELLER-S RTMI 440
C           ITERATION METHOD OF SUCCESSIVE BISECTIONS AND INVERSE       RTMI 450
C           PARABOLIC INTERPOLATION, WHICH STARTS AT THE INITIAL BOUNDS RTMI 460
C           XLI AND XRI. CONVERGENCE IS QUADRATIC IF THE DERIVATIVE OF  RTMI 470
C           FCT(X) AT ROOT X IS NOT EQUAL TO ZERO. ONE ITERATION STEP   RTMI 480
C           REQUIRES TWO EVALUATIONS OF FCT(X). FOR TEST ON SATISFACTORYRTMI 490
C           ACCURACY SEE FORMULAE (3,4) OF MATHEMATICAL DESCRIPTION.    RTMI 500
C           FOR REFERENCE, SEE G. K. KRISTIANSEN, ZERO OF ARBITRARY     RTMI 510
C           FUNCTION, BIT, VOL. 3 (1963), PP.205-206.                   RTMI 520
C                                                                       RTMI 530
C     ..................................................................RTMI 540
C                                                                       RTMI 550
      SUBROUTINE RTMI(X,F,FCT,XLI,XRI,EPS,IEND,IER)                     RTMI 560
C                                                                       RTMI 570
C                                                                       RTMI 580
C     PREPARE ITERATION                                                 RTMI 590
      IER=0                                                             RTMI 600
      XL=XLI                                                            RTMI 610
      XR=XRI                                                            RTMI 620
      X=XL                                                              RTMI 630
      TOL=X                                                             RTMI 640
      F=FCT(TOL)                                                        RTMI 650
      IF(F)1,16,1                                                       RTMI 660
    1 FL=F                                                              RTMI 670
      X=XR                                                              RTMI 680
      TOL=X                                                             RTMI 690
      F=FCT(TOL)                                                        RTMI 700
      IF(F)2,16,2                                                       RTMI 710
    2 FR=F                                                              RTMI 720
      IF(SIGN(1.,FL)+SIGN(1.,FR))25,3,25                                RTMI 730
C                                                                       RTMI 740
C     BASIC ASSUMPTION FL*FR LESS THAN 0 IS SATISFIED.                  RTMI 750
C     GENERATE TOLERANCE FOR FUNCTION VALUES.                           RTMI 760
    3 I=0                                                               RTMI 770
      TOLF=100.*EPS                                                     RTMI 780
C                                                                       RTMI 790
C                                                                       RTMI 800
C     START ITERATION LOOP                                              RTMI 810
    4 I=I+1                                                             RTMI 820
C                                                                       RTMI 830
C     START BISECTION LOOP                                              RTMI 840
      DO 13 K=1,IEND                                                    RTMI 850
      X=.5*(XL+XR)                                                      RTMI 860
      TOL=X                                                             RTMI 870
      F=FCT(TOL)                                                        RTMI 880
      IF(F)5,16,5                                                       RTMI 890
    5 IF(SIGN(1.,F)+SIGN(1.,FR))7,6,7                                   RTMI 900
C                                                                       RTMI 910
C     INTERCHANGE XL AND XR IN ORDER TO GET THE SAME SIGN IN F AND FR   RTMI 920
    6 TOL=XL                                                            RTMI 930
      XL=XR                                                             RTMI 940
      XR=TOL                                                            RTMI 950
      TOL=FL                                                            RTMI 960
      FL=FR                                                             RTMI 970
      FR=TOL                                                            RTMI 980
    7 TOL=F-FL                                                          RTMI 990
      A=F*TOL                                                           RTMI1000
      A=A+A                                                             RTMI1010
      IF(A-FR*(FR-FL))8,9,9                                             RTMI1020
    8 IF(I-IEND)17,17,9                                                 RTMI1030
    9 XR=X                                                              RTMI1040
      FR=F                                                              RTMI1050
C                                                                       RTMI1060
C     TEST ON SATISFACTORY ACCURACY IN BISECTION LOOP                   RTMI1070
      TOL=EPS                                                           RTMI1080
      A=ABS(XR)                                                         RTMI1090
      IF(A-1.)11,11,10                                                  RTMI1100
   10 TOL=TOL*A                                                         RTMI1110
   11 IF(ABS(XR-XL)-TOL)12,12,13                                        RTMI1120
   12 IF(ABS(FR-FL)-TOLF)14,14,13                                       RTMI1130
   13 CONTINUE                                                          RTMI1140
C     END OF BISECTION LOOP                                             RTMI1150
C                                                                       RTMI1160
C     NO CONVERGENCE AFTER IEND ITERATION STEPS FOLLOWED BY IEND        RTMI1170
C     SUCCESSIVE STEPS OF BISECTION OR STEADILY INCREASING FUNCTION     RTMI1180
C     VALUES AT RIGHT BOUNDS. ERROR RETURN.                             RTMI1190
      IER=1                                                             RTMI1200
   14 IF(ABS(FR)-ABS(FL))16,16,15                                       RTMI1210
   15 X=XL                                                              RTMI1220
      F=FL                                                              RTMI1230
   16 RETURN                                                            RTMI1240
C                                                                       RTMI1250
C     COMPUTATION OF ITERATED X-VALUE BY INVERSE PARABOLIC INTERPOLATIONRTMI1260
   17 A=FR-F                                                            RTMI1270
      DX=(X-XL)*FL*(1.+F*(A-TOL)/(A*(FR-FL)))/TOL                       RTMI1280
      XM=X                                                              RTMI1290
      FM=F                                                              RTMI1300
      X=XL-DX                                                           RTMI1310
      TOL=X                                                             RTMI1320
      F=FCT(TOL)                                                        RTMI1330
      IF(F)18,16,18                                                     RTMI1340
C                                                                       RTMI1350
C     TEST ON SATISFACTORY ACCURACY IN ITERATION LOOP                   RTMI1360
   18 TOL=EPS                                                           RTMI1370
      A=ABS(X)                                                          RTMI1380
      IF(A-1.)20,20,19                                                  RTMI1390
   19 TOL=TOL*A                                                         RTMI1400
   20 IF(ABS(DX)-TOL)21,21,22                                           RTMI1410
   21 IF(ABS(F)-TOLF)16,16,22                                           RTMI1420
C                                                                       RTMI1430
C     PREPARATION OF NEXT BISECTION LOOP                                RTMI1440
   22 IF(SIGN(1.,F)+SIGN(1.,FL))24,23,24                                RTMI1450
   23 XR=X                                                              RTMI1460
      FR=F                                                              RTMI1470
      GO TO 4                                                           RTMI1480
   24 XL=X                                                              RTMI1490
      FL=F                                                              RTMI1500
      XR=XM                                                             RTMI1510
      FR=FM                                                             RTMI1520
      GO TO 4                                                           RTMI1530
C     END OF ITERATION LOOP                                             RTMI1540
C                                                                       RTMI1550
C                                                                       RTMI1560
C     ERROR RETURN IN CASE OF WRONG INPUT DATA                          RTMI1570
   25 IER=2                                                             RTMI1580
      RETURN                                                            RTMI1590
      END                                                               RTMI1600

c===============================================================================
        DOUBLE PRECISION FUNCTION DQINV(P)
c-------------------------------------------------------------------------------
        DOUBLE PRECISION P,T,C0,C1,C2,D1,D2,D3
C
C**RATIONAL APPROX. FOR X(P) WHERE Q(X(P))=P
C  FORMULA 26.2.23 IN ABROMOVICH & STEGUN
C
        T=DSQRT(DLOG(1.D0/(P*P)))
        C0=2.515517D0
        C1=0.802853D0
        C2=0.010328D0
        D1=1.432788D0
        D2=0.189269D0
        D3=0.001308D0
C
C
        DQINV=T - (C0 + C1*T + C2*T*T)/(1.D0 + D1*T + D2*T*T + D3*T*T*T)
C
        RETURN
        END


c===============================================================================
      SUBROUTINE UERTST(IER,NAME)
c-------------------------------------------------------------------------------
      INTEGER IER
c      INTEGER*2 NAME(3)
c      INTEGER*2 NAMSET(3),NAMEQ(3)
      character*2 name(3), namset(3), nameq(3)
      character*1 ieq
c      DATA NAMSET/2HUE,2HRS,2HET/
c      DATA NAMEQ/2H  ,2H  ,2H  /
      data namset(1)/'UE'/, namset(2)/'RS'/, namset(3)/'ET'/
      data nameq(1)/'  '/,  nameq(2)/'  '/,  nameq(3)/'  '/
      DATA LEVEL/4/,IEQDF/0/,IEQ/'='/
      IF(IER.GT.999)GOTO 25
      IF(IER.LT.-32)GOTO 55
      IF(IER.LE.128)GOTO 5
      IF(LEVEL.LT.1)GOTO 30
      CALL UGETIO(1,NIN,IOUNIT)
      IF(IEQDF.EQ.1)WRITE(IOUNIT,35)IER,NAMEQ,IEQ,NAME
      IF(IEQDF.EQ.0)WRITE(IOUNIT,35)IER,NAME
      GOTO 30
    5 IF(IER.LE.64)GOTO 10
      IF(LEVEL.LT.2)GOTO 30
      CALL UGETIO(1,NIN,IOUNIT)
      IF(IEQDF.EQ.1)WRITE(IOUNIT,40)IER,NAMEQ,IEQ,NAME
      IF(IEQDF.EQ.0)WRITE(IOUNIT,40)IER,NAME
      GOTO 30
   10 IF(IER.LE.32)GOTO 15
      IF(LEVEL.LT.3)GOTO 30
      CALL UGETIO(1,NIN,IOUNIT)
      IF(IEQDF.EQ.1)WRITE(IOUNIT,45)IER,NAMEQ,IEQ,NAME
      IF(IEQDF.EQ.0)WRITE(IOUNIT,45)IER,NAME
      GOTO 30
   15 CONTINUE
      DO 20I=1,3
      IF(NAME(I).NE.NAMSET(I))GOTO 25
   20 CONTINUE
      LEVOLD=LEVEL
      LEVEL=IER
      IER=LEVOLD
      IF(LEVEL.LT.0)LEVEL=4
      IF(LEVEL.GT.4)LEVEL=4
      GOTO 30
   25 CONTINUE
      IF(LEVEL.LT.4)GOTO 30
      CALL UGETIO(1,NIN,IOUNIT)
      IF(IEQDF.EQ.1)WRITE(IOUNIT,50)IER,NAMEQ,IEQ,NAME
      IF(IEQDF.EQ.0)WRITE(IOUNIT,50)IER,NAME
   30 IEQDF=0
      RETURN
   35 FORMAT(19H *** TERMINAL ERROR,10X,7H(IER = ,I3,20H) FROM IMSL ROUT
     *INE ,3A2,A1,3A2)
   40 FORMAT(36H *** WARNING WITH FIX ERROR  (IER = ,I3,20H) FROM IMSL R
     *OUTINE ,3A2,A1,3A2)
   45 FORMAT(18H *** WARNING ERROR,11X,7H(IER = ,I3,20H) FROM IMSL ROUTI
     *NE ,3A2,A1,3A2)
   50 FORMAT(20H *** UNDEFINED ERROR,9X,7H(IER = ,I5,20H) FROM IMSL ROUT
     *INE ,3A2,A1,3A2)
   55 IEQDF=1
      DO 60I=1,3
   60 NAMEQ(I)=NAME(I)
   65 RETURN
      END

c===============================================================================
      SUBROUTINE UGETIO(IOPT,NIN,NOUT)
c-------------------------------------------------------------------------------
      INTEGER IOPT,NIN,NOUT
      INTEGER NIND,NOUTD
      DATA NIND/5/,NOUTD/6/
      IF(IOPT.EQ.3)GOTO 10
      IF(IOPT.EQ.2)GOTO 5
      IF(IOPT.NE.1)GOTO 9005
      NIN=NIND
      NOUT=NOUTD
      GOTO 9005
    5 NIND=NIN
      GOTO 9005
   10 NOUTD=NOUT
 9005 RETURN
      END


c===============================================================================
        REAL FUNCTION ERFC(X)
c-------------------------------------------------------------------------------
        REAL X
        DOUBLE PRECISION P,T,XD,A1,A2,A3,A4,A5,derfc
        DATA P/0.3275911D0/,A1/0.254829592D0/,
     &       A2/-0.284496736D0/,A3/1.421413741D0/,
     &       A4/-1.453152027D0/,A5/1.061405429D0/
C
C**ROUTINE TO EVALUATE THE COMPLEMENTARY ERROR FUNCTION
C  FROM FORMULA (7.1.26) OF ABROMOVICH & STEGUN
C  (ACCURATE TO AN ERROR OF LESS THAN  1.5E-07)
C
        XD=DBLE(X)
        T=1.D0/(1.D0+P*XD)
        DERFC=1.D0 - (A1*T + A2*T*T + A3*T*T*T + A4*T*T*T*T +
     &                   A5*T*T*T*T*T)*DEXP(-XD*XD)
        erfc = SNGL (derfc)
        RETURN
        END
c
c===============================================================================
      REAL FUNCTION FPCOH( IS, S, K, I, J )
c-------------------------------------------------------------------------------
      REAL FMCOH, FOCOH, OCOH, MCOH
      INTEGER I, J, K, IS
      COMPLEX S(IS,IS)
      EXTERNAL FMCOH, FOCOH
C 
C   FUNCTION TO DERIVE THE PARTIAL COHERENCE FUNCTION
C   BETWEEN THE K'TH COMPONENT (AS OUTPUT) AND THE I'TH
C   COMPONENT (AS AN INPUT), WITH THE EFFECT OF THE
C   J'TH COMPONENT (THE OTHER INPUT) ON I REMOVED IN A
C   LEAST SQUARES SENSE
C 
      OCOH = FOCOH(IS,S,K,J)
      MCOH = FMCOH(IS,S,K,I,J)
      IF( OCOH.LT.1. .AND.MCOH.LE.1. ) THEN
        FPCOH = ( FMCOH(IS,S,K,I,J) - OCOH ) /
     &          ( 1.                - OCOH )
      ELSE
        FPCOH = -1.
      END IF
      RETURN
      END    

