      SUBROUTINE IOFILE( KEY, TEXT, CHAN, FILE, * )
C-----------------------------------------------------------------------
C
C     ROUTINE TO OPEN OR CLOSE FILES ASSOC. WITH CHANNEL "CHAN"
C     DEPENDING ON THE INTEGER VALUE OF "KEY". IF RETURNED FILE
C     IS EQUAL TO A BLANK STRING, THEN A "RETURN 1" IS MADE.
C
C   PARAMETERS
C   ==========
C
C   KEY            The operation to be performed.
C             1    open a  "new" file sequentially, request file name
C             2    open a formatted "old" file readonly
C             3    open a formatted "old" file with append access
C             4    close a file
C             5    open an unformatted "old" file readonly
C
C
C   TEXT          The text to be output as a prompt for the filename
C                 ( CHARACTER ). If TEXT is equal to a blank string,
C                 then no prompt is issued and the operation is performed.
C
C   CHAN          The channel number to be opened ( INTEGER )
C
C   FILE          On input (for FILE .NE. ' ') this parameter contains the
C                 default filename should a blank string be returned. On
C                 output, it contains the name of the file opened. If the
C                 returned filename is "STOP" or "stop", 
c                 then a "RETURN 1" is made.
C
c                 The default type can be specified by setting FILE
c                 to .TYP where TYP is the default type. 
c
c                 If FILE contains a full name (including type), then 
c                 the type need not be specified in FILEIN if it is the 
c                 same as the default. If the default file is correct, 
c                 but in a different directory, then if the end of the input 
c                 file is either "]" or ":", the default file is appended.
c
C                 If FILEIN is returned as NULL, then no file is opened and
c                 a normal return is made to the programme.
c
c                 If FILEIN is returned as a tfile type only (i.e.
c                 beginning with a period "."), then that type is
c                 appended to the FILE
C
C  *              An error return label.
C
C
C  COMMON BLOCK / IODEV /
C  =======================
C  
C  IN            The terminal input channel (defaults to *)
C
C  OUT           The terminal output channel (defaults to *)
C
C-----------------------------------------------------------------------
C  a.g. jones     ottawa, canada                          feb. 1983
C-----------------------------------------------------------------------
      implicit none
      character text*(*), file*(*), filein*80, type*10, subdir*20,
     &           line*132
      integer key, chan, ierror, ios, lnblnk, lf
c-----------------------------------------------------------------------
      include 'ctrlblk.inc'
      include 'io.inc'
c-----------------------------------------------------------------------

      ios = 0

      if( iprint.ge.2 ) then
        write(*,*)'iofile: entered with key = ', key
      endif

      if( iprint.ge.3 ) then
        write(*,*)'        text=',text
        write(*,*)'        chan=',chan
        write(*,*)'        file=',file
      end if

      if( key .eq. 4 ) then
        close( unit = chan )
        return
      end if

      ierror = 0
      filein = ' '
      subdir = ' '
      type   = ' '

      call trunc( file, lf )
      type = file

    1 if( iprint.ge.0 ) then
        if( file(1:1) .eq. '.' ) then
          type = file
          line = text//' [default type: '//type//']'
          if( out.gt.0 ) then
            write( out,1001 ) line(1:lnblnk(line))
          else
            write(   *,1001 ) line(1:lnblnk(line))
          end if

        else if( file(1:1).eq.' ' ) then
          if( out.gt.0 ) then
            write( out,1001 ) text
          else
            write(   *,1001 ) text
          end if

        else if ( text.ne.' ' ) then
          if( out.gt.0 ) then
            write( out,1004 ) text, file(:lf)
          else
            write(   *,1004 ) text, file(:lf)
          end if

        end if
      end if

      if( text .ne. ' ' ) then
        if( in .ne. 0 ) then
          read( in,'(a)') filein
        else
          read(  *,'(a)') filein
        end if
      end if
      call trunc( filein, lf )

      if( iprint.ge.3 ) write(*,*)'iofile: filein =>',filein(:lf)

      if( filein.eq.'STOP' .or. filein.eq.'stop' ) then
        if( iprint.ge.2 ) write(*,*)'iofile: return 1 with stop as file'
        file = 'stop'
        return 1
      end if

      if( file(1:1).eq.'.' .and. filein.eq.' ' ) then
        file = ' '
        return 1
      end if

      if( filein(1:1).eq.'.') then
        filein = file(1:index(file,'.')-1)//filein
      end if

      if( filein .ne. ' ' ) then
        if( index( filein, '.' ) .eq. 0 ) then
          file = filein( :index(filein,' ')-1)//type
        else
          file = filein
        end if
      end if

      if( file.eq.'NULL' .or. file.eq.'null' ) return

      if( file .eq. ' ' ) return 1

      call trunc( file, lf )

      if( iprint.ge.3 ) write(*,*)'iofile: opening file =>',file(:lf)

      if( key .eq. 1 ) open( unit = chan, status = 'NEW', 
     &        access = 'SEQUENTIAL', file = file, 
     &        err = 9901, iostat=ios )
      if( key .eq. 2 ) open( unit = chan, status = 'OLD', 
     &        access = 'SEQUENTIAL', file = file, 
     &        err = 9901, iostat=ios )
      if( key .eq. 3 ) open( unit = chan, status = 'OLD',
     &        access = 'APPEND', file = file, 
     &        err = 9901, iostat=ios )
      if( key .eq. 5 ) open( unit = chan, status = 'OLD', 
     &        access = 'SEQUENTIAL', form = 'UNFORMATTED', file = file, 
     &        err = 9901, iostat=ios )
c
      return
c
 9901 if( out.gt.0 ) then
        write( out,'(1x,2a,i2,3a,i4)' ) 
     &                    'IOFILE: error opening file',
     &                    '; chan = ', chan, 
     &                    '; file = ', file(:index(file,' ')),
     &                    '; iostat = ', ios
      else
        write(   *,'(1x,2a,i2,3a,i4)' ) 
     &                    'IOFILE: error opening file',
     &                    '; chan = ', chan, 
     &                    '; file = ', file(:index(file,' ')),
     &                    '; iostat = ', ios
      end if
      call perror( ' ')
      if( text.eq.' ' ) return 1
      ierror = ierror + 1
      if( ierror .le. 3 ) then
        goto 1
      else
        return 1
      end if

 1001 format( 1x, a, ' >', $ )
 1004 format( 1x, a, ' [default: ', a, '] >', $ )

      END
