#!/bin/csh -f
#
# PROGRAM: /local/mt/sunbin/lims
#
# FUNCTION: - Master program to perform all the necessary steps to: 
#
#                [1] generate contoured plots of decomposition parameters, 
#                [2] generate contoured plots of phase split from original 
#                    (and eventually decomposed) data files, and/or
#                [3] perform 2D and/or 3D decomposition. 
#
# RUNS: /local/mt/scripts/lims_select     (script)
#       /local/mt/scripts/lims_dos2unix   (script)
#       /local/mt/scripts/lims_magplotd   (script)
#       /local/mt/scripts/lims_wtime      (script)
#       /local/mt/scripts/lims_tscascade  (script)
#       /local/mt/scripts/lims_tsrestack  (script)
#       /local/mt/scripts/lims_mtc        (script)
#       /local/mt/.../bin/lims/tscascade  (executable)
#       /local/mt/.../bin/lims/tsrestack  (executable)
#       /local/mt/.../bin/lims/mtc        (executable)
#                                                                          
#  

set ver=v4.1
echo $ver > .lims.ver

onintr abort

set old=$cwd

echo ' '
echo '             WELCOME TO LIMS PROCESSING MENU'
echo '                   version = '$ver
echo ' '


#set scripts_dir = /mnt/jones/src/lims
set scripts_dir = /local/mt/lims/$ver/scripts

#set execs_dir = /mnt/jones/src/lims
set execs_dir = /local/mt/lims/$ver/sunbin

#execs_dir_select:
#  echo ' '
#  echo -n ' Executables in   '$execs_dir'     Acceptable (y/n)? >'
#  set ans=$<
#  if ($ans == n ) then
#   set execs_dir = /local/mt/sunbin/oldlims
#    goto execs_dir_select
#  endif 

echo $scripts_dir > .lims.scripts_dir
echo $execs_dir   > .lims.execs_dir
      
set lims_dir = /local/mt/lims/data/      #LIMS data location
echo $lims_dir > .lims.lims_dir

set quick = n
echo $quick > .lims.quick

set debug   = n
set Verbose = n

 if ($#argv == 3) then
  set dbase  = $1
  set stn    = $2
  set ans    = $3

  $scripts_dir/lims_select $dbase $stn

       
else if ($#argv == 2) then
  set dbase  = $1
  set stn    = $2

  $scripts_dir/lims_select $dbase $stn

else if ($#argv == 1) then 
  set dbase = $1

  $scripts_dir/lims_select $dbase
  set stn      = `cat .lims.stn`
  set stn_dir  = `cat .lims.stn_dir`  

else if ($#argv == 0) then

  echo ' '
  echo ' lims:   can be called:   lims dbase stn action'
  echo ' '
  echo ' where dbase is the 3-char database & stn is the station number (3 digits)'
  echo ' e.g.:   lims  nod  001  M '
  echo ' '
  echo ' Note: commands with a + sign are backgroundable. e.g. M+ or s+ '
  echo ' '

  $scripts_dir/lims_select 
  set dbase    = `cat .lims.dbase`
  set stn      = `cat .lims.stn`
  set stn_dir  = `cat .lims.stn_dir`  

endif

cd $lims_dir$dbase

if( -e lims.log ) then
  echo 'Appending to log file........'
  date >> lims.log
else
  echo 'Opening log file........'
  date > lims.log
endif

  echo 'station  :' $stn  >> lims.log

cd $old

set stn     = `cat .lims.stn`
set dbase   = `cat .lims.dbase`
set stn_dir = `cat .lims.stn_dir`

echo 'lims dir:' $lims_dir >> lims.log
echo 'database:' $dbase    >> lims.log
echo 'stat dir:' $stn_dir  >> lims.log
echo 'station :' $stn      >> lims.log

#---check for existence of any .lims hidden files

ls $lims_dir$dbase/.lims* > .lims.tmp
set nw = `wc -l .lims.tmp  | awk '{print $1}'`
if ( $nw != 0 ) then
  echo ' '
  echo 'WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING'
  echo ' '
  echo 'lims hidden files found in database directory '$lims_dir$dbase
  cat .lims.tmp
  echo ' '
  echo 'Either they are left over from a previous lims run OR some other user is currently active in that database'
  echo ' '
  echo -n 'Do you wish to proceed  (y/n) ? >'
  set ans = $<
  if ( $ans == y ) then
    echo 'Proceed at your peril..........'
    echo ' '
  else
    goto exit
  endif
endif

onintr menu

if ($#argv == 3) goto choice_do

menu:
  echo ' '
  echo '   STATION SELECTED ==>'$stn
  echo ' '
  echo '   [e]      exit'
  echo '   [m]      plot time series data to screen (and print if desired)'
  echo '   [t]      get start and end time of all .ts data windows'
  echo '   [ ]+     splice all data windows together'
  echo '   [ ]      cross-covariance calculation of two time series'
  echo '   [o]      list overlapping windows & overlap duration '
  echo '   [l]+     tscascade: local-reference processing '
  echo '   [7]+     tscascade: 7-channel remote-reference processing'
  echo '   [r]+     tsrestack '
  echo '   [rr]+    rrrmt'
  echo '   [p]      plot mt responses'
  echo '   [P]+     print mt responses to PostScript printer'
  echo '   [xp]     plot mt responses using xmgr'
  echo '   [d]      GB decomposition of site'
  echo ' '
  echo '   [q]      toggle QUICK MODE (if set, takes defaults)'
  echo '   [f]      list all files for this station'
  echo '   [win]    list all windows (.ts) for this station'
  echo '   [n]      new station'
  echo '   [jobs]   list background and print jobs'
  echo '   [x]      clean up directories (removes all plot & "out" files)'
  echo '   [save]   save existing .dat files'
  echo '   [log]    lists log of activity'
  echo '   [menu]   gives this menu'
  echo '   [notes]  edits (using $EDITOR) the station notes file'
  echo '   [unix]   executes a given UNIX command'
  echo '   [hist]   gives a history of a site or an instrument'
  echo '   [debug]  toggle DEBUG mode (if set, does not delete files)'
  echo '   [verbose] toggle VERBOSE mode (if set, gives info) ***NEW***'

choice:
  cd $old
  echo ' '
  echo -n 'Command  [Default: menu]  ===>'
  set ans = $<

choice_do:

  if( $ans == "" ) then
    goto menu

  else if( $ans == "menu" ) then
    goto menu

  else if( $ans == "log" ) then
    cd $lims_dir$dbase
    tail -20l lims.log
    cd $old
    goto choice

  else if( $ans == "debug" ) then
    if ( $debug == y ) then
      set toggle = n
    else
      set toggle = y
    endif
    echo -n 'Set DEBUG mode (y/n) [default: '$toggle' ] >'
    set ans = $<
    if ( $ans == "" ) then
      set debug = $toggle
    else if ( $ans == y || $ans == Y ) then
      set debug = y
    else
      set debug = n
    endif
    if ( $debug == y ) then
      echo 'DEBUG mode set - files not deleted'
      touch .lims.debug
    else
      if( -e .lims.debug ) rm .lims.debug
    endif
    goto choice

  else if( $ans == "verbose" ) then
    if ( $Verbose == y ) then
      set toggle = n
    else
      set toggle = y
    endif
    echo -n 'Set VERBOSE mode (y/n) [default: '$toggle' ] >'
    set ans = $<
    if ( $ans == "" ) then
      set Verbose = $toggle
    else if ( $ans == y || $ans == Y ) then
      set Verbose = y
    else
      set Verbose = n
    endif
    if( $Verbose == y ) then
      echo 'VERBOSE mode set'
      touch .lims.verbose
    else
      if( -e .lims.verbose ) rm .lims.verbose
    endif
    goto choice


  else if( $ans == "hist" ) then
    echo -n 'Station or LIMS box number ==>'
    set box = ( $< )
    cd $lims_dir$dbase
    grep $box windows.$dbase | sort +3 | grep -v zz
    goto choice

  else if( $ans == "notes" ) then
    $EDITOR "$stn_dir"/notes
    goto choice

  else if( $ans == "unix" ) then
    echo -n 'Unix command ==>'
    set command = ( $< )
    $command
    goto choice


  else if( $ans == "e" ) then
    goto exit

  else if( $ans == "abort" ) then
    goto abort

  else if( $ans == "q" ) then
    if ( $quick == y ) then
      set toggle = n
    else
      set toggle = y
    endif
    echo -n 'Set QUICK MODE (y/n) [default: '$toggle' ] >'
    set ans = $<
    if ( $ans == "" ) then
      set quick = $toggle
    else if ( $ans == y || $ans == Y ) then
      set quick = y
    else
      set quick = n
    endif
    echo $quick > .lims.quick
    if ( $quick == y ) echo 'QUICK MODE set - defaults taken where possible'
    goto choice

  else if( $ans == "m" ) then
    $scripts_dir/lims_xmagp
    goto choice

  else if( $ans == "t" ) then
    $scripts_dir/lims_wtime
    cd ..
    goto choice

  else if( $ans == "s" ) then
#    $scripts_dir/lims_splice
    echo 'Splice option not available yet....'
    goto choice

  else if( $ans == "s+" ) then
    $scripts_dir/lims_splice BACKGROUND
    goto choice

  else if( $ans == "cc" ) then
#    $scripts_dir/lims_crscov 
    goto choice

  else if( $ans == "o" ) then
    $scripts_dir/lims_woverlap
    goto choice

  else if( $ans == "l" ) then
    $scripts_dir/lims_tscascade LOCAL
    goto choice

  else if( $ans == "l+" ) then
    $scripts_dir/lims_tscascade LOCAL BACKGROUND
    goto choice

  else if( $ans == "7" ) then
    $scripts_dir/lims_tscascade REMOTE
    goto choice

  else if( $ans == "7+" ) then
    $scripts_dir/lims_tscascade REMOTE BACKGROUND
    goto choice

  else if( $ans == "r" ) then
    $scripts_dir/lims_tsrestack
    goto choice

  else if( $ans == "r+" ) then
    $scripts_dir/lims_tsrestack BACKGROUND
    goto choice

  else if( $ans == "rr" ) then
    $scripts_dir/lims_rrrmt
    goto choice

  else if( $ans == "rr+" ) then
    $scripts_dir/lims_rrrmt BACKGROUND
    goto choice

  else if( $ans == "p" ) then
    $scripts_dir/lims_mtc 
    goto choice

  else if( $ans == "P" ) then
    $scripts_dir/lims_mtc PRINT 
    goto choice

  else if( $ans == "P+" ) then
    $scripts_dir/lims_mtc PRINT BACKGROUND
    goto choice

  else if( $ans == "xp" ) then
    $scripts_dir/lims_mtplot_xmgr
    goto choice

  else if( $ans == "d" ) then
    $scripts_dir/lims_dcmp
    goto choice

  else if( $ans == "save" ) then
    $scripts_dir/lims_save
    goto choice

  else if( $ans == "f" ) then
    cd $stn_dir
    echo 'LIMS files listed in oldest first order...'
    ls -1rt *.1bn *.ts *.asc *.cas *.z? *.dat *.g* notes *Z *.dat.*
    cd $old
    goto choice

  else if( $ans == "win" ) then
    cd $lims_dir$dbase
    echo ' '
    echo 'Windows available for station '$stn' ...'
    echo ' '
    echo '1 s sampling:'
    grep $stn windows.$dbase | grep ' 1.00'  
    echo ' '
    echo '5 s sampling:'
    grep $stn windows.$dbase | grep ' 0.200'
    cd $old
    goto choice

  else if( $ans == "n" ) then
    $scripts_dir/lims_select $dbase 
    set stn     = `cat .lims.stn`
    set stn_dir = `cat .lims.stn_dir`
    echo 'station  :' $stn  >> $lims_dir$dbase'/lims.log'
    goto choice

  else if( $ans == "jobs" ) then
    ps -f   > .lims.background.jobs
    lpq -Pps  > .lims.print.jobs
    echo 'Background jobs...'
    grep magplotd   .lims.background.jobs 
    grep tscascade   .lims.background.jobs
    grep limssplice .lims.background.jobs 
    grep mtc        .lims.background.jobs
    grep tsrestack   .lims.background.jobs
    echo 'Print jobs...'
    grep bytes      .lims.print.jobs
    goto choice

  else if( $ans == "x" ) then
    cd $lims_dir$dbase
    rm -f *out* *.ps *.upi */*out* */*.ps */*.upi */?.e? */?.h? $stn.1*.* $stn.2*.* $stn.3*.* 
    goto choice

  else
    echo 'Unrecognized command --- please re-enter'
    goto choice

  endif



exit:
  $scripts_dir/lims_cleanup
  exit


abort:
  echo ' CTRL-C exit: lots of lims systems files left'
  echo ' run lims_cleanup to clean them all up'
  exit
