#!/bin/csh -f

onintr exit

#
# 4.00 Hz, 1.00 Hz, 5.0 second and 30.0 second bands for data processing
# do_bands set to 1 if band is to be reprocessed, 0 if not
# do_cas  set to 1 if tscascade is to be used, 0 if rrrmt in that band
#
set bands = ( '4.000000$' '1.000000$' '0.200000$' '0.033333$' )
set do_bands = (   0        1       1      1 )
set do_cas   = (   0        1       1      1 )
#
#
# this script can be run any number of times in parallel with the
# following caveat:
#   if you do rrrmt processing in a band you must ensure multiple
#   instances of processing do not occur simultaneously for the same 
#   site
#
# Sift through all files in project windows file for remote processing
#
# D.E. Boerner July 1994
# mods         Oct  1995
#
# v4.x mods: agj may 1997
#

if ($#argv == 0) then
  echo 'Automatic .ts processing script. '
  echo ' '
  echo 'Sifts through all files in windows.xxx file for processing '
  echo ' '
  echo '* IGNORES ALL SPLICED DATA with the following convention'
  echo ' '
  echo 'Assumes 1s files for  1 sec data' 
  echo '        as files for  5 sec data'
  echo '        xs files for 30 sec data'
  echo ' '
  echo ' You can keep the intermediate cascade files by setting keep=1'
  echo ' You can debug the script by setting debug=1'
  echo ' '
  echo 'USAGE: lims_remote_all <site_number> <MT or GDS> <strike> <keep>/<debug>'
  echo 'i.e. : lims_remote_all 001 mt 0 keep/debug' 
  echo ' '
  exit(0)
else
  set num = $argv[1]
  shift
endif

#
#...set MT or GDS processing
#
if ($#argv == 0) then
  echo -n "MT or GDS ? "
  set mode = ($<) 
else
  set mode = $argv[1]
  shift
endif

#
#...set strike angle
#
if ($#argv == 0) then
  echo -n "Strike angle ? "
  set p_coord = ($<) 
 else
  set p_coord = $argv[1]
  shift
endif
set strike = $p_coord

#
# see if "keep" or "debug" set
#
if ($#argv != 0) then
  set param = $argv[1]
  if ( $param == keep ) then
    set keep  = 1
    set debug = 0
  else if ( $param == debug ) then
    set keep  = 0
    set debug = 1
  else
    set keep  = 0
    set debug = 1
  endif
else
  set keep  = 0
  set debug = 0
endif

if ( $keep  == 1 ) echo 'keep set on'
if ( $debug == 1 ) echo 'debug set on'

set ib     = 1
set srvy   = `echo $cwd | awk -F/ '{print $NF}'` 
set wf     = 'windows.'$srvy
set site   = $srvy$num
set in     = lims_remote_file_iput_$$
set out    = lims_remote_file_oput_$$
rm -f $in $out 


#
#...loop around bands...
#
foreach band ( $bands ) 
  if ( ! $do_bands[$ib] ) goto new_band
  set bd = `echo $band | cut -c1-8`
  grep $site $wf | grep $band  >& /dev/null
  if ( $status ) goto new_band
  echo '----------------------NEW BAND: '$bd' Hz -------------------------'
#
#
  if ( $band == '1.000000$' ) then
    set files = `grep $site $wf | grep $site'1s ' | grep -v '#' | awk '{print $1}'`
  else if ( $band == '0.200000$' ) then
    set files = `grep $site $wf | grep $site'as '  | grep -v '#' | awk '{print $1}'`
   else
    set files = `grep $site $wf | grep $site'xs '  | grep -v '#' | awk '{print $1}'`
  endif

  echo 'Station list of files -->'
  echo $files
  set cascade_list = ' '

  foreach file ( $files )
  echo 'File -->' $file

#...run woverlap to get remote overlaps
    echo "-1"   > $in
    echo $file >> $in
    echo "T"   >> $in
    echo "F"   >> $in
    woverlap -1 < $in > $out

  if ( $band == '1.000000$' ) then
    set remote_list = `cat $out | awk -F: '{print $1}' | grep  1s `
  else if ( $band == '0.200000$' ) then
    set remote_list = `cat $out | awk -F: '{print $1}' | grep as `
  else if ( $band == '0.033333$' ) then
    set remote_list = `cat $out | awk -F: '{print $1}' | grep xs `
  endif

  echo 'Remote list of files -->'
  echo $remote_list

  foreach rfile ( $remote_list )
  if ( $debug == 1 ) echo 'Forming cascade decimates with file '$rfile
#
# form cascade decimates with all available other files (excluding local)
#

    set rnum  = `echo $rfile | cut -c4-6`
    set rem   = `echo $rfile | cut -c4-8`

#...get the start and end times
    set wst = `grep $file $wf | cut -c16-34`
    set wen = `grep $file $wf | cut -c36-54`
    set rst = `grep $rfile $wf | cut -c16-34`
    set ren = `grep $rfile $wf | cut -c36-54`

#...convert dates to numbers for comparison purposes
    set yy = `echo $wst | cut -c1-4`
    set mm = `echo $wst | cut -c6-7`
    set dd = `echo $wst | cut -c9-10`
    set hh = `echo $wst | cut -c12-13`
    set mm = `echo $wst | cut -c15-16`
    set ss = `echo $wst | cut -c18-19`
    set wstn = `echo $yy$mm$dd$hh$mm$ss`
    set yy = `echo $wen | cut -c1-4`
    set mm = `echo $wen | cut -c6-7`
    set dd = `echo $wen | cut -c9-10`
    set hh = `echo $wen | cut -c12-13`
    set mm = `echo $wen | cut -c15-16`
    set ss = `echo $wen | cut -c18-19`
    set wenn = `echo $yy$mm$dd$hh$mm$ss`
    set yy = `echo $rst | cut -c1-4`
    set mm = `echo $rst | cut -c6-7`
    set dd = `echo $rst | cut -c9-10`
    set hh = `echo $rst | cut -c12-13`
    set mm = `echo $rst | cut -c15-16`
    set ss = `echo $rst | cut -c18-19`
    set rstn = `echo $yy$mm$dd$hh$mm$ss`
    set yy = `echo $ren | cut -c1-4`
    set mm = `echo $ren | cut -c6-7`
    set dd = `echo $ren | cut -c9-10`
    set hh = `echo $ren | cut -c12-13`
    set mm = `echo $ren | cut -c15-16`
    set ss = `echo $ren | cut -c18-19`
    set renn = `echo $yy$mm$dd$hh$mm$ss`

    if ( $wstn < $rstn ) set wst = `echo $rst`
    if ( $wenn > $renn ) set wen = `echo $ren`

    set start = `echo $wst`
    set end   = `echo $wen`
    echo 'Start: '$start    '   End: '$end

#...calculate duration
    echo "0"         >  $in          # print level
    echo $start      >> $in          # start
    echo $end        >> $in          # end
    wlength < $in > $out
    set dur   = `cat $out | awk -F= '{print $2}'`
    set ddur  = `echo $dur | cut -c1-4`
    set hdur  = `echo $dur | cut -c5-6`
    set mdur  = `echo $dur | cut -c7-8`
    set sdur  = `echo $dur | cut -c9-10`

#
# Start processing cycle
#
    if ( ! $do_cas[$ib] ) goto do_rrrmt
    set i = 1
    while ( $i <= $#start ) 
      echo "0"         > $in          # print level
      echo "Y"        >> $in          # remote reference
      echo $mode      >> $in          # MT processing
      echo "Y"        >> $in          # STACK
      echo "Y"        >> $in          # JJANAL
      echo $file      >> $in
      echo $rfile     >> $in
      echo "u"        >> $in          # unformatted local  input
      echo "u"        >> $in          # unformatted remote input
      echo $start   >> $in          # start time
      echo $end    >> $in          # end   time
      echo $p_coord   >> $in          # processing coordinate system
      rm -f $site/$file.out7
#

      set window = `echo $file | cut -c4-8`
      if ( $mode == 'GDS' || $mode == 'gds') then
        set casfile = $window$rem'_'$mode.cas_$i
       else
        set casfile = $window$rem.cas_$i
      endif
      set cascade_list = ( $cascade_list $casfile )

      set lab = $file' & '$rfile' (RR)'

      echo $lab': Overlap (days-hh:mm:ss) ='$ddur'-'$hdur':'$mdur':'$sdur

#...remove .out & .cas files if they exists
      rm -f $site/$window$rnum.out >& /dev/null
      rm -f $site/$casfile         >& /dev/null

     if ( $debug == 0 ) then
       tscascade -ask < $in >& $out
     else
       echo 'Input to tscascade.................................'
       cat $in
       echo 'End input..........................................'
       tscascade -ask < $in 
     endif


      mv $site/$window$rnum.cas $site/$casfile
      rm -f $site/$window$rnum.out >& /dev/null
      rm -f $in $out 

      @ i = ( $i + 1 )
    end
new_station:
  end

  end

#...write out cascade files
  rm -f $site/cascade.lst >& /dev/null
  touch $site/cascade.lst
  foreach file ( $cascade_list )
    echo $file >> $site/cascade.lst
  end

#
# restack section
#
  echo "Starting stacking run..."
  echo "0"     > $in
  echo $site  >> $in
  echo "Y"    >> $in
  echo $mode  >> $in 
  if ( $band == '1.000000$' ) then
    echo 0.75    >> $in
  else if ( $band == '0.200000$' ) then
    echo 0.50    >> $in
  else if ( $band == '0.033333$' ) then
    echo 0.25    >> $in
  endif
  echo "0.9999" >> $in
  echo "Y"      >> $in
  echo "JKN"    >> $in
  echo " "      >> $in
  echo "Y"      >> $in
  foreach file ( $cascade_list )
    if ( -e $site/$file ) echo $file >> $in
  end
  echo "stop"     >> $in
  echo "$strike"  >> $in
  rm -f $site/$site.xr $site/$site'.xrg' $site/$site'r.dat' $site/$site.out

  if ( $debug == 0 ) then
    tsrestack -ask < $in >& tsrestack_out
  else
    echo 'Input to tsrestack.................................'
    cat $in
    echo 'End input..........................................'
    tsrestack -ask < $in
  endif


#  delete cascade files
  if ( $keep == 0 ) then
    rm $site/*.cas*
    rm $site/cascade.lst
  endif


  if ( $mode == 'GDS' || $mode == 'gds') then
    mv $site/$site'r.dat $site/$site'_'$ib'r'$mode'.dat'
   else
    mv $site/$site'r.dat' $site/$site'_'$ib'r.dat'
  endif
  rm -f $out $in
  goto new_band

do_rrrmt:
  set i = 1
  set lab = $file' & '$rfile' (RRRMT)'
  nice lims_rrrmt_sno $file -r $rfile -s $strike -d $decl -start $start[$i] -dur $dur[$i] -process_in_strike_dir
#
# fix the header & name of the chave .dat file; this is ugly but it works
#
  if ( -e $site/"$site"c.dat ) then
    cd $site
    set file = `ls *.1mp`
    cd ..
    if ( $#file != 0 ) then
      mphead $file[1] >& head
      set lt = `grep Lat head | cut -c1-27 | awk '{print  $2 + $3/60 + $4/3600 }'`
      set ln = `grep Lon head | cut -c1-27 | awk '{print -$2 - $3/60 - $4/3600 }'`
    endif
    rm -f head

    echo '# MT and GDS estimates from RRRMT' > $site/az.$$ 
    echo '# '`date`                         >> $site/az.$$
    echo '# '                               >> $site/az.$$
    echo '>AZIMUTH    =    ' $strike'.0'    >> $site/az.$$
    echo '>LATITUDE   =    ' $lt            >> $site/az.$$
    echo '>LONGITUDE  =    ' $ln            >> $site/az.$$
    echo '>ELEVATION  =    0.0'             >> $site/az.$$
    cat $site/az.$$  $site/"$site"c.dat > $site/"$site"_"$ib"c.dat
    rm -f  $site/az.$$ 
  endif

new_band:
  @ ib = ( $ib + 1 )
end
#
# merge output files together
#
lims_merge $site 
#
#
# plot final data file
#
# mt_plot jfor/$site.dat
#
#
# create .edi file
#
cd jfor
j2edi <<EOD
$site
stop
12
EOD
mv $site.edi ~/edi
echo ' '
echo ' '
echo 'edi file created (in edi directory)'
cd ..
#
# cleanup files
#

rm -f $in $out 
rm -f $site/$site??.out7
rm -f tmp.*
rm -f lims_remote_file*

exit (0)
