#!/bin/csh  -f
#
#  lims_rrrmt
#
#  reformats data and runs rrrmt
#

onintr exit

set start_dir=$cwd

set scripts_dir = `cat .lims.scripts_dir`
set execs_dir   = `cat .lims.execs_dir`
set quick       = `cat .lims.quick`

if (-e .lims.debug ) then
  set debug = 'y'
else
  set debug = 'n'
endif

if ( $#argv != 0 ) then
  set ans = $1
  if ( $ans == BACKGROUND ) then
    set back = y
  else
    set back = n
  endif
else
  set back = n
endif

#...set scripts for background only
set back = y



start:
set root = ` `

set delta0 = 'none'

set ver      = `cat .lims.ver`
set date     = `date`
set stn      = `cat .lims.stn`
set dbase    = `cat .lims.dbase`
set stn_dir  = `cat .lims.stn_dir`
set lims_dir = `cat .lims.lims_dir`


cd $lims_dir$dbase

rm -f .lims.rrrmt.??.in > /dev/null
rm -f $stn/rrrmt.blk    > /dev/null
echo '# lims_rrrmt script: ver='$ver  >  $stn/rrrmt.blk
echo '# date: '$date                  >> $stn/rrrmt.blk
echo '#'                              >> $stn/rrrmt.blk
echo '# dbase :' $dbase               >> $stn/rrrmt.blk
echo '# stn   :' $stn                 >> $stn/rrrmt.blk
echo '#'                              >> $stn/rrrmt.blk


echo -n 'Run rrrmt locally (y/n) [default: y] ? >'$root
set ans = $<
if ( $ans == "" || $ans == y || $ans == Y ) then
  set machine = local
else
  echo -n 'Give machine name ? >'
  set machine = $<
endif


if !( -e process.cfg ) then
  echo 'This script cannot be run until the process.cfg file has been created'
  goto exit
endif

grep '#ROTANG:' process.cfg  >> .lims.process.cfg

set nl = `wc -l .lims.process.cfg  | awk '{print $1}'`

if ( $nl > 0 ) then
  echo ' '
  echo 'This script cannot be run as the following variables are commented out'
  echo 'in the process.cfg file -->'
  echo ' '
  cat .lims.process.cfg
  echo ' '
  echo 'Edit process.cfg file and re-run'
  goto exit
endif



cd $stn_dir

window:
  echo  ' ' 
  echo  '  WINDOWS FOUND IN: '$stn_dir
  echo  ' '
  ls *.ts

  set nts = `ls -1  *.ts | wc | awk '{print $1}'`
  
  echo  ' ' 
nwind_select:
  echo  -n ' Enter number of sections to process [default: 1] > '$root
  set nwindows=$<
  if( $nwindows == "" ) set nwindows = 1

  echo '# number of sections: '$nwindows     >> rrrmt.blk
  echo '#'                                   >> rrrmt.blk

  echo ' '
  echo 'Local or remote processing ?'
response_select:
  echo -n 'Response (l/r) [default: l] =>'$root
  set r=$<
  if ( $r == "" ) set r = l
  if( $r != 'r'  &&  $r != 'l' ) then
    echo 'Incorrect response -- re-enter'
    goto response_select
  else if( $r == 'r' ) then
    set local = n
  else if( $r == 'l' ) then
    set local = y
  endif


  cd $lims_dir$dbase

  if ($debug != 'y' ) then
    rm .lims.rrrmt.??.in >& /dev/null
  endif

#  if( $local == y ) then
#    if( -e $stn/rrrmt.local.proc ) rm $stn/rrrmt.local.proc
#  else
#    if( -e $stn/rrrmt.remote.proc ) rm $stn/rrrmt.remote.proc
#  endif

  set nwindow = 0

while ( $nwindow != $nwindows )
  @ nwindow = $nwindow + 1

  cd $stn_dir

  if ( $nts > 1 ) then
    echo  ' ' 
    echo 'Run number & Window (no extension) (e.g. a1) for window no. '$nwindow
window_select:
    echo ' '
    echo -n 'Window? ===> '$root
    set window=$<
    set window=$stn$window
    set tts=$window'.ts'
    if ( -e $tts ) then
      echo 'Window selected =>'$window'.ts'
    else
      echo 'Window not found; re-select'
      goto window_select
    endif

  else
    set window = `ls -1  *.ts | awk -F. '{print $1}'`

  endif

if ( $local == y ) then
  set windowrr = $window
  goto times
endif

  cd $lims_dir$dbase
  echo ' '
  set line = `grep $window windows.$dbase`
  echo $line > .lims_local
  set stime = `awk -F-  '{print $1}' .lims_local | awk '{print $4}'`
  set etime = `awk -F-  '{print $2}' .lims_local | awk '{print $1}'`

  echo $execs_dir'/wlength -1 > .lims.duration  <<END'  > .lims_wlength
  echo '-1'                             >> .lims_wlength   #iprint
  echo $stime                       >> .lims_wlength   #start time
  echo $etime                       >> .lims_wlength   #end time
  chmod +x .lims_wlength
  .lims_wlength
  set duration = `cat .lims.duration`
  echo 'Window chosen has interval: ' $stime'-'$etime'   duration: '$duration

  echo ' '
  echo 'SECTION...........' $nwindow
  echo 'Determining possible remote-reference windows'
  echo $execs_dir'/woverlap -1 <<END >.lims.remote'         >  .lims_woverlap
  echo '-1'                                       >> .lims_woverlap
  echo $window                                    >> .lims_woverlap
  echo 'T'                                        >> .lims_woverlap
  echo 'F'                                        >> .lims_woverlap
  echo 'END'                                      >> .lims_woverlap
  chmod +x .lims_woverlap
  .lims_woverlap
  sort +4 .lims.remote

  echo  ' ' 
  echo ' Enter Remote Reference station number (last 3 characters)'
remote_select:
  echo ' '
  echo -n 'Remote? ===> '$root
  set remote=$<
  set remote=$dbase$remote
  if( -e $remote ) then
    echo 'Remote selected =>'$remote
  else
    echo 'Remote not found; re-select'
    goto remote_select
  endif

  grep $remote .lims.remote > .lims.rlist

  if !( -e .lims.rlist ) then
    echo 'Incorrect station listed! Choose again:'
    cat .lims.remote
    goto remote_select
  endif

  set nstat=`wc -l .lims.rlist | awk '{print $1}'`

  if ( $nstat == 1 ) then
    set windowrr = `grep $remote .lims.remote | awk -F: '{print $1}'`
  else
    echo 'More than one window for this station:'
    cat .lims.rlist
    echo ' '
    echo -n 'Remote window (last two characters, e.g. a1) ? ===> '$root
    set windowrr=$<
    set windowrr = `grep $windowrr .lims.rlist | awk -F: '{print $1}'`
  endif

  echo 'Remote window selected =>'$windowrr'.ts'

times:

  if ( $local == y ) then
    cd $lims_dir$dbase
    set line = `grep $window windows.$dbase`
    echo $line > .lims.l
    set stime = `echo $line | awk -F-  '{print $1}' | awk '{print $4}'`
    set etime = `echo $line | awk -F-  '{print $2}' | awk '{print $1}'`

    echo $execs_dir'/wlength -1 > .lims.duration  <<END' > .lims_wlength
    echo '-1'                         >> .lims_wlength   #iprint
    echo $stime                       >> .lims_wlength   #start time
    echo $etime                       >> .lims_wlength   #end time
    chmod +x .lims_wlength
    .lims_wlength
    set duration = `cat .lims.duration`
    set delta = `echo $line | awk -F-  '{print $2}' | awk '{print $2}'`
  else
    set line = `grep $windowrr .lims.rlist`
    set stime = `echo $line | awk -F=  '{print $2}' | awk -F- '{print $1}'`
    set etime = `echo $line | awk -F=  '{print $2}' | awk -F- '{print $2}' | awk '{print $1}'`
    set duration = `echo $line | awk -F=  '{print $3}'`
    set line = `grep $window windows.$dbase`
    set delta = `echo $line | awk -F-  '{print $2}' | awk '{print $2}'`
  endif

  if ( $delta0 == 'none' ) set delta0 = $delta
  if ( $delta != $delta0 ) then
    echo 'Files with different digitization rates chosen - not permitted'
    goto exit
  endif

#  set highpass = `echo $delta | awk '{print 2./($1*30000.)}'`--for Nyquists
  set highpass = `echo $delta | awk '{print 1./($1*30000.)}'`
  echo 'Highpass filter set to '$highpass' freqs'



  echo ' '
stime_select:
  if ( $nwindow != 1 ) then
     set stime = $etime_last
  endif

  if ( $quick != y ) then
    echo 'Give start time (default >'$stime')'
    echo -n 'Start time?           ===>'$root
    set ans=$<
    if !( $ans == "" ) then
      set stime = $ans
    endif
  endif

  echo ' '
etime_select:
  if ( $quick != y ) then
    set etime_last = $etime
    echo 'Give end   time (default >'$etime')'
    echo -n 'End   time?           ===>'$root
    set ans=$<
    if !( $ans == "" ) then
      set etime = $ans
      set etime_last = $etime
    endif
  endif

points:
    echo $execs_dir'/wpoints -1 > .lims.points  <<END' > .lims_wpoints
    echo '-1'                         >> .lims_wpoints   #iprint
    echo $stime                       >> .lims_wpoints   #start time
    echo $etime                       >> .lims_wpoints   #end time
    echo $delta                       >> .lims_wpoints   #end time
    chmod +x .lims_wpoints
    .lims_wpoints
    set plen = `cat .lims.points`



  cd $lims_dir$dbase

  echo '# Section   : '$nwindow              >> $stn/rrrmt.blk
  echo '# tswindow  : '$window               >> $stn/rrrmt.blk
  echo '# tswindowrr: '$windowrr             >> $stn/rrrmt.blk
  echo '# Start time: '$stime                >> $stn/rrrmt.blk
  echo '# End   time: '$etime                >> $stn/rrrmt.blk
  echo '# Points    : '$plen                 >> $stn/rrrmt.blk
  echo '#'                                   >> $stn/rrrmt.blk

  echo 'ts2chave: window='$window' windowrr='$windowrr' stime='$stime' duration='$duration >> lims.log

  if( $local == y ) then
    echo $nwindow': window='$window' windowrr='$windowrr' stime='$stime' duration='$duration >> $stn/rrrmt.local.proc
  else
    echo $nwindow': window='$window' windowrr='$windowrr' stime='$stime' duration='$duration >> $stn/rrrmt.remote.proc
  endif


  echo $nwindow'.ex'   >> .lims.rrrmt.ex.in
  echo '0 '$plen       >> .lims.rrrmt.ex.in
  echo $nwindow'.ey'   >> .lims.rrrmt.ey.in
  echo '0 '$plen       >> .lims.rrrmt.ey.in
  echo $nwindow'.hx'   >> .lims.rrrmt.hx.in
  echo '0 '$plen       >> .lims.rrrmt.hx.in
  echo $nwindow'.hy'   >> .lims.rrrmt.hy.in
  echo '0 '$plen       >> .lims.rrrmt.hy.in
  echo $nwindow'.hz'   >> .lims.rrrmt.hz.in
  echo '0 '$plen       >> .lims.rrrmt.hz.in
  if ( $local == n ) then
    echo $nwindow'.rx'   >> .lims.rrrmt.rx.in
    echo '0 '$plen       >> .lims.rrrmt.rx.in
    echo $nwindow'.ry'   >> .lims.rrrmt.ry.in
    echo '0 '$plen       >> .lims.rrrmt.ry.in
  endif

ts2chave:
  echo ' ' 
  echo 'Reformatting windows '$window' and '$windowrr': length '$plen' points'
  echo $execs_dir'/ts2chave > /dev/null <<END1'        > .lims_ts2chave
  echo '-1'                      >> .lims_ts2chave   #iprint level
if( $local == y ) then
  echo 'n'                       >> .lims_ts2chave   #local processing
else
  echo 'y'                       >> .lims_ts2chave   #RR processing
endif
  echo $window                   >> .lims_ts2chave   #window
if( $local == n ) then
  echo $windowrr                 >> .lims_ts2chave   #rr window
endif
  echo 'u'                       >> .lims_ts2chave   #unformatted output
  echo '30000'                   >> .lims_ts2chave   #blocking size
  echo 'u'                       >> .lims_ts2chave   #input
if( $local == n ) then
  echo 'u'                       >> .lims_ts2chave   #rr input
endif
  echo $stime                    >> .lims_ts2chave   #start time
  echo $etime                    >> .lims_ts2chave   #duration
  echo 'END1'                    >> .lims_ts2chave   #END ts2chave

  chmod +x .lims_ts2chave
  .lims_ts2chave

  set rrnum = `echo $windowrr | cut -c4-6`
  mv $stn/$window$rrnum.ex $stn/$nwindow.ex
  mv $stn/$window$rrnum.ey $stn/$nwindow.ey
  mv $stn/$window$rrnum.hx $stn/$nwindow.hx
  mv $stn/$window$rrnum.hy $stn/$nwindow.hy
  mv $stn/$window$rrnum.hz $stn/$nwindow.hz
  if( $local == n ) then
    mv $stn/$window$rrnum.rx $stn/$nwindow.rx
    mv $stn/$window$rrnum.ry $stn/$nwindow.ry
  endif



end    #end while



rrrmt:
  cd $lims_dir$dbase
  echo 'rrrmt: no of windows='$nwindows  >> lims.log
  echo 'cd '$stn_dir                               > .lims_rrrmt.$$

if ( $back == y ) then
  if ( $plen > 150000 || $nwindows > 1 ) then
    if ( $machine == eos ) then
      echo '/local/mt/conbin/rrrmt.big > .lims.rrrmt.out <<END2' >> .lims_rrrmt.$$
    else
      echo '/mnt/jones/sunbin/rrrmt_lims > .lims.rrrmt.out <<END2' >> .lims_rrrmt.$$
    endif
  else
    if ( $machine == eos ) then
      echo '/local/mt/conbin/rrrmt     > .lims.rrrmt.out <<END2' >> .lims_rrrmt.$$ 
    else
      echo '/local/mt/5bin/rrrmt_lims  > .lims.rrrmt.out <<END2' >> .lims_rrrmt.$$ 
    endif
  endif
else
  echo 'echo " "'                >> .lims_rrrmt.$$
  echo 'echo "rrrmt processing (go away for a long time)....."'  >> .lims_rrrmt.$$
  if ( $plen > 150000 || $nwindows > 1 ) then
    echo 'rrrmt_lims  > .lims.rrrmt.out <<END2'      >> .lims_rrrmt.$$ 
  else
    echo 'rrrmt_lims > .lims.rrrmt.out <<END2'      >> .lims_rrrmt.$$ 
  endif
endif
  echo '3'                     >> .lims_rrrmt.$$   #3 outputs
  echo '2'                     >> .lims_rrrmt.$$   #2 inputs
  echo '1'                     >> .lims_rrrmt.$$   #1 prolate
  echo '16384 2 11'            >> .lims_rrrmt.$$   #FFT, subdiv, no.
  echo '3 2'                   >> .lims_rrrmt.$$   #3rd & 4th harmonics
  echo '1'                     >> .lims_rrrmt.$$   #fudge factor
  echo '0'                     >> .lims_rrrmt.$$   #no power norm
  echo '0 4'                   >> .lims_rrrmt.$$   #lower/upper rej
  echo '0.5 0.1'               >> .lims_rrrmt.$$   #coh thresh & trim
  echo '1'                     >> .lims_rrrmt.$$   #nz
  echo '10'                    >> .lims_rrrmt.$$   #coh block size
  echo '0.0 0.5'               >> .lims_rrrmt.$$   #coh f_lo & f_up
  echo $stn                    >> .lims_rrrmt.$$   #root name
  echo '0'                     >> .lims_rrrmt.$$   #output level
  echo '0'                     >> .lims_rrrmt.$$   #no freq rej
  echo '1'                     >> .lims_rrrmt.$$   #1 filter param
  echo $nwindows               >> .lims_rrrmt.$$   #1 piece
  echo '5'                     >> .lims_rrrmt.$$   #AR length
  echo 'u'                     >> .lims_rrrmt.$$   #unformatted
  echo '30000'                 >> .lims_rrrmt.$$   #blocking size
  echo $highpass               >> .lims_rrrmt.$$   #filter cutoff
  cat .lims.rrrmt.ex.in        >> .lims_rrrmt.$$   #ex data
  echo 'u'                     >> .lims_rrrmt.$$   #unformatted
  echo '30000'                 >> .lims_rrrmt.$$   #blocking size
  echo $highpass               >> .lims_rrrmt.$$   #filter cutoff
  cat .lims.rrrmt.ey.in        >> .lims_rrrmt.$$   #ey data
  echo 'u'                     >> .lims_rrrmt.$$   #unformatted
  echo '30000'                 >> .lims_rrrmt.$$   #blocking size
  echo '0.0'                   >> .lims_rrrmt.$$   #filter cutoff
  cat .lims.rrrmt.hz.in        >> .lims_rrrmt.$$   #hz data
  echo 'u'                     >> .lims_rrrmt.$$   #unformatted
  echo '30000'                 >> .lims_rrrmt.$$   #blocking size
  echo '0.0'                   >> .lims_rrrmt.$$   #filter cutoff
  cat .lims.rrrmt.hx.in        >> .lims_rrrmt.$$   #hx data
  echo 'u'                     >> .lims_rrrmt.$$   #unformatted
  echo '30000'                 >> .lims_rrrmt.$$   #blocking size
  echo '0.0'                   >> .lims_rrrmt.$$   #filter cutoff
  cat .lims.rrrmt.hy.in        >> .lims_rrrmt.$$   #hy data
if( $local == y ) then
  echo 'u'                     >> .lims_rrrmt.$$   #unformatted
  echo '30000'                 >> .lims_rrrmt.$$   #blocking size
  echo '0.0'                   >> .lims_rrrmt.$$   #filter cutoff
  cat .lims.rrrmt.hx.in        >> .lims_rrrmt.$$   #hx as rx data
  echo 'u'                     >> .lims_rrrmt.$$   #unformatted
  echo '30000'                 >> .lims_rrrmt.$$   #blocking size
  echo '0.0'                   >> .lims_rrrmt.$$   #filter cutoff
  cat .lims.rrrmt.hy.in        >> .lims_rrrmt.$$   #hy as ry data
else
  echo 'u'                     >> .lims_rrrmt.$$   #unformatted
  echo '30000'                 >> .lims_rrrmt.$$   #blocking size
  echo '0.0'                   >> .lims_rrrmt.$$   #filter cutoff
  cat .lims.rrrmt.rx.in        >> .lims_rrrmt.$$   #rx data
  echo 'u'                     >> .lims_rrrmt.$$   #unformatted
  echo '30000'                 >> .lims_rrrmt.$$   #blocking size
  echo '0.0'                   >> .lims_rrrmt.$$   #filter cutoff
  cat .lims.rrrmt.ry.in        >> .lims_rrrmt.$$   #ry data
endif
  echo '0 90 0'                >> .lims_rrrmt.$$   #rotation for ex,ey
  echo '0 90 0'                >> .lims_rrrmt.$$   #rotation for hx,hy
  echo '0 90 0'                >> .lims_rrrmt.$$   #rotation for rx,ry
  echo 'END2'                  >> .lims_rrrmt.$$   #END rrrmt

  echo ' '                     >> .lims_rrrmt.$$   #
  echo 'cat rrrmt.blk '$window$rrnum'.blk > '$stn'.blk'  >> .lims_rrrmt.$$
  echo ' '                     >> .lims_rrrmt.$$   #

if( $back == y ) then
  if ( $machine == eos ) then
    echo '/local/mt/conbin/c2j    > .lims.c2j.out <<END3'   >> .lims_rrrmt.$$ 
  else
    echo $execs_dir'/c2j          > .lims.c2j.out <<END3'   >> .lims_rrrmt.$$ 
  endif
else
  echo 'echo " "'              >> .lims_rrrmt.$$
  echo 'echo "converting to J-format....."'   >> .lims_rrrmt.$$
  echo 'c2j        <<END3'               >> .lims_rrrmt.$$ 
endif
  echo $stn                    >> .lims_rrrmt.$$   #stn name
  echo $delta                            >> .lims_rrrmt.$$   #delta-t in Hz
  echo 'm'                               >> .lims_rrrmt.$$   #MT/GDS data
  echo ' '                               >> .lims_rrrmt.$$   #Zxx 
  echo ' '                               >> .lims_rrrmt.$$   #Zxy 
  echo ' '                               >> .lims_rrrmt.$$   #Zyx 
  echo ' '                               >> .lims_rrrmt.$$   #Zyy 
  echo ' '                               >> .lims_rrrmt.$$   #Tzx 
  echo ' '                               >> .lims_rrrmt.$$   #Tzy 
  echo 'END3'                            >> .lims_rrrmt.$$   #END c2j

  if ($debug != 'y' ) then
    echo 'rm ?.?x ?.?y ?.hz'       >> .lims_rrrmt.$$   #rm data files
    echo 'rm *.c2 *.rf *.diag'     >> .lims_rrrmt.$$   #rm chave output
    echo 'rm *.blk'                >> .lims_rrrmt.$$   #rm chave output
  endif

  echo 'mv '$stn'c.dat '$window$rrnum'c.dat'        >> .lims_rrrmt.$$
  echo ' '                                          >> .lims_rrrmt.$$
  echo 'echo rrrmt job complete '                   >> .lims_rrrmt.$$

  chmod +x .lims_rrrmt.$$

  if ( $machine == local ) then

    if ( $plen > 150000 || $nwindows > 1 ) then
      echo 'Backgrounding rrrmt_lims job locally...'
    else
      echo 'Backgrounding rrrmt job locally...'
    endif

    .lims_rrrmt.$$ &

  else
    if ( $plen > 150000 || $nwindows > 1 ) then
      echo 'Backgrounding rrrmt.big job to machine '$machine'...'
    else
      echo 'Backgrounding rrrmt job to machine '$machine'...'
    endif

    rsh $machine $lims_dir$dbase'/.lims_rrrmt.'$$  &
  endif



exit:
  cd $start_dir
