c
c     ts2asc
c
c     reads in a .ts file and writes out the data as <nchan> ASCII

      program ts2asc

      implicit none

      include 'infoblk.inc'
      include 'io.inc' 
      include 'ctrlblk.inc'
      include 'version.inc'
      include 'op_sys.inc'
      include 'bdate.inc'

      character tswindow*20, fin*30, fout*30
      integer yy, mm, dd, hh, min, ss, dday, dhour, dmin, dsec
      integer npt, sdur, datchk
      integer byy, bmm, bdd, bhh, bmn, bss
      integer loop, i, instr, nchanx, tdiff, nskip
      integer nsec
      integer lnblnk
      real data(5), data1(5)
      character*19 stime, etime, nowtim, thistime
      character proj*3, dur*13, line*80

      real basehx, basehy, azimhx, azimex, azimey, azimuth
      real anghx, angex, angey
      real hx, hy, ex, ey
      real magor

      logical XMGR, FVAL, MAGCORR, TIMESTAMP

      external date

c-------------------------------------------------------------------------------

      call getctrl_blk()

c-------------------------------------------------------------------------------
c...write banner out

      call banner( 'TS2ASC' )

c-------------------------------------------------------------------------------

      TIMESTAMP = .FALSE.
      call lin('Timestamp each set of data', TIMESTAMP )

      if( TIMESTAMP ) then
        XMGR = .FALSE.
      else
        XMGR = .FALSE.
        call lin('Is the output for XMGR plotting', XMGR )
      endif

      FVAL = .FALSE.
      call lin('Remove first value from all measurements (obs data)?',
     &          FVAL )

c-------------------------------------------------------------------------------
c...open input file

      tswindow = ' '
      form = '?'
      call ts_open(ind1, tswindow, fin, .TRUE., .FALSE.)
      if( XMGR ) then
        fout = fin(1:index(fin,'.'))//'xmgr.asc'
      else
        fout = fin(1:index(fin,'.'))//'asc'
      endif
      open(unit=outd1,file=fout,status='unknown')
      write(*,'(2a)')'Opened output file...', fout
      proj  = tswindow(1:3)


c...read header
      call ts_rdinfo(ind1)

c...calculate azimuths on first pass
      do i = 1, nchan
        if( chan(i).eq.'HX' ) basehx = baseline(i)
        if( chan(i).eq.'HY' ) basehy = baseline(i)
      enddo
      if( basehx.eq.misdat .or. basehy.eq.misdat ) then
        write(*,'(a)')'Cannot calculate mag orientation: '//
     &                ' baselines not found'
        magor = 0.
      endif
      magor = atan2d( basehy, basehx )
      write(*,'(a,f6.2)')'Magnetic orientation from baselines = ', 
     &                    magor
      MAGCORR = .TRUE.
      call lin( 'Apply this correction', MAGCORR )
      if( .not.MAGCORR ) magor = 0.

      write(*,*)'MEASURED : Hx =', azim(1), '  Ex =', azim(4),
     &     '  Ey =', azim(5)
      write(*,*) 'Coord sys: ', azdir, 'decl =', declination, 
     &           '   magor =', magor

      if( index(azdir, 'MAGNETIC').ne.0 ) then
        azimhx = azim(1) + declination + magor
        azimex = azim(4) + declination
        azimey = azim(5) + declination
      else
        azimhx = azim(1) + magor
        azimex = azim(4)
        azimey = azim(5)
      endif
      write(*,*)'TRUE     : Hx =', azimhx, '  Ex =', azimex,
     &      '  Ey =', azimey

      azimuth = 0.
      call rin( 'Give desired azimuth of plotted data', azimuth )

c...rotation angles to apply
      anghx = azimuth - azimhx
      angex = azimuth - azimex
      angey = azimuth - azimey + 90.
      write(*,*)'ROTATIONS : Hx =', anghx, '  Ex =', angex,
     &     '  Ey =', angey

      call date( 0, starttime, byy, bmm, bdd, bhh, bmn, bss)

      if( tunits.eq.'Hz' ) then
        continue
      else if( tunits.eq.'s' ) then
        delta = 1./delta
      else
        write(*,*)'Time units not recognised >', tunits
        write(*,*)'only "s" or "Hz" permitted'
        stop
      endif


c...get start and end times of window and lunchbox number
      instr = -999
      open(unit=ind2,file='windows.'//proj, status='old',err=120)
      call getwindow( ind2, tswindow(1:8), instr, nchanx, stime, etime )
      close(unit=ind2)

110   call nin( 'Give required start time           >'//
     &          'yyyy-mm-dd hh:mm:ss')
      stime = starttime
      call tin (' ', stime )
      if( datchk( stime ).ne.0 ) goto 110

      call date( 0, stime, yy, mm, dd, hh, min, ss)
      if ( yy .ne. byy ) then
        write(*,'(a, i4)')'Requested wrong year!  Data block year is ',
     &                     byy
        go to 110
      endif


115   call nin( 'Give required end   time           >'//
     &          'yyyy-mm-dd hh:mm:ss')
      etime = endtime
      call tin (' ', etime )
      if( datchk( etime ).ne.0 ) goto 115

      if( stime.ge.etime ) then
        write(*,*) 'Start time AFTER end time..... re-enter'
        goto 110
      endif

      call date( 0, stime, yy, mm, dd, hh, min, ss)
      sdur = tdiff( stime, etime )

c instr = -999 if no window information can be found.
c
120   if (instr .eq. -999) then
        write(*,*)'  Cannot find window file: windows.'//proj,
     &                      '.  Header values used.'
        read( starttime, '(4x,i2)') dday
        read( starttime, '(6x,i2)') dhour
        read( starttime, '(8x,i2)') dmin
      else
        dday  = sdur/(1440*60)
        dhour = (sdur - dday*1440*60)/60
        dmin  = (sdur - dday*1440*60 - dhour*60)/60
        dsec  = sdur - dday*1440*60 - dhour*60*60 - dmin*60
        if( iprint.ge.1 ) then
          write(*,*)' Duration: secs = ', sdur
          write(*,*)'dday  =', dday
          write(*,*)'dhour =', dhour
          write(*,*)'dmin  =', dmin
          write(*,*)'dsec  =', dsec
        endif
      endif

      write( dur, '(i4.4,a1,i2.2,a1,i2.2,a1,i2.2)') 
     &           dday, '/', dhour, '/', dmin, ':', dsec
      write( *,'(2a)') 'Duration (dddd/hh/mm:ss) : ', dur

c...go to top of file and read in and write out comment & info blocks 
c   changing the start and end times

      if( XMGR ) goto 131
      rewind(ind1)
      if( form.eq.'BINARY' ) then
        read(ind1) line
      else
        read(ind1,'(a)') line
      endif

      do while( line(1:1).eq.'#' )
        write(outd1,'(a)') line(1:lnblnk(line))
        if( form.eq.'BINARY' ) then
          read(ind1) line
        else
          read(ind1,'(a)') line
        endif
      enddo

      do while( line(1:1).eq.'>' )

        if( index(line,'>STARTTIME').ne.0 ) then
          write(line(13:),'(a)') stime
        elseif( index(line,'>ENDTIME').ne.0 ) then
          write(line(13:),'(a)') etime
        elseif( index(line,'>FORM  ').ne.0 ) then
          write(line(13:),'(a)') 'ASCII'
        elseif( index(line,'>FORMAT').ne.0 ) then
          write(line(13:),'(a)') 'FREE'
        endif

        write(outd1,'(a)') line(1:lnblnk(line))

        if( form.eq.'BINARY' ) then
          read(ind1,err=130) line
        else
          read(ind1,'(a)') line
        endif
      enddo
130   backspace(ind1)


c...calculate the total number of seconds of data to write      

131   npt   = sdur * delta
      write(*,'(a,2i7)')'Number of seconds/data to write = ', 
     &                 sdur, npt
      write(*,'(4a)')'reading data... stime:',stime,'  etime:',etime


c...calculate the number of data to skip      
      nskip = tdiff( starttime, stime ) * delta

      if( nskip.gt.0 ) then
        write(*,*) 'Skipping',nskip,' entries'
        call ts_skip( ind1, nskip )
      endif

      do loop = 1, npt
        if( form.eq.'BINARY' ) then
          read(ind1,end=9999) (data(i),i=1,nchan)
        else if( format.eq.'FREE' ) then
          read(ind1,*,end=9999) (data(i),i=1,nchan)
        else 
          read(ind1,format,end=9999) (data(i),i=1,nchan)
        endif

c...rotate data
        hx = data(1)
        hy = data(2)
        if( hx.ne.misdat .and. hy.ne.misdat .and. 
     &      anghx.ne.0. ) then
          call rotdat( hx, hy, anghx, anghx )
          data(1) = hx
          data(2) = hy
        endif
        ex = data(4)
        ey = data(5)
        if( hx.ne.misdat .and. hy.ne.misdat .and. 
     &      anghx.ne.0. ) then
          call rotdat( ex, ey, angex, angey )
          data(4) = ex
          data(5) = ey
        endif

        if( FVAL .and. loop.eq.1 ) then
          do i = 1, nchan
            data1(i) = data(i)
          enddo
        endif
        if( XMGR ) then
          if( FVAL ) then
            write(outd1,'(i10, 5g14.5)' ) loop, 
     &                                     (data(i)-data1(i),i=1,nchan)
          else
            write(outd1,'(i10, 5g14.5)' ) loop, (data(i),i=1,nchan)
          endif
        else
          if( TIMESTAMP ) then
            nsec = nint( float(loop-1) / delta )
            thistime = nowtim( stime, 0, 0, 0, nsec )
            if( FVAL ) then
              write(outd1,'(a19, 5g12.5)' ) thistime,
     &                        (data(i)-data1(i),i=1,nchan)
            else
              write(outd1,'(a19, 5g12.5)' ) thistime,
     &                        (data(i),i=1,nchan)
            endif
          else
            if( FVAL ) then
              write(outd1,'(5g16.8)' ) (data(i)-data1(i),i=1,nchan)
            else
              write(outd1,'(5g16.8)' ) (data(i),i=1,nchan)
            endif
          endif
        endif
      enddo
      goto 9999


 
9001  write (*, '(a)') ' ERROR converting integers to characters.'
      go to 9999
9002  write (*, '(a)') ' ERROR converting characters to integers.'
      goto 9999

9999  stop
      end
