      program tscascade
c
c  Remote reference version of LIMS_CASCADE
c  requires 2 datafiles and uses 7 channels
c  Alan G Jones; 17 Sept 1992
c
c  This version of CASCADE processes time series data in ts format.
c     July 5, 1988
c  Modified to incorporate system calibration files.
c     Only the gains and telluric line lengths are corrected here.
c     RESTKMT corrects for the filters.
c     September 1, 1988.
c
c  Modified to use binary data from LIMS4
c     September 4, 1990.
c
c  August 12, 1992 (Angela Tchen)
c  dchan in common block /contst/ is a.k.a. nchan in rdhedb_sub.f and
c    input1b_sub.f
c
c  August 19, 1992 (Angela Tchen)
c  Updated for use on the SPARC stations from PC source code.
c  logical variable SUN was introduced and added to common block /contst/
c
c  Sept - Dec 1992: Alan Jones
c  lots of changes. Too many to list. This is now the world's best
c  processing code.
c
c  Sept 1993 : Jim Craven
c  trivial changes, call input1d to read in .1mp (dos binary) data
c
c  Sept 1993 : David Boerner 
c  trivial changes to check measurement azimuth and pass on to dat-file
c
c  Sept 1993 : Jim Craven
c  trivial changes to remove system dep commandline calls, added SCCSID
c
c  3.1: Sept 1994: AGJ
c                - rotates Fourier components to required angle (azimuth)
c                - does different angles for (Hx,Hy), Ex, Ey, and (Rx,Ry)
c                - sliding sizes in numstk instead of 8 for all
c                - added variance tolerance for minimisation
c                - reads in some params in "process.cfg" file
c                - added keywords DECLIN & ROTANG in process.cfg
c
c  4.0: March 1997: AGJ: renamed to TSCASCADE
c                - reads in time series in ts format
c                - all times now in 12 character format to allow seconds
c
c  4.1: Sept  1997: AGJ: numerous small code changes to run under WATCOM
c
c       Dec   1997: AGJ: changes to permit 64-bit version
c
c  4.2: Jan   1998: AGJ: added "REMOTE" to process.cfg to permit E-field to be
c                        used as remote
c       Aug   1998: AGJ: added "MAG_OR" to correct for orientation of the
c                        magnetometer using the baseline values
c                        set MISDAT data to last data value
c
c  5.0: Nov   1998: AGJ: added COHMIN: 20*0.500 as an array 
c                         (not used in tscascade)
c                        added checking of version numbers in process.cfg
c                        19-character time format
c
c-------------------------------------------------------------------------------
c
c I/O
c
c     ind1:    time series input
c
c     outd1:   process.cfg file written out - only used if process.cfg 
c              does not exist
c     outd2:   cascade output: UNFORMATTED
c

      implicit none

      include 'maxflims.inc'
      include 'cas7.inc'
      include 'defaults.inc'
      include 'version.inc'
      include 'ctrlblk.inc' 
      include 'io.inc' 
      include 'infoblk.inc'
      include 'op_sys.inc'
      include 'bdate.inc'
      
c...set as integer*4 to be compatible with SUN getarg routine
      integer*4 iarg

      integer narg, getnarg, i, j, instr1, instr2, io, jv, k
      integer m, n, ip, ix, iu
      integer numstk(MAXF)

      integer*4  lnblnk

      integer lw
      real di, dz, PI, xjv, t, e, f, xiv, ym, yn, yi, yj, yk, yl
      real yg, yh, xiu
      real vartol

      real d(5), w(MAXF2), x(MAXF2)
      real y(MAXF2,49), z(MAXF2,49)
      integer iv(MAXF2), it(MAXF2)
      real    yyy(64,49,MAXF2), zz(64,49,MAXF2)


c...data declared as real*4 in case of double precision compilation
      real*4 dat(5), dat2(5)

      real delta1, delta2, totn
      real ex, ey, hx, hy, hz, rx, ry 
      real exl, eyl, hxl, hyl, hzl, rxl, ryl
      real period(MAXF)
      integer daterr, nmax, nskip
      integer yy, mm, dd, hh, min, ss, dday, dhour, dmin, dsec, tsec
      integer nchan1, nchan2, inst1, inst2
      integer byy,  bmm,  bdd,  bhh,  bmn,  bss, nfw, inc
      integer byy1, bmm1, bdd1, bhh1, bmn1, bss1, inc1
      integer byy2, bmm2, bdd2, bhh2, bmn2, bss2, inc2
c
      integer azimex, azimey, azimhx, azimrx, azimuth
      integer dec, tdiff, sdur
      real angex, angey, anghx, angrx
      real basehx, basehy, magor, magor2
c
      real lat1, long1, lat2, long2, elev1, elev2 
      real azim1(5), azim2(5), declin1, declin2
      real misdatl, misdatu

      logical STACK, JJANAL, ORITE, RR, HSG, ASK, AZIMSET, MAGREMOTE,
     &        MAG_OR, P_NORM
      logical QSTACK, QJJANAL, QCOHMIN, QCOHMAX, QPROTYP, QORITE,
     &        QROTANG, QDECLIN, QMAXTYP, QNTYPE, QWEIGHT, QUNITS,
     &        QREMOTE, QMAG_OR, QCFGVER
      logical SINGLE, SET_ZERO

      character title*20, fout*40, indat1*40, indat2*40, line*80,
     &           word*10
      character proj*3, sep*1 
      character*5 tunits1, tunits2
      character*8 tswindow, tswindowrr
      character*6 statrr, stattmp
      character*19 stime, etime, stime1, etime1, stime2, etime2
      character*10 cmdarg
      character*1 ans, mt, stackans, jjans
      character*17 azdir1, azdir2
      character*15 form1, form2
      character*10 dur
      character*6 progprec
      character*4 cfgver
      character*60 path

      character*24 date_string

      common /degf/   nfw(MAXF)
      common /event/  title
      common /yz/     yyy, zz, JJANAL
      common /vartol/ vartol



      external date


      data indat1/ 'sno105f8' /

c---ASSUMED HERE THAT MAXF = 20
c   Its really a big deal to try to extend beyond 20 freqs. Going into the
c   spaghetti of Wight's cascade code.


c...if you use a value greater than 64 here then you must 
c   1: main:      change array dimensions in common /yz/
c   2: stack7:    change array f and the arrays in common /yz/
c                 (must add F distribution numbers for f)
c   3: minvar7:   change arrays f1 and f2 and the arrays in common /yz/
c                 (must add F distribution numbers for f1 and f2)
c   4: tsrestack: change array f dimension
c                 (must add F distribution numbers for f)
      data numstk/32,32,32,32,16,16,16,16,8,8,8,8,8,8,8,8,4,4,4,4/

c-------------------------------------------------------------------------------
c     initialize

      call getdir(path)

      call fdate(date_string)
      call chkend(sysend)

      call precision( SINGLE )
      if( SINGLE ) then
        progprec = 'SINGLE'
      else
        progprec = 'DOUBLE'
      endif

      call dirsep(sep)

c-------------------------------------------------------------------------------
c     set logicals
      
      ASK     = .FALSE.

      QCFGVER = .TRUE.
      QSTACK  = .TRUE.
      QJJANAL = .TRUE.
      QCOHMIN = .TRUE.
      QCOHMAX = .TRUE.
      QROTANG = .TRUE.
      QDECLIN = .TRUE.
      QMAG_OR = .TRUE.
      QPROTYP = .TRUE.
      QORITE  = .TRUE.
      QMAXTYP = .TRUE.
      QNTYPE  = .TRUE.
      QWEIGHT = .TRUE.
      QREMOTE = .FALSE.
      QUNITS  = .FALSE.
      AZIMSET = .FALSE.

      P_NORM  = .TRUE.

      mt = 'M'
      stackans = 'Y'
      jjans = 'Y'
      dec = 0
      declin1 = -999.
      declin2 = -999.

c...Horizontal magnetics will be used as remote unless otherwise set in
c   process.cfg file
      MAGREMOTE = .TRUE.

      inc   = 2
      inc1  = 2
      inc2  = 2
      vartol = 0.01


      iprint = 0


c-------------------------------------------------------------------------------
c     get command-line arguments

      narg = getnarg()
      do iarg = 1, narg
        call getcmdarg( iarg, cmdarg)
        if( index(cmdarg,'h').ne.0 ) then
          write(*,*)'Permitted options:-'
          write(*,*)'                    -1  or -2  iprint level'
          write(*,*)'                    -ask       ask all questions'
          stop
        else if( index(cmdarg,'ask').ne.0 ) then
          ASK = .TRUE.
        else
          read(cmdarg,'(i2)') iprint
        endif
      enddo

      if(iprint.eq.0)call iin( 'Give print level (-1/0/1/2/3)',iprint)

 
      if( iprint.ge.1 ) then
          write(*,*) 'Program start at: ', date_string
          write(*,*) 'Machine endian  : ', sysend
          write(*,*) 'UNIX setting    : ', UNIX
        endif

c-------------------------------------------------------------------------------
c     write banner for iprint >= 1

      if( iprint.ge.1 ) then
        write(*,*)' '
        write(*,*)' >>>>>>>>>>>>> TSCASCADE <<<<<<<<<<<<<<<<<<<'
        write(*,*)' '
        write(*,*)'           version = ', ver
        write(*,*)'build date = ', bdate
        write(*,*)' '
        if( SINGLE ) then
        write(*,*)'        SINGLE PRECISION version'
        else
        write(*,*)'        DOUBLE PRECISION version'
        endif
        write(*,*)' '
        write(*,*)'It is assumed that this program is being run from'
        write(*,*)'the database directory of the project '
        write(*,*)'(e.g. /local/mt/lims/nod)  '
        write(*,*)'and that all data files reside in subdirectories'
        write(*,*)'with the respective station names '
        write(*,*)'(e.g. /local/mt/lims/nod/nod001 nod002 etc.)'
        write(*,*)' '
        write(*,*)'window file is required (e.g. windows.nod) '
        write(*,*)' '
      else

        write(*,'(a)')'tscascade: ver='//ver//'   '//progprec//
     &                ' precision'
      endif

c-------------------------------------------------------------------------------
c     Check for configuration file. If doesn't exist then create it and write
c     default values into it.

      open(unit=ind3, file='process.cfg', status='old', err=101)
      if( iprint.ge.0 ) then
        write(*,'(a)')'Opened processing configuration file process.cfg'
      endif
      goto 102

101   open(unit=outd1, file='process.cfg', status='new')

      write(outd1,'(a)')'# process.cfg: version: '// ver
      write(outd1,'(a)')'#'
      write(outd1,'(a)')'# This file contains parameters that may be '
      write(outd1,'(a)')'# altered for running tscascade and tsrestack'
      write(outd1,'(a)')'# Parameters listed below are the default '
      write(outd1,'(a)')'# values in the tscascade executable.'
      write(outd1,'(a)')'#'
      write(outd1,'(a)')'# A hash (#) in the first column comments '//
     &                  'the line'
      write(outd1,'(a)')'# out. That variable is requested on execution'
      write(outd1,'(a)')'#'
      write(outd1,'(a)')'VER   :'//ver//'     cfg file version'
      QCFGVER = .FALSE.
      write(outd1,'(a,i2,a)')'NUMSTK:',MAXF,
     &                  '   number of estimates per substack'
      write(outd1,'(20i4)') (numstk(i),i=1,MAXF)
      write(outd1,'(a,f6.4,a)')'VARTOL:', vartol,
     &                  '  tolerance for minimum variance'
      write(outd1,'(a)')'# Other parameters are allowed (remove # to '//
     &                  'use)'
      write(outd1,'(a)')'OWRITE:   T     overwrite files without '//
     &                  'asking'
      write(outd1,'(a)')'PROTYP:  MT     processing type: MT, GDS '//
     &                  'or HSG'
      write(outd1,'(a)')'REMOTE:   H     type of remote: H or E '//
     &                                '(not requested if absent)'
      write(outd1,'(a)')'#DECLIN:   0     sets declination'
      write(outd1,'(a)')'#ROTANG:   0     sets rotation co-ord system'
      write(outd1,'(a)')'MAG_OR:   F     correct magnetometer '//
     &                  'orientation using baselines'
      write(outd1,'(a)')'P_NORM:   T     normalize spectra by '//
     &                  'horizontal magnetic field power'
      write(outd1,'(a)')'STACK :   T     .TRUE. substacking'
      write(outd1,'(a)')'JJANAL:   T     .TRUE. Jones-Joedicke '//
     &                  'processing'
      write(outd1,'(a)')'MAXTYP: JKN     max type: JKN, COH or VAR'
      write(outd1,'(a)')'NTYPE :   1     param to minimize'
      write(outd1,'(a)')'WEIGHT:   T     apply weighting'
      write(outd1,'(a)')'COHMIN: 20*0.500   coherence minimum'
      write(outd1,'(a)')'COHMAX: 0.999   coherence maximum'
      write(outd1,'(a)')'UNITS :  SI     impedance units: SI or FIELD'
      close(unit=outd1)

      if( iprint.ge.0 ) then
        write(*,'(a)')'Configuration file process.cfg written'//
     &            ' with default values'
      endif
      goto 110

102   do 103 while(.TRUE.)
        read(ind3,'(a)',end=104) line
        if( iprint.ge.3 ) then
          write(*,*)'Process line >',line(1:lnblnk(line))
        endif
        if( line(1:1).eq.'#' ) goto 103

        if( line(1:6).eq.'VER   ' ) then
          call word1( line(8:), cfgver, lw )
          if( iprint.ge.1 ) then
            write(*,*)'VER    found with value :', cfgver
          endif
          if( cfgver.ne.ver ) then
            write(*,*)'Processing configuration file of '//
     &                'inappropriate version number'
            write(*,*)'Program version =', ver, 
     &                '  cfg file version =', cfgver
            write(*,*)'Delete process.cfg file and re-run tscascade'//
     &                ' to generate a new one'
            stop
          endif
          QCFGVER = .FALSE.
          goto 103

        else if( line(1:6).eq.'NUMSTK' ) then
          read(line(8:),*) n
          if( n.ne.MAXF ) then
            write(*,*) 'Number of substacks not equal to MAXF'
            write(*,*) 'number =', n, '     MAXF =', MAXF
            write(*,*) 'Edit process.cfg file then re-run'
            stop
          endif
          read(ind3,*) (numstk(i),i=1,n)
          if( iprint.ge.1 ) then
            write(*,*)'NUMSTK found and read in as follows:'
            write(*,'(20i4)') (numstk(i),i=1,MAXF)
          endif
          goto 103

        else if( line(1:6).eq.'VARTOL' ) then
          read(line(8:),*) vartol
          if( iprint.ge.1 ) then
            write(*,*)'VARTOL found with value :', vartol
          endif
          goto 103

        else if( line(1:6).eq.'OWRITE' ) then
          read(line(8:),*) ORITE
          QORITE = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'OWRITE found with value :', ORITE
          endif
          goto 103

        else if( line(1:6).eq.'PROTYP' ) then
          call word1( line(8:), word, lw )
          if( index(word,'MT').ne.0 ) then
            mt  = 'M'
            HSG = .FALSE.
          elseif( index(word,'GDS').ne.0 ) then
            mt  = 'G'
            HSG = .FALSE.
          elseif( index(word,'HSG').ne.0 ) then
            HSG = .TRUE.
          else
            write(*,*)'Unrecognized processing type =', word(1:lw)
            write(*,*)'Edit process.cfg file and re-run'
            stop
          endif
          QPROTYP = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'PROTYP found with value :', word(1:lw)
          endif
          goto 103

        else if( line(1:6).eq.'REMOTE' ) then
          call word1( line(8:), word, lw )
          if( index(word,'H').ne.0 ) then
            MAGREMOTE = .TRUE.
          elseif( index(word,'E').ne.0 ) then
            MAGREMOTE = .FALSE.
          else
            write(*,*)'Unrecognized remote type =', word(1:lw)
            write(*,*)'Edit process.cfg file and re-run'
            stop
          endif
          QREMOTE = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'REMOTE found with value :', word(1:lw)
          endif
          goto 103

        else if( line(1:6).eq.'DECLIN' ) then
          read(line(8:),*) dec
          QDECLIN = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'DECLIN found with value :', dec
          endif
          goto 103

        else if( line(1:6).eq.'ROTANG' ) then
          read(line(8:),*) azimuth
          QROTANG = .FALSE.
          AZIMSET = .TRUE.
          if( iprint.ge.1 ) then
            write(*,*)'ROTANG found with value :', azimuth
          endif
          goto 103

        else if( line(1:6).eq.'MAG_OR' ) then
          read(line(8:),*) MAG_OR
          QMAG_OR = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'MAG_OR found with value :', MAG_OR
          endif
          goto 103

        else if( line(1:6).eq.'P_NORM' ) then
          goto 103

        else if( line(1:6).eq.'STACK ' ) then
          read(line(8:),*) STACK
          QSTACK = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'STACK  found with value :', STACK
          endif
          if( STACK ) then
            stackans = 'Y'
          else
            stackans = 'N'
          endif
          goto 103

        else if( line(1:6).eq.'JJANAL' ) then
          read(line(8:),*) JJANAL
          QJJANAL = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'JJANAL found with value :', JJANAL
          endif
          if( JJANAL ) then
            jjans = 'Y'
          else
            jjans = 'N'
          endif
          goto 103

c...keywords not required in tscascade processing
        elseif( line(1:6).eq.'MAXTYP' ) then
          goto 103
        elseif( line(1:6).eq.'NTYPE ' ) then
          goto 103
        elseif( line(1:6).eq.'WEIGHT' ) then
          goto 103
        elseif( line(1:6).eq.'COHMIN' ) then
          goto 103
        elseif( line(1:6).eq.'COHMAX' ) then
          goto 103
        elseif( line(1:6).eq.'UNITS ' ) then
          goto 103

        else
          write(*,*) ' '
          write(*,*) 'Unrecognised keyword in process.cfg file'
          write(*,*) 'First six characters =', line(1:6)
          write(*,*) ' '
        endif
103   continue
104   if( iprint.ge.0 ) then
        write(*,'(a)')'Read in parameters from process.cfg file'
      endif
      close(unit=ind3)
            
110   continue

      if( QCFGVER ) then
        write(*,*)'Processing configuration file without '//
     &            'version number'
        write(*,*)'Delete process.cfg file and re-run tscascade'//
     &            ' to generate a new one'
        stop
      endif

      if( ASK ) then
        QSTACK  = .TRUE.
        QJJANAL = .TRUE.
        QCOHMIN = .TRUE.
        QCOHMAX = .TRUE.
        QROTANG = .TRUE.
        QDECLIN = .TRUE.
        QPROTYP = .TRUE.
        QORITE  = .TRUE.
        QMAXTYP = .TRUE.
        QNTYPE  = .TRUE.
        QWEIGHT = .TRUE.
      endif

c-------------------------------------------------------------------------------

      ans = 'Y'
      call yin( 'Remote-reference processing (y/n)?', ans )
      if( ans.eq.'Y' ) then
        RR = .TRUE.
      else
        RR = .FALSE.
      endif

2     if( QPROTYP ) then
        call tin( 'MT, GDS or HSG processing', mt )
        call shiftup( mt )
        if( mt.eq.'H' ) then
          HSG = .TRUE.
          indat1 = 'hs2hsgxf'
        else if( mt.eq.'G' ) then
          HSG = .FALSE.
        else if( mt.eq.'M' ) then
          HSG = .FALSE.
        else
          write(*,*)'Erroneous input, M G or H only please'
          goto 2
        endif
      endif

      if( QSTACK ) then
        call yin( 'Go through STACK (y/n)?', stackans )
        if( stackans.eq.'Y' ) then
          STACK = .TRUE.
        else
          STACK = .FALSE.
        endif
      endif
 
      JJANAL = STACK
      if( QJJANAL .and. STACK ) then
        call yin( 'Jones/Joedicke substack analysis (y/n)?', jjans )
        if( jjans.eq.'Y' ) then
          JJANAL = .TRUE.
        else
          JJANAL = .FALSE.
        endif
      endif


      do i=1,MAXF2
        jf(i) = 0
        is(i) = 0
        ie(i) = 0
        iv(i) = 0
        w(i)  = 0.
        x(i)  = 0.
        it(i) = 0
        do j=1,7
          a(i,j)  = 0.
          b(i,j)  = 0.
          c(i,j)  = 0.
          g(i,j)  = 0.
          h(i,j)  = 0.
          xi(i,j) = 0.
          xj(i,j) = 0.
          xk(i,j) = 0.
          xl(i,j) = 0.
          xm(i,j) = 0.
          xn(i,j) = 0.
          zk(i,j) = 0.
          zl(i,j) = 0.
          zm(i,j) = 0.
          zn(i,j) = 0.
        end do
        do j =1,49
          y(i,j) = 0.
          z(i,j) = 0.
        end do
      end do
      do j = 1, MAXF
        nfw(j) = 0
      end do
      l   = 0
c
c  Read in the name of the input data file 

      tswindow = 'tbt101ss'
  300 call tin( 'Give window (extension .ts assumed)', tswindow )

      proj = tswindow(1:3)
      stat = tswindow(1:6)
      title(1:12) = tswindow

      if( iprint.ge.1 ) write(*,'(2a)')'Station =>', stat

      if( RR ) then
        tswindowrr = tswindow
        call tin( 'Give remote-reference window (no extension)', 
     &             tswindowrr )
        title(9:20) = ' +'//tswindowrr
        statrr = tswindowrr(1:6)
      else
        tswindowrr = tswindow
      endif

      if( iprint.ge.1 ) then
        write(*,'(a)')'Data files to open ==>'
        write(*,'(2a)')'file1 =>', tswindow
        if( RR ) then 
          write(*,'(2a)')'file2 =>', tswindowrr
        else
          write(*,'(a)')'No file2'
        end if
      end if

c-------------------------------------------------------------------------------
c     open output cascade file - only for STACK = .TRUE.

      if( STACK ) then

c...open cascade file
        fout = stat//'/'//tswindow(4:8)//tswindowrr(4:6)//'.cas'
700     open(unit=outd2,status='NEW',file=fout,form='UNFORMATTED',
     &       err=710)
        write(*,'(2a)')'Opening cascade file: ', fout
        go to 720
710     ORITE = .TRUE.
        if( iprint.ge.0 .and. QORITE ) then
          write(*,*)'tscascade: Error ***** ==> cascade file ',
     &            fout, ' already exists.'
          ans = 'Y'
          call yin( 'Do you wish to overwrite it (y/n)?', ans )
          if( ans.eq.'Y' ) then
            ORITE = .TRUE.
          else
            ORITE = .FALSE.
          endif
        endif
        if (ORITE) then
          open(unit=outd2, status='OLD', file=fout, form='UNFORMATTED' )
        else
          call tin ('Give a new filename', fout )
          go to 700
        endif
720     continue

c..write comment to cascade file

        line = '# tscascade: version: '//ver//'    precision: '//
     &          progprec
        write(outd2) line
        line = '# date: '//date_string
        write(outd2) line

        line = '#'
        write(outd2) line
        line = '# input  ts window: '//tswindow
        write(outd2) line
        if( RR ) then 
          line = '# remote ts window: '//tswindowrr
        else
          line = '# remote ts window: no remote'
        endif
        write(outd2) line
        line = '#'
        write(outd2) line

c...write process.cfg file to cascade file
        line = '# process.cfg file:---'
        write(outd2) line
        line = '#'
        write(outd2) line
        open(unit=7, file='process.cfg', status='OLD')
        do while( .TRUE. )
          read(7,'(a)',end=725) line
          write(outd2)'# '//line(1:78)
        enddo
725     line = '#'
        write(outd2) line
        line = '#'
        write(outd2) line

      endif

c-------------------------------------------------------------------------------
c Open the data file and Read in the window and header file information.


      form = '?'
      write(*,'(2a)')'Opening input file for window ', tswindow
      call ts_open( ind1, tswindow, indat1, .FALSE., .FALSE. )
      write(*,'(2a)')'Opened data file =>',indat1

c...copy comment block to .cas file
      if( STACK ) then
        call ts_copycomm(ind1, form, outd2, 'BINARY         ' )
      endif

c...read info block
      call ts_rdinfo(ind1)

      if( iprint.ge.1 ) then
        write(*,*)'Information block from input file...'
        write(*,*)'STATION   :', stat
        write(*,*)'INSTRUMENT:', instrument
        write(*,*)'WINDOW    :', window
        write(*,*)'LATITUDE  :', lat
        write(*,*)'LONGITUDE :', long
        write(*,*)'ELEVATION :', elev
        write(*,*)'COORD_SYS :', azdir
        write(*,*)'DECLIN    :', declination
        write(*,*)'FORM      :', form
        write(*,*)'FORMAT    :', format
        write(*,*)'NCHAN     :', nchan
        do i = 1, nchan
          write(*,*)'Channel......:', i
          write(*,*)'CHAN    :', chan(i)
          write(*,*)'SENSOR  :', sensor(i)
          write(*,*)'AZIM    :', azim(i)
          write(*,*)'GAIN    :', gain(i)
          write(*,*)'UNITS   :', units(i)
        enddo
        write(*,*)'STARTTIME :', starttime
      endif

c..calculate magnetometer orientation and compare with stated directions
      basehx = misdat
      basehy = misdat
      do i = 1, nchan
        if( chan(i).eq.'HX' ) basehx = baseline(i)
        if( chan(i).eq.'HY' ) basehy = baseline(i)
      enddo
      if( basehx.eq.misdat .or. basehy.eq.misdat ) then
        write(*,'(a)')'Cannot calculate mag orientation: '//
     &                ' baselines not found'
        magor = 0.
      else
        magor = atan2d( basehy, basehx )
        write(*,'(a,f8.2)')'Magnetic orientation from baselines =', 
     &                    magor
      endif


c..format common
      form1 = form

c..ts_stat common
      stattmp = stat
      inst1   = instrument

c..ts_pos common
      lat1  = lat
      long1 = long
      elev1 = elev

c..ts_data common
      if( tunits.eq.'s' ) delta = 1./delta
      delta1  = delta
      tunits1 = tunits
      nchan1  = nchan

c..ts_azim common
      do i = 1, 5
        azim1(i) = azim(i)
      enddo
      azimhx = azim(1)
c...ASSUMES HERE THAT FOR NCHAN=5 THEN ORDER = Hx, Hy, Hz, Ex, Ey
c                         NCHAN=4 THEN ORDER = Hx, Hy, Ex, Ey
      if( nchan.eq.4 ) then
        azimex = azim(3)
        azimey = azim(4)
      elseif( nchan.eq.5 ) then
        azimex = azim(4)
        azimey = azim(5)
      endif
      azdir1 = azdir
      declin1 = declination

c..ts_start common
      stime1 = starttime
      etime1 = endtime
      call date( 0, stime1, byy, bmm, bdd, bhh, bmn, bss )
      byy1 = byy
      bmm1 = bmm
      bdd1 = bdd
      bhh1 = bhh
      bmn1= bmn
      bss1 = bss


      if( RR ) then 
        form = '?'
        write(*,'(2a)')'Opening input file for window ', tswindowrr
        call ts_open( ind2, tswindowrr, indat2, .FALSE., .FALSE. )
        call ts_rdinfo(ind2)

c..calculate magnetometer orientation and compare with stated directions
      basehx = misdat
      basehy = misdat
      do i = 1, nchan
        if( chan(i).eq.'HX' ) basehx = baseline(i)
        if( chan(i).eq.'HY' ) basehy = baseline(i)
      enddo
      if( basehx.eq.misdat .or. basehy.eq.misdat ) then
        write(*,'(a)')'Cannot calculate mag orientation: '//
     &                ' baselines not found'
        magor = 0.
      endif
      magor2 = atan2d( basehy, basehx )
      write(*,'(a,f6.2)')'Remote magnetic orientation from baselines =', 
     &                    magor2
   
c..format common
        form2 = form
      
c..ts_stat common
        statrr = stat
        stat   = stattmp
        inst2   = instrument

c..ts_pos common
        lat2  = lat
        long2 = long
        elev2 = elev

c..ts_data common
        if( tunits.eq.'s' ) delta = 1./delta
        delta2  = delta
        tunits2 = tunits
        nchan2  = nchan
        nchan   = nchan1

c..ts_azim common
        do i = 1, 5
          azim2(i) = azim(i)
        enddo
        azimrx = azim(1)
        azdir2 = azdir
        declin2 = declination

c..ts_start common
        stime2 = starttime
        etime2 = endtime
        call date( 0, stime2, byy, bmm, bdd, bhh, bmn, bss )
        byy2 = byy
        bmm2 = bmm
        bdd2 = bdd
        bhh2 = bhh
        bmn2= bmn
        bss2 = bss

        if(delta1.ne.delta2) then
          write(*,*) 'Sampling rates differ between local and ',
     >               'remote data files.'
          stop
        end if

      else

        nchan2  = nchan1
        azimrx  = azimhx
        azdir2  = azdir1
        declin2 = declin1
        magor2  = magor

      end if

      go to 320
  310 write (*,*)' Error opening DATA INPUT file, re-enter.'
      go to 300
  320 continue
c
c--Get instrument number(s) from the windows file
c
      fout = 'windows.'//proj
      goto 405
  400 call tin (' Give the project windows file', fout )
  405 open ( inc, file=fout, status='old', form='formatted', err=410 )
      go to 420
  410 write (*,'(a)') 'tscascade: Error opening WINDOWS file'
      goto 400
  420 continue

      call getwindow( inc, tswindow, instr1, nchan1, stime1, etime1 )

      if( inst1.ne.instr1 ) then
        write(*,*)'Wrong instrument number in windows file'
        write(*,*)'Instrument number in time series file:', inst1,
     &            '    in windows file:', instr1
        write(*,*)'instrument number reset to ', inst1
        instr1 = inst1
        write(*,*)'endtime taken from file ', etime1
      endif

      if( iprint.ge.1 ) then
        write(*,*)'Window start time: ', stime1
        write(*,*)'       end   time: ', etime1
      endif

      sdur = tdiff( stime1, etime1 )
      if( RR ) then
        rewind( inc )
        call getwindow(inc, tswindowrr, instr2, nchan2, stime2, etime2)

        if( inst2.ne.instr2 ) then
          write(*,*)'Wrong instrument number in windows file'
          write(*,*)'Instrument number in time series file:', inst2,
     &              '    in windows file:', instr2
          write(*,*)'instrument number reset to ', inst2
          instr2 = inst2
          write(*,*)'endtime taken from file ', etime2
        endif

      else
        tswindowrr = tswindow
        instr2 = instr1
        nchan2 = nchan1
        stime2 = stime1
        etime2 = etime1
      end if
      close(inc)


      if( nchan1.eq.3 ) then
        if( mt.eq.'M' ) then
          write(*,*)'>>>>>Only 3 channels found: GDS processing to',
     &              ' be undertaken'
          mt = 'G'
        endif
      else if( nchan1.eq.4 ) then
        if( iprint.ge.1 ) write(*,*)'4 channels found'
      else if( nchan1.eq.5 ) then
        if( iprint.ge.1 ) write(*,*)'5 channels found'
      else
        write(*,*)'Wierd number of channels; not 3 or 4 or 5 -- ',
     &            'don''''t know what to do now!'
        stop
      endif
c
c---nchan = total number of channels recorded on the data file.
c   delta is the sampling rate in Hz
c
      if( iprint.ge.1 ) then
        write(*,*) 'Sampling rate in Hz =', delta 
      end if
      di = 1.0/delta
      do i = 1, MAXF2
        period(2*i-1) = (32.*di*(2.**(I-1)))/8.
        period(2*i)   = (32.*di*(2.**(I-1)))/6.
      end do

900   if( stime1.gt.stime2 ) then
        stime = stime1
      else
        stime = stime2
      end if
      if( iprint.ge.0 ) then
        write(*,'(a)') 'Give required start time           >'//stime
      end if
      call tin ( ' ', stime )
      call date( 0, stime, yy, mm, dd, hh, min, ss )
      if( iprint.ge.2 ) then
        write(*,*)'stime=',stime
        write(*,*)' yy, mm, dd, hh, min =>', yy, mm, dd, hh, min, ss
      end if

      call chkdat( yy, mm, dd, hh, min, ss, daterr )
      if( daterr .gt. 0 ) go to 900

      if ( yy .ne. byy ) then
        write(*,'(a, i4)')' Requested wrong year!  Data block year is',
     &                      byy
        go to 900
      end if


905   if( etime1.lt.etime2 ) then
        etime = etime1
      else
        etime = etime2
      end if
      if( iprint.ge.0 ) then
        write(*,'(a)') 'Give required end   time           >'//etime
      end if
      call tin ( ' ', etime )
      call date( 0, etime, yy, mm, dd, hh, min, ss )
      if( iprint.ge.2 ) then
        write(*,*)'etime=',etime
        write(*,*)' yy, mm, dd, hh, min =>', yy, mm, dd, hh, min, ss
      end if

      call chkdat ( yy, mm, dd, hh, min, ss, daterr )
      if( daterr .gt. 0 ) go to 905


      call date( 0, stime, yy, mm, dd, hh, min, ss )


c...sdur is the duration in seconds from the start time to the end
      sdur = tdiff( stime, etime )

      dday  =  sdur/(60*60*24)
      dhour =  mod(sdur,60*60*24)/(60*60)
      dmin  =  mod(sdur,60*60)/60
      dsec  =  mod(sdur,60)

      if( iprint.ge.0 ) then
        write( dur, '(i4.4,a1,i2.2,a1,i2.2)') 
     &           dday, '/', dhour, '/', dmin
        write( *, '(2a)') 'Duration (dddd/hh/mm) : ', dur
      end if

      if(dday.eq.0 .and. dhour.eq.0 .and. dmin.eq.0 .and. dsec.eq.0)then
        dday  =  sdur/(60*60*24)
        dhour =  mod(sdur,60*60*24)/(60*60)
        dmin  =  mod(sdur,60*60)/60
        dsec  =  mod(sdur,60)
      endif


c...calculate the total number of seconds of data to be analysed.

      tsec = dday*86400 + dhour*3600 + dmin*60 + dsec

c...totn is the total number of entries required for analysis.
c
      totn =  float(tsec) * delta 
      nmax = ifix(totn)

      if( iprint.ge.1 ) then 
        call iin( 'Total number of data requested', nmax )
      else
        write(*,'(a,i8)') 'Total number of data requested:', nmax
      end if


c....skip data for data start times < stime
      if( stime1.lt.stime ) then
        nskip = tdiff( stime1, stime ) * delta
        write(*,*)'Skipping', nskip, '  data for local  file'
        call ts_skip( ind1, nskip )
      endif
      if( RR ) then
        if( stime2.lt.stime ) then
          nskip = tdiff( stime2, stime ) * delta
          write(*,*)'Skipping', nskip, '  data for remote file'
          call ts_skip( ind2, nskip )
        endif
      endif


c-------------------------------------------------------------------------------
c     write out co-ordinates and request direction for rotation

      write(*,'(a)')'Co-ordinates of acquisition ==>'
      write(*,*)'LOCAL : Hx =', azimhx, '  Ex =', azimex,
     &     '  Ey =', azimey
      write(*,*) 'Coord sys: ', azdir1, 'decl =', declin1, 
     &           '   magorl =', magor
      if( RR ) then
        write(*,*)'REMOTE: Rx =', azimrx
        write(*,*) 'Coord sys: ', azdir2, 'decr =', declin2, 
     &           '   magorr =', magor2
      endif


c...request if correction to be applied for magnetometer orientation
      if( QMAG_OR ) then
        MAG_OR = .TRUE.
        call lin( 'Correct for magnetometer orientation using '//
     &            'baselines', MAG_OR )
      endif

c...get declination if site in magnetic co-ordinates and not read in
c   from time series
      if( QDECLIN .and. declin1.eq.-999. ) then
        if( index(azdir1, 'MAGNETIC').ne.0 .or.
     &      index(azdir2, 'MAGNETIC').ne.0 ) then
          call iin( 'Give declination', dec )
          declin1 = dec
          declin2 = dec
        endif
      endif


c...request co-ordinate system
      if( QROTANG ) then
        if( .NOT.AZIMSET ) azimuth = 0
        call iin( 'Desired co-ordinate system in TRUE NORTH', azimuth)
      endif

      write(*,*) 'Desired co-ordinate system in TRUE NORTH :', azimuth

c...rotation angles required
      angex = float(azimuth - azimex) - declin1
      angey = float(azimuth - azimey) - declin1 + 90.
      if( MAG_OR ) then
        anghx = float(azimuth - azimhx) - declin1 - magor
        angrx = float(azimuth - azimrx) - declin2 - magor2
      else
        anghx = float(azimuth - azimhx) - declin1
        angrx = float(azimuth - azimrx) - declin2
      endif

c      if( iprint.ge.0 ) then
        write(*,'(a)')'Rotation angles to be applied ==>'
        write(*,'(a,f10.1)')'Ex =', angex
        write(*,'(a,f10.1)')'Ey =', angey
        write(*,'(a,f10.1)')'Hx =', anghx
        write(*,'(a,f10.1)')'Rx =', angrx
c      endif

c-------------------------------------------------------------------------------
 
      PI = 3.14159265359
      DZ = DI
 
      NPT = 0
      IB  = 4
      IW  = 0
      ID  = 0

c...calculate the frequencies at the sixth (W) and the eighth (X) harmonic.
c   "32" is here as the data are taken in as 32-pt data lengths

      do I=1,MAXF2
        IT(I) = 1
        W(I) = 6./(32.*DZ*(2.**(I-1)))
        X(I) = 8./(32.*DZ*(2.**(I-1)))
      enddo
 
      DO IO=1,32
        JV=IO-1
        XJV=JV
        T=.5*(1.-COS(2.*PI*XJV/32.))
        O(IO) =  T*COS(12.*PI*XJV/32.)
        P(IO) = -T*SIN(12.*PI*XJV/32.)
        Q(IO) =  T*COS(16.*PI*XJV/32.)
        R(IO) = -T*SIN(16.*PI*XJV/32.)
      enddo


C******************** DATA GENERATION **********************************
    7 CONTINUE

c...read in data from local site 
      nchan = nchan1
      form  = form1
      call ts_in1(ind1, dat)
      if( dat(1).eq.EOF ) then
        write(*,*)'EOF input on local file'
        goto 30
      endif
      if( iprint.ge.3 ) then
        write(*,'(a,i6,a,5f10.1)') 'Input dat  ', npt+1,' =>', 
     &                             (dat(i),i=1,nchan1)
      endif

c...read in data from remote site 
      if( RR ) then
        nchan = nchan2
        form  = form2
        call ts_in1(ind2, dat2)
        if( dat2(1).eq.EOF ) then
          write(*,*)'EOF input on remote file'
          goto 30
        endif
        if( iprint.ge.3 ) then
          write(*,'(a,i6,a,5f10.1)') 'Input dat2 ', npt+1,' =>', 
     &               (dat2(i),i=1,nchan2)
        endif
        nchan = nchan1
      else
        do i = 1, nchan
          dat2(i) = dat(i)
        enddo
      endif


      if( STACK .and. npt.eq.0 ) then

c...write azimuthal information to cascade file
        line = '#'
        write(outd2) line
        line = '# Azimuthal information of cascade decimates'
        write(outd2) line
        write(line,'(a,i3)' ) '# azimuth    : ', azimuth
        write(outd2) line
        line = '# this azimuth is relative to TRUE NORTH if either the'
        write(outd2) line
        line = '#  1) data were recorded in TRUE NORTH, or'
        write(outd2) line
        line = '#  2) data were recorded in MAGNETIC NORTH and the '
        write(outd2) line
        line = '#     correct declination has been applied'
        write(outd2) line
        line = '#'
        write(outd2) line
        write(line,'(a,f7.1)' ) '# Ex rotation: ', angex
        write(outd2) line
        write(line,'(a,f7.1)' ) '# Ey rotation: ', angey
        write(outd2) line
        write(line,'(a,f7.1)' ) '# Hx rotation: ', anghx
        write(outd2) line
        if( RR ) then
          write(line,'(a,f7.1)' ) '# Rx rotation: ', angrx
          write(outd2) line
        endif
        line = '#'
        write(outd2) line
        write(line,'(a,i3)' ) '# declination: ', declin1
        write(outd2) line
        write(line,'(a,a)' ) '# coord_sys  : ', azdir1
        write(outd2) line
        write(line,'(a,i3)' ) '# Ex azimuth : ', azimex
        write(outd2) line
        write(line,'(a,i3)' ) '# Ey azimuth : ', azimey
        write(outd2) line
        write(line,'(a,i3)' ) '# Hx azimuth : ', azimhx
        write(outd2) line
        if( RR ) then
          write(line,'(a,a)' ) '# Rx co_ord  : ', azdir2
          write(outd2) line
          write(line,'(a,i3)' ) '# Rx azimuth : ', azimrx
          write(outd2) line
        endif
        line = '#'
        write(outd2) line

c...write header information to the `.cas' file:
        write(outd2) ver
        write(outd2) progprec
        write(outd2) tswindow,   instr1, nchan1, lat1, long1, elev1
        write(outd2) tswindowrr, instr2, nchan2, lat2, long2, elev2
        write(outd2) stime, delta, nmax, azimuth

        fout = stat//'/'//tswindow(4:8)//tswindowrr(4:6)//'.out'
730     write(*,'(2a)')'Opening output file: ', 
     &                 fout(1:lnblnk(fout))
        open(unit=7, status='NEW', file=fout, err=740 )
        go to 750
740     ORITE = .TRUE.
        if( iprint.ge.0 .and. QORITE ) then
          write(*,*)'tscascade: Error opening output file ', fout,
     &            ' -- probably already exist?'
          ans = 'Y'
          call yin( 'Do you wish to overwrite it (y/n)?', ans )
          if( ans.eq.'Y' ) then
            ORITE = .TRUE.
          else
            ORITE = .FALSE.
          endif
        endif
        if (ORITE) then
          open(unit=7, status='OLD', file=fout )
        else
          call tin ('Give a new filename', fout )
          go to 730
        endif
750     continue
      endif

c
c      Channel     LIMS        Cascade      Jones       HSG
c         1         Hx           Ex           Hx         Gx
c         2         Hy           Ey           Hy         Gy
c         3         Hz           Hx           Ex
c         4         Ex           Hy           Ey
c         5         Ey           Hz           Hz         Hz
c         6         Rx           Rx           Rx         Rx
c         7         Ry           Ry           Ry         Ry
c

c...assign data input
c...ASSUMES HERE THAT FOR NCHAN=5 THEN ORDER = Hx, Hy, Hz, Ex, Ey
c                         NCHAN=4 THEN ORDER = Hx, Hy, Ex, Ey
c                         NCHAN=3 THEN ORDER = Hx, Hy, Hz

      hx = dat(1)
      hy = dat(2)
      if( nchan.eq.3 .or. nchan.eq.5 ) then
        hz = dat(3)
        ex = dat(4)
        ey = dat(5)
      elseif( nchan.eq.4 ) then
        hz = 0.
        ex = dat(3)
        ey = dat(4)
      endif

      if( MAGREMOTE ) then
        rx = dat2(1)
        ry = dat2(2)
      else
        if( nchan2.eq.4 ) then
          rx = dat2(3)
          ry = dat2(4)
        elseif ( nchan2.eq.5 ) then
          rx = dat2(4)
          ry = dat2(5)
        elseif( nchan2.eq.2 ) then
          rx = dat2(1)
          ry = dat2(2)
        else
          write(*,*)'Cannot use remote: not enough channels'
          stop
        endif
      endif


c...check not equal to misdat: if equal to misdat, set all values to zero

      SET_ZERO = .FALSE.
      misdatl = misdat*0.99999
      misdatu = misdat*1.00001
      if( hx.gt.misdatl.and.hx.lt.misdatu) SET_ZERO = .TRUE.
      if( hy.gt.misdatl.and.hy.lt.misdatu) SET_ZERO = .TRUE.
      if( hz.gt.misdatl.and.hz.lt.misdatu) SET_ZERO = .TRUE.
      if( ex.gt.misdatl.and.ex.lt.misdatu) SET_ZERO = .TRUE.
      if( ey.gt.misdatl.and.ey.lt.misdatu) SET_ZERO = .TRUE.
      if( rx.gt.misdatl.and.rx.lt.misdatu) SET_ZERO = .TRUE.
      if( ry.gt.misdatl.and.ry.lt.misdatu) SET_ZERO = .TRUE.

      if( SET_ZERO ) then
        hx = 1.
        hy = 1.
        hz = 0.
        ex = 0
        ey = 0
        rx = 1.
        ry = 1.
      else
        hxl = hx
        hyl = hy
        hzl = hz
        exl = ex
        eyl = ey
        rxl = rx
        ryl = ry
      endif

c...perform rotations (if required) in time domain

      if( (anghx+angex+angey+angrx).ne.0. .and. iprint.ge.5 ) then
        write(*,*) 'Pre-rot  =>',hx,hy,ex,ey,rx,ry
      endif
      if( anghx.ne.0. ) then
        call rotdat( hx, hy, anghx, anghx )
      endif

      if( angex.ne.0. .or. angey.ne.0. ) then
        call rotdat( ex, ey, angex, angey )
      endif

      if( angrx.ne.0. ) then
        call rotdat( rx, ry, angrx, angrx )
      endif
      if( (anghx+angex+angey+angrx).ne.0. .and. iprint.ge.5 ) then
        write(*,*) 'Post-rot =>',hx,hy,ex,ey,rx,ry
      endif

c...write data into cascade arrays

      if( mt.eq.'M' ) then
        A(1,1) = ex
        A(1,2) = ey
      end if

      if( HSG ) then
        A(1,3) = hx + hy
        A(1,4) = hx + hy
        A(1,5) = hz
      else
        A(1,3) = hx
        A(1,4) = hy
        A(1,5) = hz
      endif

      A(1,6) = rx
      A(1,7) = ry

      if( iprint.ge.5 ) then
        write(*,*) 'Points to process ==>'
        write(*,*)(a(1,i),i=1,7)
      end if

      npt = npt+1
      if( iprint.eq.0 ) then
        if( 25000*(npt/25000).eq.npt ) then
          write(*,'(a,i10)') 'Number of points read... ', npt
        endif
      elseif( iprint.eq.1 ) then
        if( 10000*(npt/10000).eq.npt ) then
          write(*,'(a,i10)') 'Number of points read... ', npt
        endif
      elseif( iprint.ge.2 ) then
        if( 5000*(npt/5000).eq.npt ) then
          write(*,'(a,i10)') 'Number of points read... ', npt
        endif
      endif

C******************** DATA GENERATION **********************************
c...all uppercase code is from the original cascade routine of David Wight
c   I don't understand it, so don't ask me what's going on.

      CALL POINT7
C      ** FLAGSCAN **
    8 CONTINUE
      DO 11 K=1,10
        IF(IFLAG(K)-1) 11,9,11
    9   M = K
        GOTO 13
   11 CONTINUE

c     Loop back for more data.
c      IF(NPT-NMAX) 7,30,30
      if( npt.ge.nmax ) goto 30
      goto 7

C     ** SPECTRA **
   13 IF(IT(M)) 17,17,15
   15 IT(M) = 0
      IS(M) = 0
      IFLAG(M) = 0
      GO TO 8
   17 N = 1

c
c    M is the band counter.
c    Z for the eighth harmonic.
c    Y for the sixth harmonic.
c    The auto- and crosspowers are stored as follows
C
C  I  J    N
C  1  1    1     EXEX
C  1  2    2     EXEYr
C  1  2    3     EXEYi
C  1  3    4     EXHXr
C  1  3    5     EXHXi
C  1  4    6     EXHYr
C  1  4    7     EXHYi
C  1  5    8     EXHZr
C  1  5    9     EXHZi
C  1  6   10     EXRXr
C  1  6   11     EXRXi
C  1  7   12     EXRYr
C  1  7   13     EXRYi
C  2  2   14     EYEY
C  2  3   15     EYHXr
C  2  3   16     EYHXi
C  2  4   17     EYHYr
C  2  4   18     EYHYi
C  2  5   19     EYHZr
C  2  5   20     EYHZi
C  2  6   21     EYRXr
C  2  6   22     EYRXi
C  2  7   23     EYRYr
C  2  7   24     EYRYi
C  3  3   25     HXHX
C  3  4   26     HXHYr
C  3  4   27     HXHYi
C  3  5   28     HXHZr
C  3  5   29     HXHZi
C  3  6   30     HXRXr
C  3  6   31     HXRXi
C  3  7   32     HXRYr
C  3  7   33     HXRYi
C  4  4   34     HYHY
C  4  5   35     HYHZr
C  4  5   36     HYHZi
C  4  6   37     HYRXr
C  4  6   38     HYRXi
C  4  7   39     HYRYr
C  4  7   40     HYRYi
C  5  5   41     HZHZ
C  5  6   42     HZRXr
C  5  6   43     HZRXi
C  5  7   44     HZRYr
C  5  7   45     HZRYi
C  6  6   46     RXRX
C  6  7   47     RXRYr
C  6  7   48     RXRYi
C  7  7   49     RYRY
C     

c...test that there are reasonable values for fields
c   (i.e., that they are smaller than 10**15)
c   if not, then skip

      do i = 1, 7
	  if( g(m,i).ne.0. .and. h(m,i).ne.0. ) then
          if( alog10(abs(g(m,i))).gt.15. .or. 
     &        alog10(abs(h(m,i))).gt.15.) then
            write(*,*)'Bad spectra (>10^15) Skipped: band =', m,
     &                '  chan =', i, '  npt =', npt
            do j = 1, 7
              g(m,j) = 0.
              h(m,j) = 0.
            enddo
            goto 25
	    endif
        endif
      enddo



c...increase number of estimates available by one for this band
21    iv(m) = iv(m) + 1
      if( i.ge.10 ) then
	  write(*,*)'iv increased for band: ', m, '   to', iv(m)
	endif

c...compute 7x7 cross-spectral matrix from cascade decimates
c   and add to the existing summation
      do I=1,7
        do J=I,7
          Y(M,N) =  G(M,J)* G(M,I) +  H(M,I)* H(M,J) + Y(M,N)
          Z(M,N) = XI(M,J)*XI(M,I) + XJ(M,I)*XJ(M,J) + Z(M,N)
          if( JJANAL ) then
            yyy(iv(m),n,m) =  G(M,J)* G(M,I) +  H(M,I)* H(M,J)
            zz (iv(m),n,m) = XI(M,J)*XI(M,I) + XJ(M,I)*XJ(M,J)
          end if
          if( iprint.ge.4 ) then
            write(*,*)'M,N,Y,Z = ', M,N,Y(M,N),Z(M,N)
          end if 
            
          N = N+1

          IF (I.NE.J) THEN
            Y(M,N) =  H(M,I)* G(M,J) - G(M,I)* H(M,J) + Y(M,N)
            Z(M,N) = XJ(M,I)*XI(M,J) -XI(M,I)*XJ(M,J) + Z(M,N)
            if( JJANAL ) then
              yyy(iv(m),n,m) =  H(M,I)* G(M,J) - G(M,I)* H(M,J)
              zz (iv(m),n,m) = XJ(M,I)*XI(M,J) -XI(M,I)*XJ(M,J)
            end if
            if( iprint.ge.4 ) then
              write(*,*)'M,N,Y,Z = ', M,N,Y(M,N),Z(M,N)
            end if 

            N = N+1

          endif
        enddo
      enddo

24    continue


C     Call STACK each time numstk(m) sets are collected in band M.
c      IV is reset to 0 in STACK.
      if( mod(iv(m),numstk(m)).eq.0 ) then
        if( STACK ) call stack7( mt, m, iv, y, z )
      endif

25    IS(M) = 0
      IT(M) = 0
      IF(M-IB) 27,29,29
27    IS(M) = 0
29    IFLAG(M) = 0
      GO TO 8
 
 
c----------------------------------------------------------------------
c     ended data read-in - start processing of final cross-powers
 
   30 continue
 
      write(*, '(a,i8)')' Total number of data entries found NPT=',
     &                       npt

      if( STACK ) then
c     With m = -1 IV is reset to the total number of stacks in each band.
        m = -1
        call stack7( mt, m, iv, y, z )
      end if
c
c Set the coefficients of the digital anti-alias filter and remove its
c response later from the final spectral estimates.
c
      D(1) = 1.
      D(2) = 3.41421356
      D(3) = 4.87100924
      D(4) = D(2)
      D(5) = 1.

      E = 1.
      F = 1.
      DO 41 IP = 1,10
        XIV=IV(IP)
        YM = .125/E
        YN = .125/F
        IF(IV(IP)) 35,31,35
   31   DO IX=1,49
          Y(IP,IX) = 0.
          Z(IP,IX) = 0.
        END DO
        GO TO 37
   35   DO 36 IX=1,49
          Y(IP,IX) = YM*YM*Y(IP,IX)/XIV
          Z(IP,IX) = YN*YN*Z(IP,IX)/XIV
   36   CONTINUE
   37   YI = 0.
        YJ = 0.
        YK = 0.
        YL = 0.
        YG = 3./(32.*(2.**(IP-1)))
        YH = 4./(32.*(2.**(IP-1)))
        DO 39 IU=1,5
          XIU=IU-1
          YI = YI+D(IU)*COS(2.*PI*XIU*YG)
          YJ = YJ+D(IU)*SIN(2.*PI*XIU*YG)
          YK = YK+D(IU)*COS(2.*PI*XIU*YH)
          YL = YL+D(IU)*SIN(2.*PI*XIU*YH)
   39   CONTINUE
        E = E*SQRT(YI*YI+YJ*YJ)
        F = F*SQRT(YK*YK+YL*YL)
   41 CONTINUE

c     
c     Period in seconds;  shortest period in PERIOD(1)
c     IV = number of records averaged in each level.
c

      call jsubs7( mt, period, iv, y, z )

 9999 continue
      stop 'tscascade ended'
      END
