c  Program to calculate the crosscovariance for two long period data sets.
c     The first time series is used as a reference for the second.
c     The first should be at least twice as long as the second as the 
c     second is moved along the first and the crosscovariance calculated for 
c     each lag.
c     For example: Series1=450 data points starts at 870606120000
c                  Series2=150                       870606123000
c     If digitizing interval is 20 s, this gives 300 lags and if the two
c     time series are in sync max crosscovariance will occur at
c     lag = (30 min * 60/20) = 90 .  Actually 91 since 0 lag is counted as 1.
c     Data files must be in SCAN format.
c     Ron Kurtz, May 2, 1989.

c     New version adapted for Sun by  - Gervais Perron -
c     October 8, 1992.

c     Since some of the Uniras subroutines haven't been modified for Sun
c     yet, CRSCOV is forced to use as plotting device XVGR witch is a 
c     software installed on the network

c     The output consist of two files: 1 is a " .xvgr " file for XVGR use
c                                      
c                                      2 is a " .lags " file indicating the
c                                        lags at witch the crosscovariance is
c                                        the highest and the lowest
c
c     The given "root" filename must have the format specified as default
c     This means that the filename must have the '_' underscore character
c     in position number 6 and must be followed by 8 characters
c     The  'root' filename must have 14 characters long
c     The suggestion is that the 'root' filename should have this format
c
c                      001a1_vs_003b1
c
c     where the first five characters is the number of the first file used 
c     as reference followed by '_' followed by 'vs' followed by '_' followed
c     by the last five characters representing the file to be compered to the
c     reference file
c
c    Changed to tscov: v4.x format ts data: agj may 1997
c    changed to v5.0: agj oct 2000

      implicit none

      include 'version.inc' 
      include 'bdate.inc' 
      include 'ctrlblk.inc' 
      include 'infoblk.inc'
      include 'op_sys.inc'
      include 'defaults.inc'
      include 'io.inc' 

      integer MAXPNT
      parameter ( MAXPNT = 18000 )

      integer narg, getnarg
      character*10 cmdarg
      logical SINGLE, ASK

      real dat(5), frac

      real hx1(MAXPNT), hy1(MAXPNT), hz1(MAXPNT), ex1(MAXPNT), 
     &     ey1(MAXPNT)
      real hx2(MAXPNT), hy2(MAXPNT), hz2(MAXPNT), ex2(MAXPNT), 
     &     ey2(MAXPNT)
      real hxhx(MAXPNT), hyhy(MAXPNT), hzhz(MAXPNT), 
     &     exex(MAXPNT), eyey(MAXPNT), xmean

      real delta1, delta2

      real lat1, long1, lat2, long2, elev1, elev2 
      real azim1(5), azim2(5), declin1, declin2
      integer azimex, azimey, azimhx


      integer tsec, i, indat2, npt, np, npt1, npt2 
      integer ll, lag, iout, j, lnblnk, imax

      integer inst1, inst2, nchan1, nchan2

      integer tdiff, nskip

      character*1 sep
      character*8 tswindow, tswindowrr
      character*6 stat, statrr, stat1, stat2
      character*40 indat1, indat1
      character*6 progprec
      character*7 rout
      character*19 fout, flout
      character*40 fin1, fin2

      character*15 form1, form2
      character*19 stime, etime, stime1, etime1, stime2, etime2
      character*17 azdir1, azdir2
      character*5 tunits1, tunits2


c-------------------------------------------------------------------------------
c  Initialize

      call dirsep(sep)

      call precision( SINGLE )
      if( SINGLE ) then
        progprec = 'SINGLE'
      else
        progprec = 'DOUBLE'
      endif

c-------------------------------------------------------------------------------
c     get command-line arguments

      narg = getnarg()
      do i = 1, narg
        call getcmdarg( i, cmdarg)
        if( index(cmdarg,'h').ne.0 ) then
          write(*,*)'Permitted options:-'
          write(*,*)'                    -1  or -2  iprint level'
          write(*,*)'                    -ask       ask all questions'
          stop
        else if( index(cmdarg,'ask').ne.0 ) then
          ASK = .TRUE.
        else
          read(cmdarg,'(i2)') iprint
        endif
      enddo

      call iin( 'Give print level (-1/0/1/2/3)',iprint)

c-------------------------------------------------------------------------------
c     write banner for iprint >= 1

      if( iprint.ge.1 ) then
        write(*,*)' '
        write(*,*)' >>>>>>>>>>>>> TSCOV <<<<<<<<<<<<<<<<<<<'
        write(*,*)' '
        write(*,*)'           version = ', ver
        write(*,*)'build date = ', bdate
        write(*,*)' '
        if( SINGLE ) then
        write(*,*)'        SINGLE PRECISION version'
        else
        write(*,*)'        DOUBLE PRECISION version'
        endif
        write(*,*)' '
        write(*,*)'It is assumed that this program is being run from'
        write(*,*)'the database directory of the project '
        write(*,*)'(e.g. /local/mt/lims/nod)  '
        write(*,*)'and that all data files reside in subdirectories'
        write(*,*)'with the respective station names '
        write(*,*)'(e.g. /local/mt/lims/nod/nod001 nod002 etc.)'
        write(*,*)' '
        write(*,*)'window file is required (e.g. windows.nod) '
        write(*,*)' '
      else

        write(*,'(a)')'tscascade: ver='//ver//'   '//progprec//
     &                ' precision'
      end if


c-------------------------------------------------------------------------
c-------------------SECTION REGARDING THE FIRST FILE----------------------   
c-------------------------------------------------------------------------


c  Read in the name of the input data file 

      tswindow = 'tbt101ss'
  300 call tin( 'Give window (extension .ts assumed)', tswindow )
      stat = tswindow(1:6)

      tswindowrr = tswindow
      call tin( 'Give remote-reference window (no extension)', 
     &             tswindowrr )
      statrr = tswindowrr(1:6)

      if( iprint.ge.1 ) then 
        write(*,*)'Station    =>', stat
        write(*,*)'Station-RR =>', statrr
      endif


c Open the data file and Read in the window and header file information.

      form = '?'
      write(*,*)'Opening input file for window ', tswindow
      call ts_open( ind1, tswindow, indat1, .FALSE., .FALSE. )
      write(*,*)'Opened data file =>',indat1

c...read info block
      call ts_rdinfo(ind1)

c..format common
      form1 = form
c..ts_stat common
      stat1 = stat
      inst1 = instrument

c..ts_pos common
      lat1  = lat
      long1 = long
      elev1 = elev

c..ts_data common
      if( tunits.eq.'s' ) delta = 1./delta
      delta1  = delta
      tunits1 = tunits
      nchan1  = nchan

c..ts_azim common
      do i = 1, 5
        azim1(i) = azim(i)
      enddo
      azimhx = azim(1)
c...ASSUMES HERE THAT FOR NCHAN=5 THEN ORDER = Hx, Hy, Hz, Ex, Ey
c                         NCHAN=4 THEN ORDER = Hx, Hy, Ex, Ey
      if( nchan.eq.4 ) then
        azimex = azim(3)
        azimey = azim(4)
      elseif( nchan.eq.5 ) then
        azimex = azim(4)
        azimey = azim(5)
      endif
      azdir1 = azdir
      declin1 = declination

c..ts_start common
      stime1 = starttime
      etime1 = endtime



c Open the remote data file and read in the window and header file information.

      form = '?'
      write(*,*)'Opening input file for window ', tswindowrr
      call ts_open( ind2, tswindowrr, indat2, .FALSE., .FALSE. )
      write(*,*)'Opened data file =>',indat2

c...read info block
      call ts_rdinfo(ind2)

c..format common
      form2 = form

c..ts_stat common
      stat2 = stat
      inst2 = instrument

c..ts_pos common
      lat2  = lat
      long2 = long
      elev2 = elev

c..ts_data common
      if( tunits.eq.'s' ) delta = 1./delta
      delta2  = delta
      tunits2 = tunits
      nchan2  = nchan

c..ts_azim common
      do i = 1, 5
        azim2(i) = azim(i)
      enddo
      azdir2 = azdir
      declin2 = declination

c..ts_start common
      stime2 = starttime
      etime2 = endtime


c----------------------------------------------------------------------------
c     get start time and calculate duration and number of points
      if( stime1.gt.stime2 ) then
        stime = stime1
      else
        stime = stime2
      endif
      call tin('Give start time', stime )


      npt = MAXPNT
      call iin('Number of points', npt )

      if( npt.gt.MAXPNT ) then
        write(*,*)'npt reset to MAXPNT'
        npt = MAXPNT
      endif
      tsec = ifix( npt / delta )
      write(*,*)'Number of seconds to read in file1 =', tsec
      write(*,*)'Number of  points to read in file1 =', npt

      frac = 0.25
      call rin('Give fraction of this length for sliding', frac )

      npt1 = npt
      npt2 = npt*frac

c----------------------------------------------------------------------------
c     skip data if required

c....skip data for data start times < stime
      if( stime1.lt.stime ) then
        write(*,*) 'Skipping data for first file:'
        write(*,*) ' stime1 = ', stime1, '   stime = ', stime
        nskip = ifix(float(tdiff( stime1, stime )) * delta)
        write(*,*)'Skipping', nskip, '  data for local  file'
        form = form1
        call ts_skip( ind1, nskip )
      else
        write(*,*)'Skipping', 0, '  data for local  file'
      endif

      if( stime2.lt.stime ) then
        write(*,*) 'Skipping data for second file:'
        write(*,*) ' stime2 = ', stime2, '   stime = ', stime
        nskip = ifix(float(tdiff( stime2, stime )) * delta)
        write(*,*)'Skipping', nskip, '  data for remote file'
        form = form2
        call ts_skip( ind2, nskip )
      else
        write(*,*)'Skipping', 0, '  data for remote  file'
      endif


c----------------------------------------------------------------------------
c     read in data


c...read in data from local site 
      write(*,*)'Reading in data from file 1: npt1 =', npt1
      if( form1.eq.'ASCII' ) then
        do i = 1, npt1
          read( ind1,*) (dat(j),j=1,nchan)
          hx1(i) = dat(1)
          hy1(i) = dat(2)
          if( nchan1.eq.3 .or. nchan1.eq.5 ) then
            hz1(i) = dat(3)
            ex1(i) = dat(4)
            ey1(i) = dat(5)
          elseif( nchan1.eq.4 ) then
            hz1(i) = 0.
            ex1(i) = dat(3)
            ey1(i) = dat(4)
          endif
        enddo
      else
        do i = 1, npt1
          read( ind1 ) (dat(j),j=1,nchan)
          hx1(i) = dat(1)
          hy1(i) = dat(2)
          if( nchan1.eq.3 .or. nchan1.eq.5 ) then
            hz1(i) = dat(3)
            ex1(i) = dat(4)
            ey1(i) = dat(5)
          elseif( nchan1.eq.4 ) then
            hz1(i) = 0.
            ex1(i) = dat(3)
            ey1(i) = dat(4)
          endif
        enddo
      endif


c...read in data from remote site 
      write(*,*)'Reading in data from file 2: npt2 =', npt2
      if( form2.eq.'ASCII' ) then
        do i = 1, npt2
          read( ind2,*) (dat(j),j=1,nchan)
          hx2(i) = dat(1)
          hy2(i) = dat(2)
          if( nchan2.eq.3 .or. nchan2.eq.5 ) then
            hz2(i) = dat(3)
            ex2(i) = dat(4)
            ey2(i) = dat(5)
          elseif( nchan2.eq.4 ) then
            hz2(i) = 0.
            ex2(i) = dat(3)
            ey2(i) = dat(4)
          endif
        enddo
      else
        do i = 1, npt2
          read( ind2 ) (dat(j),j=1,nchan)
          hx2(i) = dat(1)
          hy2(i) = dat(2)
          if( nchan2.eq.3 .or. nchan2.eq.5 ) then
            hz2(i) = dat(3)
            ex2(i) = dat(4)
            ey2(i) = dat(5)
          elseif( nchan2.eq.4 ) then
            hz2(i) = 0.
            ex2(i) = dat(3)
            ey2(i) = dat(4)
          endif
        enddo
      endif

      write(*,*)'All data read in...'

c----------------------------------------------------------------------------
c  Remove the mean from the second time series.
c     The mean is removed from the first time series (in subroutine CROSSV)
c     in the segment being crosscovaried with the second series.  

      write(*,*)'Removing means from file2 data...'
      call meanrm( npt2, hx2, xmean )
      call meanrm( npt2, hy2, xmean )
      if( nchan2.eq.3 .or.nchan2.eq.5 ) then
        call meanrm( npt2, hz2, xmean )
      endif
      if( nchan2.ge.4 ) then
        call meanrm( npt2, ex2, xmean )
        call meanrm( npt2, ey2, xmean )
      endif

c  Calculate the crosscovariance of each channel for each lag
c  done for "np" lags

      np = npt1 - npt2 + 1

      write(*,*)'Computing Hx-Hx cross covariance...'
      call crossv(hx1, hx2, npt1, npt2, hxhx)
      open(unit=10,file='Hx-Hx')
      do i = 1, np
        write(10,*) float(i-1)/delta, hxhx(i)
      enddo
      close(unit=10)

      write(*,*)'Computing Hy-Hy cross covariance...'
      call crossv(hy1, hy2, npt1, npt2, hyhy)
      open(unit=10,file='Hy-Hy')
      do i = 1, np
        write(10,*) float(i-1)/delta, hyhy(i)
      enddo
      close(unit=10)

      if( nchan2.eq.3 .or.nchan2.eq.5 ) then
        write(*,*)'Computing Hz-Hz cross covariance...'
        call crossv(hz1, hz2, npt1, npt2, hzhz)
      endif
      if( nchan2.ge.4 ) then
        write(*,*)'Computing Ex-Ex cross covariance...'
        call crossv(ex1, ex2, npt1, npt2, exex)
        write(*,*)'Computing Ey-Ey cross covariance...'
        call crossv(ey1, ey2, npt1, npt2, eyey)
      endif

c  Initialize, ask and read the output filename
c
      rout = tswindow(4:6)//'_'//tswindowrr(4:6)

      call tin('Give output root filename', rout)

      ll = lnblnk( rout )
      fout = tswindow(1:6)//'/'//rout(1:ll)//'.xvgr'
      flout = tswindow(1:6)//'/'//rout(1:ll)//'.lags'
      

      np = npt1 - npt2 + 1
      
c  Searching for the highest and the lowest crosscovariance between the two
c  files and giving the corespondant lag number for each channel
c
      call fndmax(hxhx, np, imax )
      lag = imax
      call fndmax(hyhy, np, imax )
      call fndmax(hzhz, np, imax )
      call fndmax(exex, np, imax )
      call fndmax(eyey, np, imax )
c
c  Tell if the two files are in sync! and if not tell by how many seconds
c  they are apart
c
      call timdif(lag, stime1, stime2, fin1, fin2 )

c  Writing to the output file the header the results of the crosscovariance processing

      open(unit=iout, file=fout, form ='FORMATTED')
c
      do j = 1, np
        write(iout, 1000) j, hxhx(j), hyhy(j), hzhz(j), exex(j), eyey(j)
      enddo
 1000 format(1x, i4, 1p5e13.4 )

c  Closing the output unit
c
c      close ( unit = iout )
c      close ( unit = pout )

      goto 9999

 9001 write (*, '(a)') 'crscov: ERROR converting integers to
     & characters.'
      go to 9999
 9002 write (*, '(a)') 'crscov: ERROR converting characters
     & to integers.'

 9999 stop 'End of CRSCOV.'
      end

 

c  ===================================================================
      subroutine crossv( b, d, n1, n2, g )
c  Subroutine to find the cross-covariance between b and d.
c     The result is returned in g.
c     See Kanasewich, Time Sequence Analysis, page 70.
c     Ron Kurtz, May 2, 1989.

      parameter (MAXPNT=18000)
      real a(MAXPNT), b(*), d(*), g(*), pb, pd, sum, xmean

      integer n1, n2, m, j, k, kk, jk

      m = n1 - n2 + 1
      do 200 j =1, m
        g(j) = 0.0
        pb = 0.0
        pd = 0.0

c  Remove the mean from the section of b that is being covaried with d
        sum = 0.0
        kk = n2 + j - 1
        do 10 k = j, kk
          sum = sum + b(k)
   10   continue
        xmean = sum/float(n2)
        jk = 1
        do 20 k = j, kk
          a(jk) = b(k) - xmean
          jk = jk + 1
   20   continue

        do 100 k = 1, n2
          g(j) = g(j) + a(k)*d(k)
          pb = pb + a(k)*a(k)
          pd = pd + d(k)*d(k)
  100   continue
        g(j) = g(j)/(SQRT(pb*pd))
  200 continue
      return
      end


c===========================================================================
      subroutine fndmax(x, n, imax )
c  Find the largest and smallest crosscovariance value.

      real x(*), xmax, xmin, j

      integer n, imax, imin, ichan, pout

      data ichan/ 1 /

      xmax = -1.0E-30
      xmin =  1.0E+30

      do 100 j = 1, n
        if( x(j) .gt. xmax ) then
          imax = j
          xmax = x(j)
        endif
  
        if( x(j) .lt. xmin ) then
          imin = j
          xmin = x(j)
        endif
  100 continue

      write(*, '(a, i4, a, i4,x,i4)')' In channel ', ichan, 
     &   ' the max and min crosscovariance occur at lag ', imax,    imin
      write(pout, '(a, i2, a, i4,x,i4)')' In channel ', ichan, 
     &   ' the max and min crosscovariance occur at lag ', imax,    imin

      ichan = ichan + 1
      return
      end

c  ======================================================================
      subroutine timdif(lag, stime1, stime2, fil1, fil2 )

      real idel

      integer lag, idelt, l1, l2, idif
      integer yy, mm, dd, hh, mn, ss, sec1, sec2, secs
      integer inc, prterr, nchan

      character stime1*19, stime2*19, fil1*40, fil2*40

      common /contst/ inc, rdel, prterr, nchan

      l1 = INDEX(fil1, '.') + 3
      l2 = INDEX(fil2, '.') + 3

      call iin(' Give your best guess for the lag number', lag)
      idelt = 1/idel
      call iin(' Give digitizing interval (seconds)', idelt )

      read(stime1, '(6i2)')yy, mm, dd, hh, mn, ss
      call consec(sec1, yy, mm, dd, hh, mn, ss)
      read(stime2, '(6i2)')yy, mm, dd, hh, mn, ss
      call consec(sec2, yy, mm, dd, hh, mn, ss)
      
      idif = lag - ((sec2 - sec1)/idelt) - 1

      write(*,'(a)')' *************************************************
     &***************'
      if( idif .eq. 0 ) then
        write(*,'(a)')' The two time series are in sync!'
      else if( idif .lt. 0 ) then
        secs = -1*(idif*idelt)
        write(*,'(5a,i5,a  )')' Series in file ', fil2(1:l2), 
     &          ' lags file ', fil1(1:l1),' by ', secs, ' seconds.'
      else
        secs = idif*idelt
        write(*,'(5a,i5,a  )')' Series in file ', fil2(1:l2),
     &          ' leads file ', fil1(1:l1),' by ', secs, ' seconds.'
      endif
      write(*,'(a)')' *************************************************
     &***************'

       write(*,'(a)')' *************************************************
     &***************'
      if( idif .eq. 0 ) then
        write(*,'(a)')' The two time series are in sync!'
      else if( idif .lt. 0 ) then
        secs = -1*(idif*idelt)
        write(*,'(5a,i5,a  )')' Series in file ', fil2(1:l2), 
     &          ' lags file ', fil1(1:l1),' by ', secs, ' seconds.'
      else
        secs = idif*idelt
        write(*,'(5a,i5,a  )')' Series in file ', fil2(1:l2),
     &          ' leads file ', fil1(1:l1),' by ', secs, ' seconds.'
      endif
      write(*,'(a)')' *************************************************
     &***************'


      write(*,'(a)')' This run of CRSCOV compares ',fil1,' vs ',fil2,
     &          '.'

      return
      end


c  ======================================================================

      subroutine meanrm( n, x, xmean )

      real x(*), xmean, sum

      integer n, j      

      sum = 0.0
      do 100 j = 1, n
        sum = sum + x(j)
  100 continue
      xmean = sum/FLOAT( n )
      do 200 j = 1, n
        x(j) = x(j) - xmean
  200 continue
      return
      end
