      program tsplot
      implicit none
c--------------------------------------------------------------------
c     tsplot: plots time series in ts format
c
c--------------------------------------------------------------------

      include 'maxnum.inc'
      include 'maxchan.inc'
      include 'version.inc'
      include 'ctrlblk.inc' 
      include 'infoblk.inc' 
      include 'defaults.inc' 
      include 'io.inc' 
      include 'bdate.inc' 

c...set plotting page size parameters
      real xmax, ymax
      parameter ( xmax = 13.5, ymax = 9.0 )

c---------------------------------------------------------------------

      integer nticks, tstart, ht, daterr, nframe, nchr
      integer nplot, prterr, dchan, pstartgap
      integer yy, mm, dd, hh, min, ss, dday, dhour, dmin, dss, tsec
      integer byy, bmm, bdd, bhh, bmn, bss
      integer ntot, j, kk, k, i, nmean
      integer dec, idec, nskip
      integer colmap(16)

      integer   tdiff
      integer narg, lbox, instr
      integer nchanx, mwindow, swindow, nhour, nday, npt, ipen

      integer getnarg
      real vsep, tinc, hsep, xpp, totn, datmax, datmin, dpi
      real xscale, yp, xn, scvbar, yscale, vbar, vbar2, vplus, vminus
      real sc(MAXCHAN), xp(MAXNUM), cal(MAXCHAN), tot
      real calibs(MAXCHAN), cutsl(MAXCHAN), cutsh(MAXCHAN), xascle
      real data(MAXCHAN,MAXNUM), ddelta, xstart, ymean(MAXCHAN)
      real data1(MAXCHAN), xxmax(MAXCHAN), xxmin(MAXCHAN)
      real basehx, basehy, azimhx, azimex, azimey, azimuth
      real anghx, angex, angey
      real hx, hy, ex, ey
      real magor

      character tscale*6, indat*40, sep*1
      character tempst*90, compp*2, ttime*32
      character arg*10, proj*3, tswindow*20, dur*13, unit*5, cm*2
      character*19 stime, etime, nowtim, mtime, tst

      logical SECOND, MINUTE, HOUR, HOUR6, DAY, DAY7, BIMONTH, MONTH
      logical REPLOT, NEWFRM, ANS, LOCAL, MAGCORR, FIRSTPASS
      logical FIELD_COORDS


      data colmap/2,3,4,3,2,5,6,7,8,9,1,1,1,1,1,1/


c-------------------------------------------------------------------------------
c...set misdat here to an absurd number. May get overwritten when reading
c   data in

      misdat = DEFMISDAT

      FIRSTPASS = .TRUE.


c...set dots/inch resolution required
      dpi = 600.

c-------------------------------------------------------------------------------
c...get command arguments

      LOCAL = .FALSE.
      ANS = .TRUE.
      iprint = 0
      narg = getnarg()

      do 10 i = 1, narg
         call getcmdarg( i, arg)

c... -h or -help
         if( arg(1:2).eq.'-h' ) then
5          write(*,*)'Command line arguments are:-'
           write(*,*)'  h or help   - give this list'
           write(*,*)'  print n     - print level n'
           write(*,*)'  ps          - write PS file'
           write(*,*)'  local       - look for data in local directory'
           stop

c        --- ensure -ps option is not used
         elseif( arg(1:3).eq.'-ps' ) then

c
         elseif( arg(1:6).eq.'-local' ) then
           LOCAL = .TRUE.

         elseif( arg(1:6).eq.'-print' ) then
            i = i+1
            call getcmdarg( i, arg)
            read(arg,*,err=8) iprint
            ANS=.FALSE.
            goto 10
8           call getctrl_blk()
            ANS=.FALSE.
         else
           write(*,*)'Unknown argument given to program'
           goto 5
         endif
10    enddo

      if( ANS ) call getctrl_blk()
      prterr  = iprint

c-------------------------------------------------------------------------------

c...get directory separator ('/' for UNIX, '\' for DOS)
      call dirsep(sep)

      nframe = 1
      REPLOT = .FALSE.
      NEWFRM = .FALSE.
      SECOND = .FALSE.
      MINUTE = .FALSE.
      HOUR   = .FALSE.
      HOUR6  = .FALSE.
      DAY    = .FALSE.
      DAY7   = .FALSE.
      BIMONTH= .FALSE.
      MONTH  = .FALSE.
c
      if( iprint.ge.1 ) then
        write(*,*)' '
        write(*,*)' >>>>>>>>>>>>> TSPLOT <<<<<<<<<<<<<<<<<<<'
        write(*,*)' '
        write(*,*)'           version = ', ver
        write(*,*)'build date = ', ver
        write(*,*)' '
        write(*,*)'It is assumed that this program is being run from'
        write(*,*)'the database directory of the project '
        write(*,*)'  of the project.    For SUN: /lenz/lims/tbt)'
        write(*,*)'                     For PC:  c:\lims\tbt'
        write(*,*)' '
        write(*,*)'  All data files must reside in subdirectories with',
     &               ' their respective station'
        write(*,*)'  names:             e.g. /lenz/lims/alt/alt001 etc.'
        write(*,*)' '
        write(*,*)'  Windows and hardware database files are requested'
        write(*,*)'                  (e.g. windows.alt & hardware.alt) '
        write(*,*)' '
      endif
c
c Open input data file 
c
      form = '?'
      tswindow = 'tbt101ss'
      call ts_open(ind1,tswindow,indat,.TRUE.,LOCAL)
      if( iprint.ge.1 ) write(*,*)'Opened data file =>',indat

c Entry point for plotting another frame.
c
   50 continue
c
      call ts_rdinfo(ind1)

c...calculate azimuths on first pass
      if( FIRSTPASS ) then

        FIELD_COORDS = .TRUE.
        call lin('Plot data in field coordinates', FIELD_COORDS )

        if( FIELD_COORDS ) then
          anghx =  0.
          angex =  0.
          angey = 90.

        else
          do i = 1, nchan
            if( chan(i).eq.'HX' ) basehx = baseline(i)
            if( chan(i).eq.'HY' ) basehy = baseline(i)
          enddo
          if( basehx.eq.misdat .or. basehy.eq.misdat ) then
            write(*,'(a)')'Cannot calculate mag orientation: '//
     &                  ' baselines not found'
            magor = 0.
          endif
          magor = atan2d( basehy, basehx )
          write(*,'(a,f6.2)')'Magnetic orientation from baselines = ', 
     &                      magor
          MAGCORR = .TRUE.
          call lin( 'Apply this correction', MAGCORR )
          if( .not.MAGCORR ) magor = 0.

          write(*,*)'MEASURED : Hx =', azim(1), '  Ex =', azim(4),
     &       '  Ey =', azim(5)
          write(*,*) 'Coord sys: ', azdir, 'decl =', declination, 
     &             '   magor =', magor

          if( index(azdir, 'MAGNETIC').ne.0 ) then
            azimhx = azim(1) + declination + magor
            azimex = azim(4) + declination
            azimey = azim(5) + declination
          else
            azimhx = azim(1) + magor
            azimex = azim(4)
            azimey = azim(5)
          endif
          write(*,*)'TRUE     : Hx =', azimhx, '  Ex =', azimex,
     &        '  Ey =', azimey

          azimuth = 0.
          call rin( 'Give azimuth of plotted data', azimuth )

c...rotation angles to apply
          anghx = azimuth - azimhx
          angex = azimuth - azimex
          angey = azimuth - azimey + 90.
          write(*,*)'ROTATIONS : Hx =', anghx, '  Ex =', angex,
     &       '  Ey =', angey

        endif

        FIRSTPASS = .FALSE.
      endif

      proj  = tswindow(1:3)
      call date( 0, starttime, byy, bmm, bdd, bhh, bmn, bss )
      dchan = nchan
      if( tunits.eq.'Hz' ) then
        continue
      else if( tunits.eq.'s' ) then
        delta = 1./delta
      else
        write(*,*)'Time units not recognised >', tunits
        write(*,*)'only "s" or "Hz" permitted'
        stop
      endif

c...get start and end times of window and lunchbox number
      open(unit=ind2,file='windows.'//proj, status='old',err=70)
      call getwindow( ind2, tswindow(1:8), instr, nchanx, stime, etime )
      close(unit=ind2)
      lbox = instr
      goto 80
70    write(*,*)'  Cannot find window file: windows.'//proj,
     &                      '.  Header values used.'

c...get hardware info

80    indat = 'hardware.'//proj
      open( unit=ind2, file=indat, status='old', err=90 )
      call gethardware( ind2, lbox, dchan, calibs, cutsl, cutsh )
      close(unit=ind2)


c...apply calibs to cal: note difference in channel order!

      cal(1) = calibs(1)             ! Hx
      cal(2) = calibs(2)             ! Hy
      cal(3) = calibs(5)             ! Hz
      cal(4) = calibs(3)             ! Ex
      cal(5) = calibs(4)             ! Ey
      go to 95
90    continue
      write(*,*)'Cannot find hardware file: hardware.'//proj,
     &                      '.  Default values used.'
c...set default calibration values.
      cal(1) = 1.0
      cal(2) = 1.0
      cal(3) = 1.0
      cal(4) = 1.0
      cal(5) = 1.0
95    continue


c...if first frame then initialize the required event start time with
c   the start time of the data

      if ( nframe .eq. 1 ) then
        yy = byy
        mm = bmm
        dd = bdd
        hh = bhh
        min= bmn
        ss = bss
      end if

c  dchan = total number of channels recorded on the data file.
      nchan = dchan
      goto 110
100   call iin( 'Give the number of channels to be plotted', nchan)
      if (nchan .gt. MAXCHAN) then
        write(*,'(a,i3,a)')' Maximum dimensioned channels =',MAXCHAN,
     &                     ' Try again.'
        go to 100
      end if

      if (nchan .gt. dchan) then
        write(*,'(a,i3,a)')' Maximum number of channels recorded =',
     &                       dchan, 'Try again.'
        go to 100
      end if

c     delta is the sampling rate in Hz
c      call rin( 'Give the sampling rate in Hz ', delta )
c

c...request start time

110   if( instr.eq.-999 ) stime = starttime
      call date( 1, stime, yy, mm, dd, hh, min, ss)
      call nin( 'Give plot start time               >'//stime)
      call tin (' ', stime )
      call date( 0, stime, yy, mm, dd, hh, min, ss )
      call date( 1, stime, yy, mm, dd, hh, min, ss )

      call chkdat( yy, mm, dd, hh, min, ss, daterr )
      if( daterr.gt.0 ) go to 110

      if ( yy .ne. byy ) then
        write(*,'(a, i4)')' Requested wrong year!  Data block year is',
     &                      byy
        go to 110
      end if


c...compute time gap (in s) if plot start time < data start time
c   set to zero if plot start > data start

      pstartgap = tdiff( stime, starttime )
      if( iprint.ge.1 ) then
        write(*,'(a,i)')'Gap between plot start and data start (s) =',
     &                  pstartgap
      endif
      if( pstartgap.lt.0 ) pstartgap = 0


c...request end time

115   if( instr.eq.-999 ) etime = endtime
      call nin( 'Give plot end   time               >'//etime)
      call tin (' ', etime )
      call date( 0, etime, yy, mm, dd, hh, min, ss )
      call date( 1, etime, yy, mm, dd, hh, min, ss )
 
      call chkdat( yy, mm, dd, hh, min, ss, daterr )
      if( daterr .gt. 0 ) go to 115

      swindow = tdiff( stime, etime )
      mwindow = swindow/60
      dday  = mwindow/1440
      dhour = (mwindow - 1440*dday)/60
      dmin  = mwindow - dday*1440 - dhour*60
      dss   = swindow - 60*mwindow

      write( dur, '(i4.4,a1,i2.2,a1,i2.2,a1,i2.2)') 
     &           dday, '/', dhour, '/', dmin,'/',dss
      write( *, '(2a)') 'Duration (dddd/hh/mm/ss) : ', dur

c
c  Calculate the total number of seconds of data to be plotted.      
c
      tsec =   ((dday*24+dhour)*60 + dmin)*60 + dss
      if( iprint.ge.1 ) then
        write(*,'(a,2i)')'Number of seconds/data to plot = ', tsec, 
     &                 ifix(tsec*delta)
      endif

c...compute decimation factor for dpi dots/inch, 11 inch page
c   dec must be an integer multiple of seq_rec

      dec = ifix( ( tsec * delta ) / (dpi*11.) )
      if( dec.eq.0 ) dec = 1
      dec = seq_rec*(dec/seq_rec)
      if( dec.eq.0 ) dec = seq_rec
135   call iin( 'Give decimation factor', dec )
      if( mod(dec,seq_rec).ne.0) then
        write(*,'(a,i)')'decimation factor must be an integer '//
     &                  'multiple of seq_rec =', seq_rec
        dec = seq_rec*(dec/seq_rec)
        if( dec.eq.0 ) dec = seq_rec
        goto 135
      endif


c...select the units for the horizontal time axis.

      call date( 0, stime, yy, mm, dd, hh, min, ss )
      if( tsec .le. 10 ) then

c...tlen < 10 secs
        tinc = 1.0
        tstart = ss
        hsep = xmax/float(tsec)
        nticks = tsec + 1
        SECOND = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'SECOND: tinc, nticks, tstart = ', 
     &                       tinc, nticks, tstart
        endif

c...tlen < 100 secs
      else if( tsec .le. 100 ) then
        tinc = 10.
        tstart = ss
        hsep = (xmax*tinc)/float( tsec )
        nticks = (tsec/ ifix(tinc) ) + 1
        SECOND = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'SECOND: tinc, nticks, tstart = ', 
     &                       tinc, nticks, tstart
        endif

c...tlen < 1000 s   MINUTE = .TRUE.  tick every 1 min
      else if( tsec .le. 1000 ) then
        tinc = 1.
        tstart = min
        hsep = (xmax*60.)/float(tsec)
        nticks = (tsec/60.) + 1
        MINUTE = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'MINUTE: tinc, nticks, tstart = ', 
     &                       tinc, nticks, tstart
        endif

c...tlen < 2000 s   MINUTE = .TRUE.  tick every 2 mins
      else if( tsec .le. 2000 ) then
        tinc = 2.
        tstart = min
        hsep = (xmax*60.*tinc)/float(tsec)
        nticks = (tsec/(60.*tinc)) + 1
        MINUTE = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'MINUTE: tinc, nticks, tstart = ', 
     &                       tinc, nticks, tstart
        endif

c...tlen < 1 hour  MINUTE = .TRUE.  tick every 5 mins
      else if( tsec .le. 60*60 ) then
        tinc = 5.
        tstart = min
        hsep = xmax*60.*tinc/float( tsec )
        nticks = (tsec/(60.*tinc)) + 1
        MINUTE = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'MINUTE: tinc, nticks, tstart = ', 
     &                       tinc, nticks, tstart
        endif

c...tlen < 3 hours  MINUTE = .TRUE.  tick every 10 mins
      else if( tsec .le. 3*60*60 ) then
        tinc = 10.
        tstart = min
        hsep = xmax*60.*tinc/float( tsec )
        nticks = (tsec/(60.*tinc)) + 1
        MINUTE = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'MINUTE: tinc, nticks, tstart = ', 
     &                       tinc, nticks, tstart
        endif

c  tlen < 1 day     HOUR = .TRUE.  tick every hour
      else if( tsec .le. 24*60*60 ) then
        nhour = 24*dday + dhour + (min + dmin)/60
        if( mod(min+dmin,60).ne.0 ) nhour = nhour+1
        tinc = 1.
        tstart = hh
        hsep = xmax*tinc/float( nhour )
        nticks = nhour + 1
        HOUR = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'HOUR: nhour, nticks, tstart = ', 
     &                     nhour, nticks, tstart
        endif

c  tlen < 2 days    HOUR6= .TRUE.   tick every 6 hours
      else if( tsec .le. 2*24*60*60 ) then
        nhour = 24*dday + dhour + (min + dmin)/60
        if( mod(min+dmin,60).ne.0 ) nhour = nhour+1
        if( mod(nhour,6).ne.0 ) nhour = 6*(nhour/6) + 6
        tinc = 6.
        tstart = hh
        tstart = 6*(tstart/6)
        hsep = xmax*tinc/float( nhour )
        nticks = nhour/6 + 1
        HOUR6 = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'HOUR6: nhour, nticks, tstart = ', 
     &                      nhour, nticks, tstart
        end if

c  tlen < 20 days    DAY  = .TRUE.   tick every day
      else if(tsec.le. 20*24*60*60) then
        nday = dday + (hh + dhour)/24
        if( mod(hh+dhour,24).ne.0 ) nday = nday+1
        tinc = 1.
        tstart = dd
        nticks = nday + 1
        hsep = xmax*tinc/float( nday )
        DAY = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'DAY: nday, nticks, tstart = ', 
     &                    nday, nticks, tstart
        end if

c  tlen < 50 days    DAY7  = .TRUE.   tick every 7 days
      else if(tsec.le. 50*24*60*60) then
        nday = dday + (hh + dhour)/24
        if( mod(hh+dhour,24).ne.0 ) nday = nday+1
        tinc = 7.
        tstart = dd
        nticks = (nday/7 + 1)
        hsep = xmax*tinc/float( nday )
        DAY7 = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'DAY7: nday, nticks, tstart = ', 
     &                     nday, nticks, tstart
        end if


c  tlen < 100 days    BIMONTH  = .TRUE.   tick every 15 days
      else if(tsec.le. 100*24*60*60) then
        nday = dday + (hh + dhour)/24
        if( mod(hh+dhour,24).ne.0 ) nday = nday+1
        tinc = 15.
        tstart = dd
        nticks = (nday/15 + 1)
        hsep = xmax*tinc/float( nday )
        BIMONTH = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'BIMONTH: nday, nticks, tstart =',
     &                        nday, nticks, tstart
        end if

c  tlen > 100 days    MONTH  = .TRUE.   tick every 31 days
      else 
        nday = dday + (hh + dhour)/24
        if( mod(hh+dhour,24).ne.0 ) nday = nday+1
        tinc = 31.
        tstart = mm
        nticks = (nday/31 + 1)
        hsep = xmax*tinc/float( nday )
        MONTH = .TRUE.
        if( iprint.ge.1 ) then
          write(*,*)'MONTH: nday, nticks, tstart =', nday,nticks,tstart
        endif

      endif


c  npt is the number of entries found in each channel
c  nplot  is the number to plot (npt/dec)

      nskip = tdiff( starttime, stime ) * delta

      if( iprint.ge.1 ) then
        swindow = tdiff( starttime, stime )
        mwindow = swindow/60
        dday  = mwindow/1440
        dhour = (mwindow - 1440*dday)/60
        dmin  = mwindow - dday*1440 - dhour*60
        dss   = swindow - 60*mwindow
        write( dur, '(i4.4,a1,i2.2,a1,i2.2,a1,i2.2)') 
     &           dday, '/', dhour, '/', dmin,'/',dss

        write(*,*)'Data start time         : ', starttime
        write(*,*)'Plot start time         : ', stime
        write(*,*)'Duration (dddd/hh/mm/ss): ', dur
        write(*,*)'Delta-t (in s)          : ', delta
        write(*,*)'No. seconds to skip     : ', swindow
        write(*,*)'No. points to skip      : ', nskip
      endif

      npt   = tsec * delta
      nplot = npt/dec/seq_rec

      if( npt/dec.gt.MAXNUM ) then
        write(*,*)'number of points to plot > MAXNUM !'
        write(*,*)'npt/dec =',npt/dec,'  MAXNUM =',MAXNUM
        stop
      endif

      if( iprint.ge.1 ) then
        write(*,'(4a)')'Reading data... stime:',stime,
     &                               '  etime:',etime
      endif


c...read data to beginning skipping over nskip entries
      if( nskip.gt.0 ) then
        if( mod(nskip,seq_rec).ne.0 ) then
          write(*,'(a,i)')'nskip reset to integer multiple of '//
     &                    'seq_rec =', seq_rec
          nskip = seq_rec*(nskip/seq_rec)
          write(*,'(a,i)')'data to skip =', nskip
        endif
        nskip = nskip/seq_rec
        write(*,'(3(a,i10))') 'Skipping',nskip,' records'
        call ts_skip( ind1, nskip )
      endif

c...read in data only writing every "dec" point into array "data"
      if( iprint.ge.1 ) then
        write(*,'(a,i,a,i,a)') 'reading',npt,' entries into', 
     &                          nplot, ' locations'
      endif

      do i = 1, npt/seq_rec, dec/seq_rec
c...read in data, take first values off each record
        call ts_inrec( ind1, data1 )
        if( data1(1).eq.EOF ) then
          npt = i-1
          nplot = npt/dec
          if( iprint.ge.1 ) then
            write(*,'(a,2i)')'npt/nplot rescaled to', npt, nplot
          endif
          goto 200
        endif
c...check if = misdat (or v.close to misdat: within .1%)
        do j = 1, nchan
          if( data1(j).gt.misdat*0.999 .and. 
     &        data1(j).lt.misdat*1.001) then
            data1(j) = misdat
          endif
        enddo

c...rotate data
        if( .NOT.FIELD_COORDS ) then
          hx = data1(1)
          hy = data1(2)
          ex = data1(4)
          ey = data1(5)
          if( hx.ne.misdat .and. hy.ne.misdat .and. 
     &        anghx.ne.0. ) then
            call rotdat( hx, hy, anghx, anghx )
            data1(1) = hx
            data1(2) = hy
          endif
          if( ex.ne.misdat .and. ey.ne.misdat .and. 
     &        anghx.ne.0. ) then
            call rotdat( ex, ey, angex, angey )
            data1(4) = ex
            data1(5) = ey
          endif
        endif

        if( iprint.ge.20 ) then
          write(*,*)'Data written into array location...',
     &              (i-1)/dec+1
	  write(*,*)(data1(j),j=1,nchan)
        endif
        do j = 1, nchan
          data(j,(i-1)/dec +1) = data1(j)
        enddo
c...skip over next dec-1 data
        if( dec/seq_rec.gt.1 ) call ts_skip( ind1, (dec/seq_rec)-1 )
      enddo

c      close ( unit=ind1, status='KEEP' )


c...change delta (sampling rate) to reflect decimation level

200   ddelta = delta / float(dec) / float(seq_rec)

      if( nplot.le.0 ) go to 9999



c...calculate the x-axis scaling factor
c   totn is the total number of entries required to fill the plot
c   xstart is the start of the first data entry

      totn =  float(tsec) * ddelta
      xscale = xmax/totn
      xascle = xmax/(float(tsec)/600)
      ntot = ifix(totn)
      if( HOUR ) then
        xscale = xmax/(float(nhour*60*60)*ddelta)
        xascle = xmax/(float(nhour*60*60)/600)
        xstart = (float(min)/60. + pstartgap/3600.) * xmax/float(nhour)
      else if( HOUR6 ) then
        xscale = xmax/(float(nhour*60*60)*ddelta)
        xascle = xmax/(float(nhour*60*60)/600)
        xstart = (float(mod(hh,6))+(float(min)/60. + 
     &                              pstartgap/3600.))*xmax/float(nhour)
      else if( DAY ) then
        xscale = xmax/(float(nday*24*60*60)*ddelta)
        xascle = xmax/(float(nday*24*60*60)/600)
        xstart = (float(hh)/24. + pstartgap/86400.) * xmax/float(nday)
      else if( DAY7 ) then
        xscale = xmax/(float(nday*24*60*60)*ddelta)
        xascle = xmax/(float(nday*24*60*60)/600)
        xstart = (float(hh)/24. + pstartgap/86400.) * xmax/float(nday)
      end if

      if( iprint.ge.1 ) then
        write(*,'(a,f)')'Plotting scale (cm/pt) = ', xscale
        write(*,'(a,f)')'Data start     (cm)    = ', xstart
        if( DAY ) then
          write(*,*)'Plotting scale (cm/day) = ',xmax/float(nday)
        end if
      end if


c...scale the horizontal axis.

      do j = 1, ntot
        xp(j) = float(j-1)*xscale + xstart
      end do


c...derive maxima & minima
      
      do j = 1, nchan
        nmean = 0
        datmax = -1.E+32
        datmin = +1.E+32
        tot = 0.
        do i = 1, nplot
          if( data(j,i).ne.misdat ) then
            nmean = nmean+1
            tot  = tot + data(j,i)
            if( data(j,i).gt.datmax ) datmax = data(j,i)
            if( data(j,i).lt.datmin ) datmin = data(j,i)
          endif
        enddo
        ymean(j) = tot/float(nmean)
        xxmax(j) = datmax
        xxmin(j) = datmin
      enddo
      if( iprint.ge.1 ) then
        write(*,'(a)') 'chan   nmean      mean       max       min'//
     &                 '      *cal'
        do j = 1, nchan
          write(*,'(i2,i10,4f10.2)')
     &               j, nmean, ymean(j), 
     &               xxmax(j), xxmin(j), cal(j)
        enddo
      endif

c...remove the mean from each channel and apply calibration factors
      do j = 1, nchan
        xxmax(j) = (xxmax(j) - ymean(j))*cal(j)
        xxmin(j) = (xxmin(j) - ymean(j))*cal(j)
        do i = 1, nplot
          if( data(j,i).ne.misdat ) then
            data(j,i) = (data(j,i) - ymean(j))*cal(j)
          endif
        enddo
      enddo

c...calculate the vertical separation between channels.

      vsep = ymax / float(nchan + 1)


c...calculate the scaling factors for each channel:
c   set to nearest integer, or if nint = 0 set to nearest tenth

      do i = 1, nchan
        yscale = float(nchan + 1) / float(2 * nchan)
        sc(i) = (xxmax(i) - xxmin(i) )*yscale
        if( sc(i).ge.1. ) then
          idec  = alog10(sc(i))
          sc(i) = float(10**idec)*float(nint(sc(i)/float(10**idec)))
        else
          sc(i) = float(nint(sc(i)*10.))/10.
        endif
      enddo
      if( iprint.ge.1 ) then
        write(*,'(a)') 'chan   nmean      max       min      scale'
        do i = 1, nchan
          write(*,'(i2,i10,3f10.2)')
     &               i, nmean, xxmax(i), xxmin(i), sc(i)
        enddo
      endif


c...equalize the scaling factors for channels 1 & 2 (for Hx & Hy)
c   (take geometric mean value)

      if( chan(1).eq.'HX' ) then
c        sc(1) = amin1( sc(1), sc(2) )
        sc(1) = sqrt( sc(1)*sc(2) )
        if( sc(1).ge.1. ) then
          idec  = alog10(sc(1))
          sc(1) = float(10**idec)*float(nint(sc(1)/float(10**idec)))
        else
          sc(1) = float(nint(sc(1)*10.))/10.
        endif
        sc(2) = sc(1)
      endif


c...entry point to replot data with a different vertical scale.

250   continue

      do j = 1, nchan
        cm = chan(j)
        if(index(cm,'H').ne.0) unit = 'nT'
        if(index(cm,'T').ne.0) unit = 'C'
        if(index(cm,'B').ne.0) unit = 'V'
        if(index(cm,'E').ne.0) unit = 'mV/km'
        call rin(cm//' scale bar, in '//unit,sc(j))
      end do


c...open plotting file

      if( iprint.ge.1 ) write(*,'(a)')'opening plot....'
      call opnplt('list')
      if( iprint.ge.1 ) write(*,'(a)')'calling scale....'
      call scale(17.0)


c...shift the origin up and to the right to provide space for axis labels.

      if(iprint.ge.1)write(*,'(a)')'calling plot to shift origin....'
      call plot (0.7, 1.2, -3)

   
c...draw a box

      if( iprint.ge.1 ) write(*,'(a)')'drawing box....'
      call plot(xmax, 0.0, 2)
      call plot(xmax,ymax, 2)
      call plot( 0.0,ymax, 2)
      call plot( 0.0, 0.0, 2)


c...place time marks on the horizontal axis.

      if( iprint.ge.1 ) write(*,'(a)')'placing tick marks....'
      do j = 1, nticks
        xpp = float( j-1 )*hsep 
        call plot( xpp, 0.0, 3 )
        call plot( xpp,-0.125, 2 )
        call plot( xpp,-0.5, 3 )
        ht = tstart + (j-1)*ifix( tinc )
        if( SECOND ) then
          tscale = 'SECOND'
          write(tst(1:2),'(i2)') mod(ht,60)
          nchr = 2
        elseif( MINUTE ) then
          tscale = 'MINUTE'
          write(tst(1:2),'(i2)') mod(ht,60)
          nchr = 2
        elseif( HOUR .or. HOUR6 ) then
          tscale = 'HOURS '
          write(tst(1:2),'(i2)') mod(ht,24)
          nchr = 2
        elseif( DAY ) then
          mtime = stime
          mtime = nowtim( mtime, (j-1)*ifix( tinc ), 0, 0, 0 )
          tst(1:2) = mtime(9:10)
          tscale = 'DAYS' 
          nchr = 2
        elseif( DAY7 ) then
          mtime = stime
          mtime = nowtim( mtime, (j-1)*ifix( tinc ), 0, 0, 0 )
          tst(1:5) = mtime(6:) 
          tscale = 'DAYS' 
          nchr = 5
        elseif( BIMONTH ) then
          mtime = stime
          mtime = nowtim( mtime, (j-1)*ifix( tinc ), 0, 0, 0 )
          tst(1:5) = mtime(6:) 
          tscale = 'DAY' 
          nchr = 5
        elseif( MONTH ) then
          mtime = stime
          mtime = nowtim( mtime, (j-1)*ifix( tinc ), 0, 0, 0 )
          tst(1:5) = mtime(6:) 
          tscale = 'DAY' 
          nchr = 5
        endif
        call symbol( xpp, -0.5, .30, tst, 0.0, nchr)
        call plot( xpp, 0.0, 3 )
      enddo
      yp = 0.2
      xpp = xmax - 1.5
      nchr = 6
      call symbol( xpp, yp, 0.30, tscale, 0.0, nchr )


c...plot the input data window name in the upper right hand corner.

      xpp = xmax - 3.5
      yp  = ymax + 0.20
      nchr = 8 
      if( iprint.ge.1 ) write(*,'(a)')'plotting window name....'
      call symbol( xpp, yp, 0.35, tswindow, 0.0,  nchr)


c...plot the start time on the upper left corner.

2001  format('Start Time: ',a19)
      write (ttime, 2001, err= 9001) stime
      xpp = 0.0
      nchr=32
      call symbol(xpp,yp,0.35,ttime,0.0,nchr)

c...relate the length of the vertical scale bars to the data.

      kk = 1
      do j = 1, nchan
        scvbar = sc(j)
        cm = chan(j)
        if(index(cm,'H').ne.0) unit = 'nT'
        if(index(cm,'T').ne.0) unit = 'C'
        if(index(cm,'B').ne.0) unit = 'V'
        if(index(cm,'E').ne.0) unit = 'mV/km'
        call trunc(unit,i)
        if(abs(scvbar).lt.10.0) then
          write( tempst(kk:), 2003 ) scvbar, unit(1:i)
        else
          write( tempst(kk:), 2002 ) nint(scvbar), unit(1:i)
        endif
        kk = kk + i + 7
      enddo
      yp = -1.0
      call trunc(tempst,j)
      call symbol( xpp, yp, 0.30, tempst, 0.0, j)

2002  format (i6,a,1x)
2003  format (f6.2,a,1x)


c...calculate the length of the vertical scale bars.

      vbar = ymax/float( nchan*2 )
      vbar2 = vbar/2.0

      kk = nchan + 1
      do 500 j = nchan, 1, -1


c...calculate the y-axis scaling factor.

        kk = kk - 1
        call pcolor( colmap(j) )
        yscale = ymax/(2.0*nchan*sc(j))
        call plot (0., vsep, -3)
        xn = float(j)
        call number( -.7, 0.1, .25, xn, 0.0, 'i2' )
        compp = chan(j)
        if(compp(1:1).eq.'H') compp(1:1) = 'B'
        nchr=2
        call symbol( -.7, -.3, .25, compp, 0.0, nchr )


c...plot the vbars.

        call plot (-.17,  vbar2, 3)
        call plot (-.13,  vbar2, 2)
        call plot (-.15,  vbar2, 3)
        call plot (-.15, -vbar2, 2)
        call plot (-.17, -vbar2, 3)
        call plot (-.13, -vbar2, 2)
        call plot (0., 0., 3)
        vplus  =  float(kk)*vsep
        vminus = -float(nchan+1-kk)*vsep


c...plot the time series

      if( iprint.ge.1 ) write(*,'(a)')'plotting data....'

        yp = data(j,1)*yscale
        call plotlm(xp(1), yp, vminus, vplus, 3)
        ipen = 2
        do k = 2, nplot
          if( data(j,k).eq.misdat .or. data(j,k-1).eq.misdat) then
            ipen = 3
          else
            ipen = 2
          endif
          yp = data(j,k)*yscale
          call plotlm(xp(k), yp, vminus, vplus, ipen)
        end do

500   continue

600   call endplt

      call lin( 'Replot data with another vertical scale?', REPLOT )
      if( REPLOT ) go to 250

      call lin( 'Plot a new time frame?', NEWFRM )
      if( NEWFRM ) then
        nframe = nframe + 1
        SECOND = .FALSE.
        MINUTE = .FALSE.
        HOUR   = .FALSE.
        HOUR6  = .FALSE.
        DAY    = .FALSE.
        rewind( ind1 )
        go to 50
      end if
      go to 9999

 9001 write (*, '(a)') ' ERROR converting integers to characters.'
      go to 9999
 9002 write (*, '(a)') ' ERROR converting characters to integers.'
      goto 9999
 
 9999 continue
      stop
      end
c-------------------------------------------------------------------
      subroutine plotlm(x,y,y1,y2,ipen)
c-------------------------------------------------------------------
      integer ipen
      real x, y, y1, y2, yy
c
      yy = y
      if(yy.gt.y2) yy = y2
      if(yy.lt.y1) yy = y1
      call plot(x,yy,ipen)
      return
      end
