      program tsrestack
c-------------------------------------------------------------------------
c      T S R E S T A C K
c-------------------------------------------------------------------------
c
c  Program for stacking data o/p from TSCASCADE
c
c
c
c  ver=v3.0
c       modified to rotate spectra on input so that all jacknife errors
c       are in right co-ordinate system
c       also, jackknife estimates of transfer functions
c
c
c  ver=v3.1
c     - modified to have choice of which impedance element variance to
c       minimize (ntype in jknvarmin)
c     - corrections for GDS-only data
c     - asks for absolute angle to rotate TO (not BY)
c     - normalises spectra by magnetic field power
c     - corrected rotations (see "rotang" and "angle")
c     - reads in ROTANG from process.cfg file
c
c
c  ver=v4.0: name changed from restack7 to tsrestack
c     - version 4 modifications
c       times now 12 characters (to allow seconds)
c
c  ver=v4.1: changes for PC version
c
c  ver=v4.2: added P_NORM
c
c  ver=v4.3: added COHMIN(20) as an array
c
c-------------------------------------------------------------------------

      implicit none

      include 'maxspc.inc'
      include 'maxflims.inc'
      include 'ctrlblk.inc' 
      include 'version.inc' 
      include 'io.inc'
      include 'op_sys.inc'
      include 'bdate.inc'
      
c...set as integer*4 to be compatible with SUN getarg routine
      integer*4 iarg

      integer MTOT
      parameter ( MTOT = MAXF2*49 )
      integer MAXFILES
      parameter ( MAXFILES = 400 )

      integer nacpt(MAXF), iflag(MAXSPC), isum, ntot(MAXSPC,MAXF)
      integer extra(MAXF)
      integer ndeg(MAXF), azim, ntype, ncas(MAXF)
      integer nacptgds(MAXF), ndeggds(MAXF)
      integer numstk(MAXF), accepted(MAXSPC,MAXF)
      integer narg, getnarg
      integer ke, km, km2
      integer lnblnk, lw
      integer i, i1, i2, j, j1, j2, k, m, if, iff, nc, nfile, nspec
      integer loop, lf, iunit, ifile, istart, iend, npt, maxtyp
      integer instr1, instr2, nchan1, nchan2
      integer kspecy, kspecz
      integer dec

      integer system
      external system

      real weight(MAXSPC), angle, angle1
      real y(MAXF2,49), z(MAXF2,49), ytot(MAXF2,49), ztot(MAXF2,49),
     &     wytot(MAXF2), wztot(MAXF2),
     &     ytemp(49), ztemp(49), calibs5(5), cutsl5(5), cutsh5(5),
     &     cohmin(MAXF), cohmin1, cohmax, rz(4,MAXF), f(64)
      real zvar(4), rtf(2,MAXF), rotang, xlat, xlon, prd(MAXF)
      real lat1, long1, elev1, lat2, long2, elev2
      real mcoh, ocoh, pcoh, fpcoh, fmcoh, focoh
      real g12, g13, g14, g16, g17, g23, g24, g26, g27, g34
      real g312, g412, g512, g612, g712
      real cohy, cohz
      real pcndhy, pcndhz, pcehdy, pcehdz, pczhdy, pczhdz, pczdhy, 
     &     pczdhz
      real mcnhdy, mcehdy, mcnhdz, mcehdz, mczhdy, mczhdz
      real ochdy, ocnhy, ocndy, ocehy, oczhy, ocedy, oczdy
      real ochdz, ocnhz, ocndz, ocehz, oczhz, ocedz, oczdz
      real per, perr, pmax, pmin, rho, pha, rmax, rmin
      real zerr, zr, zi
      real r1, r2
      real convz2r, convz2p
      real w, wy, wz, why, whz, wdy, wdz, wny, wnz, wey, wez
      real scale, scaley, scalez
      real vartol, samp, convert
      
      real cosd, sind, atand

      complex ztf(4,MAXF), sy(7,7), sz(7,7), tf(2,MAXF), cy, cz,
     &        ss(7,7,MAXSPC), t1, t2, zz, zmax, zmin, 
     &        s7gds(7,7,MAXF)

      character comp(4)*2, fin*40, fout*40, casfile(MAXFILES)*40,
     &          stat*6, stime*19, ctype*3, line*80, word*10
      character window*8, windowrr*8, proj*3, sep*1
      character*1 ans, jjans, wans, mtans
      character*10 cmdarg
      character*4 casver
      character hash*1
      character*6 progprec, dataprec
      character*4 cfgver

      character*24 date_string

      logical JJANAL, ORITE, MT, GDS, ALL, RR, WGTS, HSG, REJECT,
     &        AZIMSET, ASK, SI, MAGREMOTE, P_NORM
      logical QSTACK, QJJANAL, QCOHMIN, QCOHMAX, QPROTYP, QORITE,
     &        QROTANG, QDECLIN, QMAXTYP, QNTYPE, QWEIGHT, QUNITS,
     &        QREMOTE, QCFGVER
      logical SINGLE

      external isum

c---worksp array must be 2*7*7*nfreq(here=20)

      integer inc, nnchan, prterr
      real ddelta
      common /contst/ inc, ddelta, prterr, nnchan

      real worksp
      common /work/ worksp(1000)

      real delta, tlen, period
      common /time/ delta, tlen, period(MAXF)

      complex s7

      common /power7/ s7(7,7,MAXF)

      integer ndev, ndevm
      common /nwrite/ ndev, ndevm

      integer ipre, ntrace
      common /traces7/ ipre(7), ntrace

      integer npre, epre, hpre, dpre, zpre, rxpre, rypre
      common /pres7/ npre, epre, hpre, dpre, zpre, rxpre, rypre

      real alpha
      common /confl/ alpha

      character*20 title
      common /event/ title

      real cutsh
      common /cutsh7/ cutsh(7)

      real cutsl
      common /cutsl7/ cutsl(7)

      integer n
      common n
 
      data wytot/MAXF2*0./, wztot/MAXF2*0./, nacpt/MAXF*0/
      data ytot/MTOT*0./, ztot/MTOT*0./
      data ndeg/MAXF*0/
      data comp/'XX', 'XY', 'YX', 'YY' /

c---F values for F(4:n-4:0.05); where n = 2x(no. of estimates)
      data f/ 0.0,  0.0,19.25, 6.39, 4.53, 3.84, 3.48, 3.25,
     &        3.11, 3.01, 2.93, 2.87, 2.82, 2.78, 2.74, 2.71,
     &        2.69, 2.67, 2.65, 2.62, 2.61, 2.60, 2.59, 2.58,
     &       2.574,2.565,2.557,2.550,2.543,2.537,2.531,2.525,
     &       2.520,2.515,2.511,2.507,2.503,2.499,2.495,2.492,
     &       2.489,2.486,2.483,2.480,2.478,2.475,2.473,2.471,
     &       2.469,2.466,2.465,2.463,2.461,2.459,2.457,2.456,
     &       2.454,2.453,2.451,2.450,2.449,2.447,2.446,2.445 /
     
      data vartol/0.01/
      data maxtyp/1/

c-------------------------------------------------------------------------------
c     initialize

      call fdate(date_string)

	call chkend(sysend)



      call precision( SINGLE )
      if( SINGLE ) then
        progprec = 'SINGLE'
      else
        progprec = 'DOUBLE'
      endif


      call dirsep(sep)


c-------------------------------------------------------------------------------
c     initialize some variables

      ASK     = .FALSE.

      QCFGVER = .TRUE.
      QSTACK  = .TRUE.
      QJJANAL = .TRUE.
      QCOHMIN = .TRUE.
      QCOHMAX = .TRUE.
      QPROTYP = .TRUE.
      QORITE  = .TRUE.
      QROTANG = .TRUE.
      QDECLIN = .TRUE.
      QMAXTYP = .TRUE.
      QNTYPE  = .TRUE.
      QWEIGHT = .TRUE.
      QREMOTE = .FALSE.
      QUNITS  = .TRUE.
      AZIMSET = .FALSE.

      ndev = outd1
      iprint = 0
      mtans = 'M'
      jjans = 'Y'
      dec = 0
      do i = 1, MAXF
        cohmin(i) = 0.75
      enddo
      cohmax = 1.00
      ctype = 'JKN'
      ntype = 1
      wans = 'Y'
      MAGREMOTE = .TRUE.
      P_NORM = .TRUE.

c-------------------------------------------------------------------------

      narg = getnarg()
      do iarg = 1, narg
        call getcmdarg( iarg, cmdarg)
        if( index(cmdarg,'h').ne.0 ) then
          write(*,*)'Permitted options:-'
          write(*,*)'                    -1  or -2  iprint level'
          write(*,*)'                    -ask       ask all questions'
          stop
        else if( index(cmdarg,'ask').ne.0 ) then
          ASK = .TRUE.
        else
          read(cmdarg,'(i2)') iprint
        endif
      enddo

c-------------------------------------------------------------------------------
c     get command-line arguments

      call iin( 'Set error message print level (-1/0/1/2)', iprint)
      prterr  = iprint

c-------------------------------------------------------------------------------
c...write out banner

      call banner( 'TSRESTACK' )

c-------------------------------------------------------------------------------
c     Check for configuration file. If doesn't exist then create it and write
c     default values into it.

      open(unit=4, file='process.cfg', status='old', err=101)
      if( iprint.ge.0 ) then
        write(*,*)'Opened processing configuration file process.cfg'
      endif
      goto 102

101   open(unit=7, file='process.cfg', status='new')
      write(7,'(a)')'# process.cfg'
      write(7,'(a)')'# This file contains parameters that may be'
      write(7,'(a)')'# alterred for running limscas7 and restack7.'
      write(7,'(a)')'# The parameters listed below are the default'
      write(7,'(a)')'# values in the limscas7 executable.'
      write(7,'(a)')'VER   :  '//ver//'     cfg file version'
      QCFGVER = .FALSE.
      write(7,'(a)')'NUMSTK:',MAXF,'  number of estimates per substack'
      write(7,'(20i4)') (numstk(i),i=1,MAXF)
      write(7,'(a)')'VARTOL:', vartol,' tolerance for minimum variance'
      write(7,'(a)')'# Other parameters are allowed (remove # to use)'
      write(7,'(a)')'OWRITE:   T     overwrite files without asking'
      write(7,'(a)')'PROTYP:  MT     processing type: MT, GDS or HSG'
      write(7,'(a)')'REMOTE:   H     type of remote: H or E'
      write(7,'(a)')'DECLIN:   0     sets declination'
      write(7,'(a)')'ROTANG:   0     sets rotation co-ord system'
      write(7,'(a)')'MAG_OR:   T     correct magnetometer '//
     &                  'orientation using baselines'
      write(7,'(a)')'P_NORM:   T     normalize spectra by '//
     &                  'horizontal magnetic field power'
      write(7,'(a)')'STACK :   T     .TRUE. substacking'
      write(7,'(a)')'JJANAL:   T     .TRUE. Jones-Joedicke processing'
      write(7,'(a)')'COHMIN: 20*0.75 coherence minima array'
      write(7,'(a)')'COHMAX: 0.99    coherence maximum'
      write(7,'(a)')'UNITS :  SI     trans fn units: FIELD or SI'

      close(unit=7)
      if( iprint.ge.0 ) then
        write(*,*)'Configuration file process.cfg written with '//
     &            'default values'
      endif

102   do 103 while(loop.eq.0)
        read(4,'(a)',end=104) line
        if( line(1:1).eq.'#' ) goto 103

        if( line(1:6).eq.'VER   ' ) then
          call word1( line(8:), cfgver, lw )
          if( iprint.ge.1 ) then
            write(*,*)'VER    found with value :', cfgver
          endif
          if( cfgver.ne.ver ) then
            write(*,*)'Processing configuration file of '//
     &                'inappropriate version number'
            write(*,*)'Program version =', ver, 
     &                '  cfg file version =', cfgver
            write(*,*)'Delete process.cfg file and re-run tscascade'//
     &                ' to generate a new one'
            stop
          endif
          QCFGVER = .FALSE.
          goto 103

        else if( line(1:6).eq.'NUMSTK' ) then
          read(line(8:),*) n
          if( n.ne.MAXF ) then
            write(*,*) 'Number of substacks not equal to MAXF'
            write(*,*) 'number =', n, '     MAXF =', MAXF
            write(*,*) 'Edit process.cfg file then re-run'
            stop
          endif
          read(4,*) (numstk(i),i=1,n)
          if( iprint.ge.1 ) then
            write(*,*)'NUMSTK found and read in as follows:'
            write(*,'(20i4)') (numstk(i),i=1,MAXF)
          endif
          goto 103

        else if( line(1:6).eq.'VARTOL' ) then
          goto 103

        else if( line(1:6).eq.'OWRITE' ) then
          read(line(8:),*) ORITE
          QORITE = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'OWRITE found with value :', ORITE
          endif
          goto 103

        else if( line(1:6).eq.'PROTYP' ) then
          call word1( line(8:), word, lw )
          if( index(word,'MT').ne.0 ) then
            MT  = .TRUE.
            GDS = .FALSE.
            HSG = .FALSE.
            REJECT = .FALSE.
          else if( index(word,'GDS').ne.0 ) then
            MT  = .FALSE.
            GDS = .TRUE.
            HSG = .FALSE.
            REJECT = .FALSE.
          else if( index(word,'HSG').ne.0 ) then
            MT  = .FALSE.
            GDS = .FALSE.
            HSG = .TRUE.
            REJECT = .TRUE.
          else
            write(*,*)'Unrecognized processing type =', word(1:lw)
            write(*,*)'Edit process.cfg file and re-run'
            stop
          endif
          QPROTYP = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'PROTYP found with value :', word(1:lw)
          endif
          goto 103

        else if( line(1:6).eq.'REMOTE' ) then
          call word1( line(8:), word, lw )
          if( index(word,'H').ne.0 ) then
            MAGREMOTE = .TRUE.
          elseif( index(word,'E').ne.0 ) then
            MAGREMOTE = .FALSE.
          else
            write(*,*)'Unrecognized remote type =', word(1:lw)
            write(*,*)'Edit process.cfg file and re-run'
            stop
          endif
          QREMOTE = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'REMOTE found with value :', word(1:lw)
          endif
          goto 103

        else if( line(1:6).eq.'DECLIN' ) then
          goto 103

        else if( line(1:6).eq.'ROTANG' ) then
          read(line(8:),*) rotang
          QROTANG = .FALSE.
          AZIMSET = .TRUE.
          if( iprint.ge.1 ) then
            write(*,*)'ROTANG found with value :', rotang
          endif
          goto 103
 
        else if( line(1:6).eq.'MAG_OR' ) then
          goto 103
 
        else if( line(1:6).eq.'P_NORM' ) then
          read(line(8:),*) P_NORM
          if( iprint.ge.1 ) then
            write(*,*)'P_NORM found with value :', P_NORM
          endif
          goto 103

        else if( line(1:6).eq.'STACK ' ) then
          goto 103

        else if( line(1:6).eq.'JJANAL' ) then
          read(line(8:),*) JJANAL
          QJJANAL = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'JJANAL found with value :', JJANAL
          endif
          if( JJANAL ) then
            jjans = 'Y'
          else
            jjans = 'N'
          endif
          goto 103


        else if( line(1:6).eq.'MAXTYP' ) then
          read(line(8:),'(a10)') line
          if( index(line,'JKN').ne.0 ) then
            ctype = 'JKN'
          else if( index(line,'COH').ne.0 ) then
            ctype = 'COH'
          else if( index(line,'VAR').ne.0 ) then
            ctype = 'VAR'
          else
            write(*,*)'Unrecognized max type =', line(1:5)
            write(*,*)'Edit process.cfg file and re-run'
            stop
          endif
          QMAXTYP = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'MAXTYP found with value :', line(1:5)
          endif
          goto 103

        else if( line(1:6).eq.'NTYPE ' ) then
          read(line(8:),*) ntype
          QNTYPE = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'NTYPE  found with value :', ntype
          endif
          if( ctype.eq.'COH' ) then
            if( ntype.lt.1 .and. ntype.gt.7 ) then
              write(*,*)'Impermissable NTYPE for given MAXTYP'
              write(*,*)'MAXTYP =', MAXTYP,'  NTYPE =', ntype
              write(*,*)'Edit process.cfg file and re-run'
              stop
            endif
          else if( ctype.eq.'VAR' ) then
            if( ntype.lt.1 .and. ntype.gt.4 ) then
              write(*,*)'Impermissable NTYPE for given MAXTYP'
              write(*,*)'MAXTYP =', MAXTYP,'  NTYPE =', ntype
              write(*,*)'Edit process.cfg file and re-run'
              stop
            endif
          else if( ctype.eq.'JKN' ) then
            if( ntype.lt.1 .and. ntype.gt.3 ) then
              write(*,*)'Impermissable NTYPE for given MAXTYP'
              write(*,*)'MAXTYP =', MAXTYP,'  NTYPE =', ntype
              write(*,*)'Edit process.cfg file and re-run'
              stop
            endif
          endif
          goto 103
          

        else if( line(1:6).eq.'WEIGHT' ) then
          read(line(8:),*) WGTS
          QWEIGHT = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'WEIGHT found with value :', WGTS
          endif
          if( WGTS ) then
            wans = 'Y'
          else
            wans = 'N'
          endif
          goto 103

        else if( line(1:6).eq.'COHMIN' ) then
          read(line(8:),*) cohmin
          QCOHMIN = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'COHMIN found with value :', cohmin
          endif
          goto 103

        else if( line(1:6).eq.'COHMAX' ) then
          read(line(8:),*) cohmax
          QCOHMAX = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'COHMAX found with value :', cohmax
          endif
          goto 103

        else if( line(1:6).eq.'UNITS ' ) then
          read(line(8:),'(a10)') line
          QUNITS = .FALSE.
          if( iprint.ge.1 ) then
            write(*,*)'UNITS found with value :', line(1:10)
          endif
          if( index(line,'SI').ne.0 ) then
            SI = .TRUE.
          else if( index(line,'FIELD').ne.0 ) then
            SI = .FALSE.
          else
            write(*,*)'Unrecognized units  =', line(1:lnblnk(line))
            write(*,*)'Edit process.cfg file and re-run'
            stop
          endif
          goto 103


        else
          write(*,*) 'Unrecognised keyword in process.cfg file'
          write(*,*) 'First six characters =', line(1:6)
        endif
103   continue
104   if( iprint.ge.0 ) then
        write(*,*)'Read in parameters from process.cfg file'
      endif
      
110   continue

      if( QCFGVER ) then
        write(*,*)'Processing configuration file without '//
     &            'version number'
        write(*,*)'Delete process.cfg file and re-run tscascade'//
     &            ' to generate a new one'
        stop
      endif


      if( ASK ) then
        QSTACK  = .TRUE.
        QJJANAL = .TRUE.
        QCOHMIN = .TRUE.
        QCOHMAX = .TRUE.
        QROTANG = .TRUE.
        QDECLIN = .TRUE.
        QPROTYP = .TRUE.
        QORITE  = .TRUE.
        QMAXTYP = .TRUE.
        QNTYPE  = .TRUE.
        QWEIGHT = .TRUE.
      endif


c-------------------------------------------------------------------------------

      do i = 3, 8
        f(i) = (4./ (float(2*i)-4.) ) * f(i)
      end do

      alpha = 0.32

      npre = 1
      epre = 1
      hpre = 1
      dpre = 1
      zpre = 1
      rxpre = 1
      rypre = 1
      ntrace = 7
      do i = 1, 7
        ipre(i) = i
      enddo


      stat = 'nod001'
      call tin( 'Give station', stat )
      proj = stat(1:3)

c---open o/p file .out ----

      fout = stat//sep//stat//'.out'
    7 open(unit=outd1, status='NEW', file=fout, err=8 )
      go to 9
    8 call trunc( fout, lf )
      write(*,*)'Error ***** ==> file ', fout(:lf), ' already exists.'
      ORITE = .TRUE.
      if( iprint.ge.0 .and. QORITE ) then 
        ans = 'Y'
        call yin( 'Do you wish to overwrite it (y/n)?', ans )
        if( ans.eq.'Y' ) then
          ORITE = .TRUE.
        else
          ORITE = .FALSE.
        endif
      else
        write(*,*)'Being overwritten'
      end if
      if (ORITE) then
        open(unit=outd1, status='OLD', file=fout )
      else
        call tin ('Give a new filename', fout )
        go to 7
      end if
    9 continue

      write( outd1, 1010 ) stat(:6)
 1010 format ( 3(/), 1x,75('*'),/,1x,'*',73x,'*',/,1x,'*',4x,a,t76,
     &         '*',/,1x,'*',73x,'*',/,1x,75('*') )

c-------------------------

      ans = 'Y'
      call yin( 'Remote-reference processing (y/n)?', ans )
      if( ans.eq.'Y' ) then
        RR = .TRUE.
      else
        RR = .FALSE.
      endif

c-------------------------

2     if( QPROTYP ) then
        call tin( 'MT, GDS, or HSG processing (M/G/H)', mtans )
        call shiftup( mtans )
        if( mtans.eq.'M' ) then
          MT  = .TRUE.
          GDS = .FALSE.
          HSG = .FALSE.
          REJECT = .FALSE.
        else if( mtans.eq.'G' ) then
          MT  = .FALSE.
          GDS = .TRUE.
          HSG = .FALSE.
          REJECT = .FALSE.
        else if( mtans.eq.'H' ) then
          MT  = .FALSE.
          GDS = .FALSE.
          HSG = .TRUE.
          REJECT = .TRUE.
          call lin( 'Reject unphysical estimates of C', REJECT )
        else
          write(*,*)'---erroneous response; answer M, G or H'
          goto 2
        endif
      endif

      if( QCOHMIN ) then
        cohmin1 = cohmin(1)
        call rin( 'Give minimum partial coherence', cohmin1 )
        do i = 1, MAXF
          cohmin(i) = cohmin1
        enddo
      endif

      if( QCOHMAX ) then
        call rin( 'Give maximum partial coherence', cohmax )
        cohmax = amin1( cohmax, 1.0 )
      endif


      if( QUNITS ) then
        line = 'SI'
3       call tin( 'Give units (SI or FIELD', line )
        if( index(line,'SI').ne.0 ) then
          SI = .TRUE.
        else if( index(line,'FIELD').ne.0 ) then
          SI = .FALSE.
        else
          write(*,*)'Unrecognized units  =', line(1:lnblnk(line))
          goto 3
        endif
      endif

      if( QJJANAL ) then 
        call yin( 'Jones/Joedicke analysis (y/n)?', jjans )
        if( jjans.eq.'Y' ) then
          JJANAL = .TRUE.
        else
          JJANAL = .FALSE.
        endif
      endif


c...Open temporary files to store the accumulated spectra files.
      if ( JJANAL ) then
        do iunit = 41, 40+MAXF
          open( unit=iunit, form='UNFORMATTED', status='SCRATCH' )
        end do

4       if( QMAXTYP ) then
          call tin( 'Give CTYPE: COH, VAR or JKN', ctype )
          call shiftup( ctype )
          if( ctype.ne.'COH' .and. ctype.ne.'VAR' .and. 
     &       ctype.ne.'JKN' ) then
            write(*,*)'Erroneous ctype=',ctype,'  only COH, VAR or JKN'
            goto 4
          endif
        endif

      if( QNTYPE ) then
5       if( iprint.ge.0 ) then
          write(*,'(1x,a,$)') 'Give NTYPE (? for options: default=1) >'
        endif
        read(*,'(a)') ans
        if( ans.eq.'?' ) then
          if( ctype.eq.'COH' ) then
      write(*,*) ' NTYPE = 1  maximize Mcoh(ex,hx,hy)*Mcoh(ey,hx,hy)'
      write(*,*) '         2           Mcoh(ex,hx,hy)'
      write(*,*) '         3           Mcoh(ey,hx,hy)'
      write(*,*) '         4           Pcoh(ex,hy,hx)*Pcoh(ey,hx,hy)'
      write(*,*) '         5           Pcoh(ex,hy,hx)'
      write(*,*) '         6           Pcoh(ey,hx,hy)'
      write(*,*) '         7           Mcoh(rx,hx,hy)*Mcoh(ry,hx,hy)'
         else if( ctype.eq.'VAR' ) then
      write(*,*) ' NTYPE = 1  minimize SdExHx*SdExHy*SdEyHx*SdEyHy'
      write(*,*) '         2           SdExHx*SdExHy'
      write(*,*) '         3           SdEyHx*SdEyHy'
      write(*,*) '         4           SdExHy*SdEyHx'
         else if( ctype.eq.'JKN' ) then
      write(*,*) ' NTYPE = 1  minimize Var(Zxy) * Var(Zyx) '
      write(*,*) '         2           Var(Zxy)'
      write(*,*) '         3           Var(Zyx)'
          endif
          goto 5
        end if
        read( ans, '(i1)', err=5 ) ntype
        if( ntype.eq.0 ) ntype = 1

        if( ctype.eq.'COH' ) then
          if( ntype.lt.1 .and. ntype.gt.7 ) goto 5
        else if( ctype.eq.'VAR' ) then
          if( ntype.lt.1 .and. ntype.gt.4 ) goto 5
        else if( ctype.eq.'JKN' ) then
          if( ntype.lt.1 .and. ntype.gt.3 ) goto 5
        endif
      endif


        if( QWEIGHT ) then
          call yin( 'Stack using weights (or unweighted) (y/n)?', wans )
          if( wans.eq.'Y' ) then
            WGTS = .TRUE.
          else
            WGTS = .FALSE.
          endif
        endif

      endif
 
      write(outd1,*) ' '
      write(outd1,*) ' ctype=',ctype,'   ntype=',ntype, 
     &               '   weights=',WGTS
c---------------------------------------------------------------
c     read all data in & place into arrays
 
      ifile = 0
 

c...UNIX code only calls "system"
      fin = '*'
6     call tin( 'Give input .cas file (* for all .cas [UNIX only])',fin)
      call trunc( fin, lf )
        write(*,*)'Input read in...: ', fin(1:lf)
      if( iprint.ge.3 ) then
        write(*,*)'Input read in...: ', fin(1:lf)
      endif
      
      if( fin.eq.'*' ) then
        if( UNIX ) then
          ALL = .TRUE.
          if( RR ) then
            line = 'ls -1 '//stat//'/'//stat(4:6)//'??'//
     &                  '???'//'.cas > .restack7'
          else
            line = 'ls -1 '//stat//'/*.cas > .restack7'
          endif
          i = system( line )
          open( ind2, file='.restack7' )
        else
          write(*,*)'Cannot use this input for DOS version'
          goto 6
        endif
      else
        ALL = .FALSE.
        if( index(fin,'.cas').eq.0 ) then
          call trunc( fin, lf )
          line = fin(1:lf)//'.cas'
          fin  = line
        endif
        line = stat//sep//fin
        fin  = line
        goto 12
      end if

10    if( ALL ) then 
        read(ind2,'(a)',end=20) fin
      else
        fin = 'stop'
        call tin( 'Give input .cas file (STOP to end)', fin )
        if( fin.eq.'stop' ) goto 20
        if( index(fin,'.cas').eq.0 ) then
          call trunc( fin, lf )
          line = fin(1:lf)//'.cas'
          fin = line
        endif
        line = stat//sep//fin
        fin  = line
      end if

12    ifile = ifile+1
      casfile(ifile) = fin
      call trunc( fin, lf )
      open( unit=4, status='OLD', file=fin(1:lf), form='UNFORMATTED')
      if( iprint.ge.1 ) write(*,*)'Opened file =>', fin(:lf)

      if( ifile.eq.1 ) then
        write(outd1,1002) fin(:lf)
 1002   format(13x,'Files read in ==>', a )
      else
        write(outd1,1003) fin(:lf)
 1003   format( 30x, a )
      end if


c...read over comment information in cascade file
      read(4,err=10,end=10) hash
      do while( hash.eq.'#' )
        read(4,err=10,end=10) hash
      enddo
      backspace(4)

c...read in header information
      read(4,err=10,end=10) casver
      if( casver.ne.ver ) then
        write(*,*)'Cascade file with different version'
        write(*,*)'Program version =',ver,'   Cascade file =',casver
        write(*,*)'Re-run tscascade'
        stop
      endif

      read(4,err=10,end=10) dataprec
      if( progprec.ne.dataprec ) then
        write(*,*)'Cascade file with different precision'
        write(*,*)'Program version ='//progprec//'   Cascade file ='//
     &            dataprec
        stop
      endif

      read(4,err=10,end=10)window,  instr1,nchan1,lat1,long1,elev1
      read(4,err=10,end=10)windowrr,instr2,nchan2,lat2,long2,elev2
      read(4,err=10,end=10)stime, samp, npt, azim

      if( nchan1.eq.4 ) then
        zpre = 0
      endif

      if( RR ) then
        write(*,*)'Cascade file with remote data >', fin(:lf),
     &            '     angle=',azim
      else
        write(*,*)'Cascade file with local  data >', fin(:lf),
     &            '     angle=',azim
      end if

      angle = float(azim)
      if( ifile.eq.1 ) then
        angle1 = angle
      else
        if( angle.ne.angle1 ) then
          write(*,*)'Cascade files not at same angle! Fix and re-run'
          stop
        endif
      endif

      if( iprint.ge.1 ) then
        write(*,'(5a,2i3,a,2i2)')' window=', window,'  remote=',
     &             windowrr,' instruments = ', instr1, instr2,
     &             ' channels=', nchan1, nchan2
        write(*,'(3a,f10.3,a,i6)')' start time=',stime,
     &       '  sample rate (Hz)=',samp, '  max no. points=', npt
      endif


      if( ifile.eq.1 ) then

c        call rin( 'Give digitizing rate in Hertz', samp )
        delta = 1.0/samp
        do 1 i = 1, MAXF2
          period(2*i-1) = (32.*delta*(2.**(I-1)))/8.
          period(2*i)   = (32.*delta*(2.**(I-1)))/6.
    1   continue


c...get -3 dB high pass filter coeffs

        fin = 'hardware.'//proj
        open(unit=ind3,file=fin)
        call gethardware(ind3, instr1, nchan1, calibs5, cutsl5, cutsh5)
        do i = 1, 5
          cutsl(i) = cutsl5(i)
          cutsh(i) = cutsh5(i)
        end do

        if( RR ) then
          rewind(ind3)
          call gethardware( ind3, instr2, nchan2, calibs5, 
     &                      cutsl5, cutsh5)
        endif
        cutsl(6) = cutsl5(1)
        cutsh(6) = cutsh5(1)
        cutsl(7) = cutsl5(2)
        cutsh(7) = cutsh5(2)
        close(unit=ind3)

        xlat = lat1
        xlon = long1

      endif



c...read cascade file here.....................................................

11    read(4,end=10) m, kspecy, (y(m,i),i=1,49),
     &                  kspecz, (z(m,i),i=1,49)
      if( iprint.ge.3 ) then
        write(*,'(3(a,i8))')'  m=',m,'  kspecy=',kspecy,
     &                               '  kspecz=',kspecz
      end if

      if(m.lt.1 .or. m.gt.10 ) then
        write(*,*)'Unknown m = ', m
        goto 11
      end if

      if( ncas(2*m-1).eq.MAXSPC ) then
        extra(m) = extra(m) + 1
        if( extra(m).eq.1 ) then
          write(*,*)'>>>>Reached maximum number of spectra for m =',m
        endif
        goto 11
      endif

      ncas(2*m-1) = ncas(2*m-1)+1
      ncas(2*m  ) = ncas(2*m  )+1

      if( kspecy.eq.0 .or. kspecz.eq.0 ) goto 11

c...if HxHx or HyHy spectrum = 0. then skip
      if( y(m,25).le.0. .or. y(m,34).le.0.) then
        write(*,*) 'Zero (or negative) horizontal magnetic field',
     &             'amplitude; m = ',m
        write(*,*) ' y(m,25), y(m,34) = ', y(m,25), y(m,34)
        write(*,*) 'spectra not added to stack'
        goto 11
      endif
 
c...write auto & cross-spectra into S arrays after normalizing by
c   horizontal magnetic field amplitudes
      if( P_NORM ) then
        scaley = 10.**((log10(y(m,25)) + log10(y(m,34)))/2.)
      else
        scaley = 1.
      endif

c...if HxHx or HyHy spectrum = 0. then skip
      if( z(m,25).le.0. .or. z(m,34).le.0. ) then
        write(*,*) 'Zero (or negative) horizontal magnetic field',
     &             'amplitude; m = ',m
        write(*,*) ' z(m,25), z(m,34) = ', z(m,25), z(m,34)
        write(*,*) 'spectra not added to stack'
        goto 11
      endif

      if( P_NORM ) then
        scalez = 10.**((log10(z(m,25)) + log10(z(m,34)))/2.)
      else
        scalez = 1.
      endif

      do i = 1, 49
          ytemp(i) = y(m,i)/scaley
          ztemp(i) = z(m,i)/scalez
      end do
      call acspec7( ytemp, sy )
      call acspec7( ztemp, sz )
 
c---derive weighting function for mini-stack held in y(*,*) & z(*,*)
c   weighting is function of variance given by
c
c   var(xy)  =   4        (1-mcoh(ex,hx,hy))   S(ex)
c              ------- F  ------------------  -------
c              n - 4      (1-ocoh(hx,hy)   )   S(hy)
c
c
 
c---ocoh(hx,hy)
      ochdy = focoh( 7, sy, 1, 2 )
      ochdz = focoh( 7, sz, 1, 2 )
      ochdy = min( 0.999, ochdy )
      ochdz = min( 0.999, ochdz )

      if( MT ) then 
c---ocoh(ex,hx)
        ocnhy = focoh( 7, sy, 3, 1 )
        ocnhz = focoh( 7, sz, 3, 1 )
        ocnhy = min( 0.999, ocnhy )
        ocnhz = min( 0.999, ocnhz )
c---ocoh(ex,hy)
        ocndy = focoh( 7, sy, 3, 2 )
        ocndz = focoh( 7, sz, 3, 2 )
        ocndy = min( 0.999, ocndy )
        ocndz = min( 0.999, ocndz )
c---ocoh(ey,hx)
        ocehy = focoh( 7, sy, 4, 1 )
        ocehz = focoh( 7, sz, 4, 1 )
        ocehy = min( 0.999, ocehy )
        ocehz = min( 0.999, ocehz )
c---ocoh(ey,hy)
        ocedy = focoh( 7, sy, 4, 2 )
        ocedz = focoh( 7, sz, 4, 2 )
        ocedy = min( 0.999, ocedy )
        ocedz = min( 0.999, ocedz )
c---mcoh(ex,hx,hy)
        mcnhdy = fmcoh( 7, sy, 3, 1, 2 )
        mcnhdz = fmcoh( 7, sz, 3, 1, 2 )
        mcnhdy = min( 0.999, mcnhdy )
        mcnhdz = min( 0.999, mcnhdz )
 
c---mcoh(ey,hx,hy)
        mcehdy = fmcoh( 7, sy, 4, 1, 2 )
        mcehdz = fmcoh( 7, sz, 4, 1, 2 )
        mcehdy = min( 0.999, mcehdy )
        mcehdz = min( 0.999, mcehdz )

      else if( GDS ) then
c---ocoh(hz,hx)
        oczhy = focoh( 7, sy, 5, 1 )
        oczhz = focoh( 7, sz, 5, 1 )
        oczhy = min( 0.999, oczhy )
        oczhz = min( 0.999, oczhz )
c---ocoh(hz,hy)
        oczdy = focoh( 7, sy, 5, 2 )
        oczdz = focoh( 7, sz, 5, 2 )
        oczdy = min( 0.999, oczdy )
        oczdz = min( 0.999, oczdz )
c---mcoh(hz,hx,hy)
        mczhdy = fmcoh( 7, sy, 5, 1, 2 )
        mczhdz = fmcoh( 7, sz, 5, 1, 2 )
        mczhdy = min( 0.999, mczhdy )
        mczhdz = min( 0.999, mczhdz )

      else if( HSG ) then
c---ocoh(hz,hx)
        oczhy = focoh( 7, sy, 5, 1 )
        oczhz = focoh( 7, sz, 5, 1 )
        oczhy = min( 0.999, oczhy )
        oczhz = min( 0.999, oczhz )
      end if
 
c---weights
 
      if( MT ) then
        if( kspecy.ge.3 ) then
          wny = f(kspecy) * ((1.-mcnhdy)/(1.-ochdy)) * 
     &                             (real(sy(3,3))/real(sy(2,2)))
          wey = f(kspecy) * ((1.-mcehdy)/(1.-ochdy)) * 
     &                             (real(sy(4,4))/real(sy(1,1)))
          pcndhy = ( mcnhdy - ocnhy ) / ( 1. - ocnhy )
          pcehdy = ( mcehdy - ocedy ) / ( 1. - ocedy )
          if( pcndhy.lt.0. ) pcndhy = 0.
          if( pcehdy.lt.0. ) pcehdy = 0.
          wnz = f(kspecz) * ((1.-mcnhdz)/(1.-ochdz)) * 
     &                             (real(sz(3,3))/real(sz(2,2)))
          wez = f(kspecz) * ((1.-mcehdz)/(1.-ochdz)) * 
     &                             (real(sz(4,4))/real(sz(1,1)))
          pcndhz = ( mcnhdz - ocnhz ) / ( 1. - ocnhz )
          pcehdz = ( mcehdz - ocedz ) / ( 1. - ocedz )
          if( pcndhz.lt.0. ) pcndhz = 0.
          if( pcehdz.lt.0. ) pcehdz = 0.
          wy = 1./sqrt(wny*wey)
          wz = 1./sqrt(wnz*wez)
          cohy = sqrt(pcndhy*pcehdy)
          cohz = sqrt(pcndhz*pcehdz)

        else
          wy = 0.
          wz = 0.
          cohy = -1.
          cohz = -1.

        end if 

      else if( GDS ) then
        if( kspecy.ge.3 ) then
          why = f(kspecy) * ((1.-mczhdy)/(1.-ochdy)) * 
     &                             (real(sy(5,5))/real(sy(1,1)))
          wdy = f(kspecy) * ((1.-mczhdy)/(1.-ochdy)) * 
     &                             (real(sy(5,5))/real(sy(2,2)))
          pczhdy = ( mczhdy - oczdy ) / ( 1. - oczdy )
          pczdhy = ( mczhdy - oczhy ) / ( 1. - oczhy )
          if( pczhdy.lt.0. ) pczhdy = 0.
          if( pczdhy.lt.0. ) pczdhy = 0.
          whz = f(kspecz) * ((1.-mczhdz)/(1.-ochdz)) * 
     &                             (real(sz(5,5))/real(sz(1,1)))
          wdz = f(kspecz) * ((1.-mczhdz)/(1.-ochdz)) * 
     &                             (real(sz(5,5))/real(sz(2,2)))
          pczhdz = ( mcehdz - oczdz ) / ( 1. - oczdz )
          pczdhz = ( mcnhdz - oczhz ) / ( 1. - oczhz )
          if( pcehdz.lt.0. ) pcehdz = 0.
          if( pcndhz.lt.0. ) pcndhz = 0.
          wy = 1./sqrt(why*wdy)
          wz = 1./sqrt(whz*wdz)
          cohy = sqrt(pczhdy*pczdhy)
          cohz = sqrt(pczhdz*pczdhz)

        else
          wy = 0.
          wz = 0.
          cohy = -1.
          cohz = -1.
        endif 

      else if( HSG ) then
        if( kspecy.ge.3 ) then
          call trfn1( 7, sy, 1, 5, 5, cy )
          call trfn1( 7, sy, 1, 5, 5, cz )
          wy = f(kspecy) * (1.- oczhy) * (real(sy(5,5))/real(sy(1,1)))
          wz = f(kspecz) * (1.- oczhz) * (real(sz(5,5))/real(sz(1,1)))
          cohy = oczhy
          cohz = oczhz
          if( REJECT ) then
            if( real(cy).le.0. .and. aimag(cy).gt.0. ) then
              wy   = 0.
              cohy = -1.
            endif
            if( real(cz).le.0. .and. aimag(cz).gt.0. ) then
              wz   = 0.
              cohz = -1.
            endif
          endif
        else
          wy = 0.
          wz = 0.
          cohy = -1.
          cohz = -1.
        endif 

      endif
 
      if( iprint.ge.2 ) then
        write(*,*)' Band ', m,'   weights ==>', wy, wz
      end if

      if = 2*m
      if( cohy.ge.cohmin(if) .and. cohy.le.cohmax ) then
        if = 2*m
        accepted(ncas(if),if) = 1
        ndeg(if) = ndeg(if) + 2*kspecy
        nacpt(if) = nacpt(if) + 1
        wytot(m) = wytot(m) + wy
        do i = 1, 49
          ytot(m,i) = ytot(m,i) + wy*y(m,i)
        end do
        if( JJANAL ) then
          if( nacpt(if).gt.MAXSPC) then
            write(*,*)'nacpt(if)>',MAXSPC,' for if = ', if,
     &                ' --not added'
          else
            ntot(nacpt(if),if) = kspecy
            write(40+if) if, wy, ((sy(i,j), i=1,7), j=1,7)
          end if
        end if
      end if
 
 
      if = 2*m-1
      if( cohz.ge.cohmin(if) .and. cohz.le.cohmax ) then
        accepted(ncas(if),if) = 1
        ndeg(if) = ndeg(if) + 2*kspecz
        nacpt(if) = nacpt(if) + 1
        wztot(m) = wztot(m) + wz
        do i = 1, 49
          ztot(m,i) = ztot(m,i) + wz*z(m,i)
        end do
        if( JJANAL ) then
          if( nacpt(if).gt.MAXSPC) then
          write(*,*)'nacpt(if)>',MAXSPC,' for if = ', if,
     &              ' --not added'
          else
            ntot(nacpt(if),if) = kspecz
            write(40+if) if, wz, ((sz(i,j), i=1,7), j=1,7)
          end if
        end if
      end if
 
      goto 11
 
 
c----------------------------------------------------------------
c   all data read in, proceed with analysis
 
20    continue
      write(*,'(a)')'All data read in; proceeding with analysis'

      do m = 1, 10
        if(extra(m).gt.0 ) then
          write(*,*)'Unused spectra for m =',m, ':', extra(m)
        endif
      enddo

      if( ALL ) then
        close(ind2)
c        i = system('rm .restack7')
      end if

      do if = 1, MAXF
        write(outd1,*)' '
        if( period(if).gt.1. ) then
          prd(if) = period(if)
        else
          prd(if) = -1./period(if)
        endif
        write(outd1,*)'Period: ', prd(if), '   ncas =', ncas(if)
        write(outd1,'(a10,70i1,100(/,10x,70i1))')
     &  'Accepted :',(accepted(j,if),j=1,ncas(if))
      enddo


c------------------------------------------------------------------------
c   request desired co-ordinate system
c
c   rotang    is the desired coordinate system
c   angle     is the required rotation of the cross-spectra

      write(*,991) angle
991   format(' Cascade data co-ordinate system is ',f6.1,
     >       ' degrees from TRUE NORTH')
      if( QROTANG ) then
        if( .NOT.AZIMSET ) rotang = angle
        call rin('Desired co-ordinate system in TRUE NORTH', rotang)
      endif
      angle = -angle + rotang
      write(outd1,1011) angle 
      write(*,1011) angle 
 1011 format( 1x, 'Spectra to be rotated by angle = ', f6.1, / )
      write(outd1,1001) rotang 
      write(*,1001) rotang 
 1001 format( 1x, 'All estimates in co-ordinate system',
     >        ' of angle = ', f6.1, / )

c------------------------------------------------------------------------

      nfile = ifile
      n = 0
      do 30 if = 1, MAXF
        m = (if+1)/2

        if( nacpt(if).eq.0 ) goto 30

        n = n+1

        ndeg(n) = ndeg(if)
        nacpt(n) = nacpt(if)
        period(n) = period(if)
        if( period(n).gt.1. ) then
          prd(n) = period(n)
        else
          prd(n) = -1./period(n)
        endif

        write(*,*)'N, period, nacpt, ndeg =', 
     &                 n, prd(n), nacpt(n), ndeg(n)

        if( JJANAL .and. 
     &       ((ctype.ne.'JKN'.and.nacpt(n).gt.4).or.
     &        (ctype.eq.'JKN'.and.nacpt(n).gt.1))     ) then
          nspec = min(MAXSPC,nacpt(n))

c--Load all the specta for frequency "if" from input "40+if"
c
          rewind(40+if) 

c...NC loop start.............................................................
          do nc = 1, nspec
            read(40+if) iff, w, ((ss(i,j,nc), i=1,7), j=1,7)
            if ( iff .ne. if ) then
              write(*,*)'wrong spectra in file, if, iff = ', if, iff
            endif

c...rotate spectra for angle .ne. 0.
            if( angle.ne.0. ) then
              call rotss7( angle, ss(1,1,nc) )
            endif

c---NORMALIZATION STEP NOW DONE AFTER DATA READ IN
c...normalize spectra by horizontal magnetic field power
c            hxpower = real(ss(1,1,nc))
c            hypower = real(ss(2,2,nc))
c            pnorm = sqrt( hxpower**2. + hypower**2. )
c            do j = 1, 7
c              do i = 1, 7
c                ss(i,j,nc) = ss(i,j,nc)/pnorm
c              enddo
c            enddo

          enddo
c...NC loop end...............................................................

c---loop around jacknife improvement TWICE. 
c...First time for MT case
c...Second Time for GDS case
c
c   This leads to TWO estimates of spectra
c
c   If only GDS, then start with loop=2 and only loop once
c

          if( MT ) then
            istart = 1
            if( zpre.eq.1 ) then
              iend = 2
            else
              iend = 1
            endif
          else
            istart = 2
            iend   = 2
          endif

c...LOOP loop start...........................................................
          do loop = istart, iend

            if( iprint.ge.1 ) then
              if( loop.eq.1 ) then
                write(*,*)'...looping for MT optimization estimates'
              else
                write(*,*)'...looping for GDS optimization estimates'
              endif
            endif

c---Write arrays to deal with GDS or HSG case
            if( loop.eq.2 ) then
              do nc = 1, nspec
                ss(1,3,nc) = ss(1,5,nc)
                ss(1,4,nc) = ss(1,5,nc)

                ss(2,3,nc) = ss(2,5,nc)
                ss(2,4,nc) = ss(2,5,nc)

                ss(3,1,nc) = ss(5,1,nc)
                ss(3,2,nc) = ss(5,2,nc)
                ss(3,3,nc) = ss(5,5,nc)
                ss(3,4,nc) = ss(5,5,nc)
                ss(3,5,nc) = ss(5,5,nc)
                ss(3,6,nc) = ss(5,6,nc)
                ss(3,7,nc) = ss(5,7,nc)

                ss(4,1,nc) = ss(5,1,nc)
                ss(4,2,nc) = ss(5,2,nc)
                ss(4,3,nc) = ss(5,5,nc)
                ss(4,4,nc) = ss(5,5,nc)
                ss(4,5,nc) = ss(5,5,nc)
                ss(4,6,nc) = ss(5,6,nc)
                ss(4,7,nc) = ss(5,7,nc)

                ss(5,3,nc) = ss(5,5,nc)
                ss(5,4,nc) = ss(5,5,nc)

                ss(6,3,nc) = ss(6,5,nc)
                ss(6,4,nc) = ss(6,5,nc)

                ss(7,3,nc) = ss(7,5,nc)
                ss(7,4,nc) = ss(7,5,nc)

              enddo
            endif


            if( ctype.eq.'VAR' .or.ctype.eq.'COH' ) then
            write(*,*) 'Calling SCMAX2S for period, freq no., nspec = ', 
     &                 prd(n), if, nspec
              call scmax2s7( MT, ctype, ntype, nspec, cohmax, 0.001, 
     &               nspec-4, ss, s7(1,1,n), iflag, WGTS, weight )


            else

              if( HSG ) then
                call jknvarmin1( nspec, ss, iflag, zvar )
              else
                call jknvarmin( ntype, nspec, ss, iflag, zvar )
              endif

              if( loop.eq.1 ) then
                write(outd1,*)' '
                write(outd1,*)'Period: ', prd(n), 
     &          '   nspec =', nspec, '   ncas =', ncas(if)
                write(outd1,'(a10,70i1,100(/,10x,70i1))')
     &          'MT  Flags:',(iflag(i),i=1,nspec)
              else
                write(outd1,'(a10,70i1,100(/,10x,70i1))')
     &          'GDS Flags:', (iflag(i),i=1,nspec)
              endif

c...store kacknife errors in arrays "rz" for MT, "rtf" for GDS & HSG

              if( loop.eq.1 ) then
                do j = 1, 4
                  rz(j,n) = sqrt(zvar(j))
                end do
              else
                rtf(1,n) = sqrt(zvar(1))
                rtf(2,n) = sqrt(zvar(4))
              endif

c...zero the cross-power elements
              if( loop.eq.1 ) then
                do j = 1, 7
                  do i = 1, 7
                    s7(i,j,n) = cmplx(0.,0.)
                  enddo
                enddo
              else
                do j = 1, 7
                  do i = 1, 7
                    s7gds(i,j,n) = cmplx(0.,0.)
                  enddo
                enddo
              endif

c...derive best mean value by adding all non-rejected spectral estimates
              if( loop.eq.1 ) then
                do k = 1, nspec
                  do j = 1, 7
                    do i = 1, 7
                      s7(i,j,n) = s7(i,j,n) + iflag(k)*ss(i,j,k)
                    enddo
                  enddo
                enddo
              else
                do k = 1, nspec
                  do j = 1, 7
                    do i = 1, 7
                      s7gds(i,j,n) = s7gds(i,j,n) + iflag(k)*ss(i,j,k)
                    enddo
                  enddo
                enddo
              endif
            end if

c...derive no. of degrees of freedom
            if( loop.eq.1 ) then
              ndeg(n) = 0
              nacpt(n) = isum(iflag, nspec )
              do j = 1, nspec
                ndeg(n) = ndeg(n) + 2*iflag(j)*ntot(j,if)
              enddo
            else
              ndeggds(n) = 0
              nacptgds(n) = isum(iflag, nspec )
              do j = 1, nspec
                ndeggds(n) = ndeggds(n) + 2*iflag(j)*ntot(j,if)
              enddo
            endif

          enddo
c...LOOP loop end.............................................................

        else

          if( 2*m.ne.if ) then
            if( ztot(m,25).gt.0. .and. ztot(m,34).gt.0. ) then
              scale = amax1(alog10(ztot(m,25)), alog10(ztot(m,34)) )
            else
              write(*,*) 'Mag field <=0, m = ', m, ' Hx,Hy(ztot)=',
     &                     ztot(m,25), ztot(m,34)
              n = n-1
              goto 30
            endif 

            scale = 10.**scale
            do j = 1, 49
              ztemp(j) = ztot(m,j)/scale
            end do
            call acspec7( ztemp, s7(1,1,n))

          else
            if( ytot(m,25).gt.0. .and. ytot(m,34).gt.0. ) then
              scale = amax1(alog10(ytot(m,25)), alog10(ytot(m,34)) )
            else
              write(*,*) 'Mag field <=0, m = ', m, ' Hx,Hy =',
     &                     ytot(m,25), ytot(m,34)
              n = n-1
              goto 30
            endif 

            scale = 10.**scale
            do j = 1, 49
              ytemp(j) = ytot(m,j)/scale
            end do
            call acspec7( ytemp, s7(1,1,n))

          endif

        endif
 
   30 continue

      write(*,*)'Number of accepted periods = ', n
 
c---correct for instrument response
      call bessel7
      call high17

c------------------------------------------------------------------------
c     write GDS cross-spectra array into s7 if GDS = .TRUE.

      if( GDS ) then
        do k = 1, n
          ndeg(k) = ndeggds(k)
          do j = 1, 5
            j1 = j
            if( j1.gt.2 ) j1 = j1+2
            do i = 1, 5
              i1 = i
              if( i1.gt.2 ) i1 = i1+2
              s7(i1,j1,k) = s7gds(i1,j1,k)
            enddo
          enddo
        enddo
      endif

c------------------------------------------------------------------------
c     write out final cross-spectra

      if( RR ) then
        fout = stat//sep//stat//'.xr'
      else
        fout = stat//sep//stat//'.xl'
      end if
   36 open(unit=outd3, status='NEW', file=fout, err=37)
      goto 38
   37 call trunc( fout, lf )
      write(*,*)'Error ***** ==> file ', fout(:lf), ' already exists.'
      ORITE = .TRUE.
      if( iprint.ge.0 .and. QORITE ) then
        ans = 'Y'
        call yin( 'Do you wish to overwrite it (y/n)?', ans )
        if( ans.eq.'Y' ) then
          ORITE = .TRUE.
        else
          ORITE = .FALSE.
        endif
      else
        write(*,*) 'Being overwritten'
      end if
      if (ORITE) then
        open(unit=outd3, status='OLD', file=fout )
      else
        call tin ('Give a new filename', fout )
        go to 36
      end if
   38 continue

      write(outd3,'(a)') '# spectral file written by restack version='//
     &                   ver
      write(outd3,'(a)') '#'
      write(outd3,'(a  )') '>STATION   ='//stat
      write(outd3,'(a  )') '>STATION_RR='//windowrr(1:6)
      write(outd3,'(a12,f12.1)') '>AZIMUTH   =', rotang
      write(outd3,'(a12,f12.4)') '>LATITUDE  =', xlat
      write(outd3,'(a12,f12.4)') '>LONGITUDE =', xlon
      write(outd3,'(a12,f12.1)') '>ELEVATION =', elev1

      do k = 1, n

        write(outd3,*) prd(k), ndeg(k)
        do j = 1, 7
          do i = j, 7
            write(outd3,*) k, i, j, s7(i,j,k)
          end do
        end do
      end do

      close( unit=outd3 )

      if( zpre.eq.0 ) goto 50

      if( RR ) then
        fout = stat//sep//stat//'.xrg'
      else
        fout = stat//sep//stat//'.xlg'
      end if
   46 open(unit=outd3, status='NEW', file=fout, err=47)
      goto 48
   47 call trunc( fout, lf )
      write(*,*)'Error ***** ==> file ', fout(:lf), ' already exists.'
      ORITE = .TRUE.
      if( iprint.ge.0 .and. QORITE ) then
        ans = 'Y'
        call yin( 'Do you wish to overwrite it (y/n)?', ans )
        if( ans.eq.'Y' ) then
          ORITE = .TRUE.
        else
          ORITE = .FALSE.
        endif
      else
        write(*,*) 'Being overwritten'
      end if
      if (ORITE) then
        open(unit=outd3, status='OLD', file=fout )
      else
        call tin ('Give a new filename', fout )
        go to 46
      end if
   48 continue


      write(outd3,'(a)') '# spectral file written by restack version='//
     &                   ver
      write(outd3,'(a)') '#'
      write(outd3,'(a  )') '>STATION   :'//stat
      write(outd3,'(a  )') '>STATION_RR:'//windowrr(1:6)
      write(outd3,'(a12,f12.1)') '>AZIMUTH   =', rotang
      write(outd3,'(a12,f12.4)') '>LATITUDE  =', xlat
      write(outd3,'(a12,f12.4)') '>LONGITUDE =', xlon
      write(outd3,'(a12,f12.1)') '>ELEVATION =', elev1

      do k = 1, n
        write(outd3,*) prd(k), ndeggds(k)
        do j = 1, 5
          j1 = j
          if( j1.gt.2 ) j1 = j1+2
          j2 = j
          if( j2.gt.3 ) j2 = j2+2
          do i = j, 5
            i1 = i
            if( i1.gt.2 ) i1 = i1+2
            i2 = i
            if( i2.gt.3 ) i2 = i2+2
            write(outd3,*) k, i1, j1, s7gds(i2,j2,k)
          end do
        end do
      end do

      close( unit=outd3 )

c------------------------------------------------------------------------

50    if ( JJANAL ) then
        do iunit = 41, 40+MAXF
          close( unit=iunit, status='DELETE' )
        end do
      end if
 
c---write out ndf
      write(outd1,1004)
 1004 format(//,10x,'PERIOD',10x,'NACPT',10x,'NDEG',7x,'NDEG-GDS'/)
      do 27 i = 1, n
        write(outd1,1005) prd(i), nacpt(i), ndeg(i), ndeggds(i)
 1005 format(6x, g14.5, 6x, i5, 7x, i7, 7x, i7 )
   27 continue
 

c-----------------------------------------------------------------------
 
      write(outd1,*)'Ordinary coherences ==>'
      write(outd1,*)'Period      g12   g13   g14   g23   g24   g34',
     &          '   g16   g17   g26   g27'
      do if = 1, n
        g12 = focoh( 7, s7(1,1,if), 1, 2 )
        g13 = focoh( 7, s7(1,1,if), 1, 3 )
        g14 = focoh( 7, s7(1,1,if), 1, 4 )
        g23 = focoh( 7, s7(1,1,if), 2, 3 )
        g24 = focoh( 7, s7(1,1,if), 2, 4 )
        g34 = focoh( 7, s7(1,1,if), 3, 4 )
        g16 = focoh( 7, s7(1,1,if), 1, 6 )
        g17 = focoh( 7, s7(1,1,if), 1, 7 )
        g26 = focoh( 7, s7(1,1,if), 2, 6 )
        g27 = focoh( 7, s7(1,1,if), 2, 7 )
        write(outd1,2002) prd(if), g12, g13, g14, g23, g24, g34, g16,
     &                    g17, g26, g27
      end do
      write(outd1,*)' ' 
      write(outd1,*)'Multiple coherences ==>'
      write(outd1,*)'Period      g312  g412  g512  g612  g712'
      do if = 1, n
        g312 = fmcoh( 7, s7(1,1,if), 3,  1, 2 )
        g412 = fmcoh( 7, s7(1,1,if), 4,  1, 2 )
        g512 = fmcoh( 7, s7gds(1,1,if), 5,  1, 2 )
        g612 = fmcoh( 7, s7(1,1,if), 6,  1, 2 )
        g712 = fmcoh( 7, s7(1,1,if), 7,  1, 2 )
        write(outd1,2002) prd(if), g312, g412, g512, g612, g712
      end do
      write(outd1,*)' ' 

      call plrzn7



c...derive GDS transfer functions...............................................
c   NOTE that have to read from <s7gds> array for best GDS estimates

      if( zpre.eq.0 ) goto 60
      write(outd1,*)'GDS transfer functions ==>'
      do i = 1, n
        call trfn( 7, s7gds(1,1,i), 1, 2, 6, 7, 5, t1, t2 )
        tf(1,i) = t1
        tf(2,i) = t2
c---calculate errors only for ctype.ne.JKN or ndeggds.lt.4
        if( ctype.ne.'JKN' .or. nacptgds(i).le.4 ) then
          mcoh = fmcoh( 7, s7gds(1,1,i), 5,  1, 2 )
          ocoh = focoh( 7, s7gds(1,1,i), 1, 2 )
          call rccf2( ndeggds(i), mcoh, ocoh, real(s7gds(5,5,i)),
     &                real(s7gds(1,1,i)), r1 )
          call rccf2( ndeggds(i), mcoh, ocoh, real(s7gds(5,5,i)),
     &                real(s7gds(2,2,i)), r2 )
          rtf(1,i) = r1
          rtf(2,i) = r2
        endif
        write(outd1,2001) prd(i), t1, t2, rtf(1,i), rtf(2,i)
      end do
      write(outd1,*)' '


c...derive MT transfer functions................................................
60    if( MT ) then 

        write(outd1,*)'MT transfer function Ex (Hx,Hy) & error ==>'
        do i = 1, n
        call trfn( 7, s7(1,1,i), 1, 2, 6, 7, 3, t1, t2 )
        ztf(1,i) = t1
        ztf(2,i) = t2

c---calculate errors only for ctype.ne.JKN or ndeg.lt.4
          if( ctype.ne.'JKN' .or. nacpt(i).le.4 ) then
            mcoh = fmcoh( 7, s7(1,1,i), 3,  1, 2 )
            ocoh = focoh( 7, s7(1,1,i), 1, 2 )
            call rccf2( ndeg(i), mcoh, ocoh, real(s7(3,3,i)),
     &                   real(s7(1,1,i)), r1 )
            call rccf2( ndeg(i), mcoh, ocoh, real(s7(3,3,i)),
     &                  real(s7(2,2,i)), r2 )
            rz(1,i) = r1
            rz(2,i) = r2
          endif

          write(outd1,2001) prd(i), t1, t2, rz(1,i),rz(2,i)
        enddo

        write(outd1,*)' '
        write(outd1,*)'MT transfer function Ey (Hx,Hy) & error ==>'
        do i = 1, n
          call trfn( 7, s7(1,1,i), 1, 2, 6, 7, 4, t1, t2 )
          ztf(3,i) = t1
          ztf(4,i) = t2

c---calculate errors only for ctype.ne.JKN or ndeg.lt.4
          if( ctype.ne.'JKN'  .or. nacpt(i).le.4 ) then
            mcoh = fmcoh( 7, s7(1,1,i), 4,  1, 2 )
            ocoh = focoh( 7, s7(1,1,i), 1, 2 )
            call rccf2( ndeg(i), mcoh, ocoh, real(s7(4,4,i)),
     &                  real(s7(1,1,i)), r1 )
            call rccf2( ndeg(i), mcoh, ocoh, real(s7(4,4,i)),
     &                  real(s7(2,2,i)), r2 )
            rz(3,i) = r1
            rz(4,i) = r2
          endif

          write(outd1,2001) prd(i), t1, t2, rz(3,i),rz(4,i)
        end do

      end if

c-----------------------------------------------------------------------
c   write data out to .dat file

      if( RR ) then
        fout = stat//sep//stat//'r.dat'
      else
        fout = stat//sep//stat//'l.dat'
      end if
   33 open(unit=outd2, status='NEW', file=fout, err=34)
      goto 35
   34 call trunc( fout, lf )
      write(*,*)'Error ***** ==> file ', fout(:lf), ' already exists.'
      ORITE = .TRUE.
      if( iprint.ge.0 .and. QORITE ) then
        ans = 'Y'
        call yin( 'Do you wish to overwrite it (y/n)?', ans )
        if( ans.eq.'Y' ) then
          ORITE = .TRUE.
        else
          ORITE = .FALSE.
        endif
      else
        write(*,*) 'Being overwritten'
      end if
      if (ORITE) then
        open(unit=outd2, status='OLD', file=fout )
      else
        call tin ('Give a new filename', fout )
        go to 33
      end if
   35 continue

 
c-------------------------------------------------------------------------
c write out transfer functions & impedances
c leave as field units?????
c--------

      convert = 4. * 3.141592 * 1.e-4   ! to convert to SI (ohm)

      write(outd2,'(a)')
     $   '# tsrestack: version: '//ver//'    precision: '//progprec
      write(outd2,'(2a)')'# date: ', date_string
      write(outd2,'(a)')'#'
      write(outd2,'(a,g12.7)')'# Coherence minima:'
      write(outd2,'(a,5x,10f5.2)') '# ', (cohmin(i),i= 1,10)
      write(outd2,'(a,5x,10f5.2)') '# ', (cohmin(i),i=11,20)
      write(outd2,'(a,g12.7)')'# Coherence maximum:', cohmax
      write(outd2,'(a,1x,a3)')'# Maxtype:', ctype
      write(outd2,'(a,i5)')'# Ntype:',   ntype
      write(outd2,'(a,a)')'# Weight: ',   wans
      write(outd2,'(a)')'#'
        write(outd2,'(a)')'# Cascade files used:'
      do ifile = 1, nfile
        write(outd2,'(2a)')'# ', casfile(ifile)
      enddo
      write(outd2,'(a)')'#'
c...rewind cascade file and write comment info into file
      rewind(4)
      read(4) hash
      do while( hash.eq.'#' )
        backspace(4)
        read(4) line
        write(outd2,'(a)') line(1:lnblnk(line))
        read(4) hash
      enddo
      write(outd2,'(a)')'#'

      write(outd2,'(a  )')       '>STATION   :'//stat
      write(outd2,'(a  )')       '>STATION_RR:'//windowrr(1:6)
      write(outd2,'(a12,f12.1)') '>AZIMUTH   =', rotang
      write(outd2,'(a12,f12.4)') '>LATITUDE  =', xlat
      write(outd2,'(a12,f12.4)') '>LONGITUDE =', xlon
      write(outd2,'(a12,f12.1)') '>ELEVATION =', elev1

      write(outd2,'(a6,1x,f6.1)') stat, rotang

      if( MT ) then

        do j = 1, 4
          ke = j/3 + 3
          km = mod(j,2)
          if(km.eq.0)km=2
          km2 = abs(km-3)
          if( SI ) then
            write(outd2,'(3a)')'Z',comp(j),' SI units (ohms)'
          else
            write(outd2,'(3a)')'Z',comp(j),' field units (nT/mV/km)'
          endif
          write(outd2,*) n
          do i = 1, n
            pcoh = fpcoh( 7, s7(1,1,i), ke, km, km2 )
            if( SI ) then
              write(outd2,2005) prd(i), convert*real(ztf(j,i)),
     &         convert*aimag(ztf(j,i)), convert*rz(j,i), pcoh
            else
              write(outd2,2005) prd(i), real(ztf(j,i)),
     &         aimag(ztf(j,i)), rz(j,i), pcoh
            endif
          end do
        end do

        do j = 1, 4
          ke = j/3 + 3
          km = mod(j,2)
          if(km.eq.0)km=2
          km2 = abs(km-3)
          write(outd2,'(3a)')'R',comp(j),' SI units (ohms)'
          write(outd2,*) n
          do i = 1, n
            per  = period(i)
            zz   = convert*ztf(j,i)
            rho  = convz2r( zz, per )

            pha  = convz2p( zz, per )

            zerr = convert*rz(j,i)
            zr = zerr*cosd(pha)
            zi = zerr*sind(pha)
            perr = atand(zerr/cabs(zz))
            pcoh = fpcoh( 7, s7(1,1,i), ke, km, km2 )
            zmax = zz + cmplx(zr,zi)
            zmin = zz - cmplx(zr,zi)
            rmax = convz2r( zmax, per )
            rmin = convz2r( zmin, per )
            pmax = pha + perr
            pmin = pha - perr
            pcoh = fpcoh( 7, s7(1,1,i), ke, km, km2 )
            if( pcoh.gt.0. .and. pcoh.lt.1.0 ) then
              write(outd2,2003) prd(i), rho, pha, 
     &                        amax1(rmax, rmin), 
     &                        amin1(rmax, rmin), 
     &                        pmax, pmin, pcoh, pcoh
            else
              write(outd2,2013) prd(i), rho, pha,
     &                        amax1(rmax, rmin),
     &                        amin1(rmax, rmin),
     &                        pmax, pmin, pcoh, pcoh
            endif
          enddo
        enddo

      endif

      if( zpre.eq.0 ) goto 9999
      write(outd2,'(3a)') 'TZX'
      write(outd2,*) n
      do i = 1, n
        pcoh = fpcoh( 7, s7gds(1,1,i), 5, 1, 2 )
        write(outd2,2004) prd(i), real(tf(1,i)), aimag(tf(1,i)),
     &                 rtf(1,i), pcoh
      end do
      write(outd2,'(3a)') 'TZY'
      write(outd2,*) n
      do i = 1, n
        pcoh = fpcoh( 7, s7gds(1,1,i), 5, 2, 1 )
        write(outd2,2004) prd(i), real(tf(2,i)), aimag(tf(2,i)),
     &                 rtf(2,i), pcoh
      end do

 9999 stop

2001  format(1x,f10.1,2x,2f10.4,1x,2f10.4,1x,2f10.4)
2002  format(1x,f10.1,10f6.2)
2003  format( g12.5,1x,g11.5,1x,f6.1,1x,2(g11.5,1x),
     &                    2(f6.1,1x),f4.2,1x,f4.2 )
2004  format( g12.5, 2f10.3, f8.3, f8.3 )
2005  format( g12.5, 2g16.4, g14.4, f8.3 )
2013  format( g12.5,1x,g11.5,1x,f6.1,1x,2(g11.5,1x),
     &                    2(f6.1,1x),f4.1,1x,f4.1 )

      end
